# $Source: /cvsroot/civ4geneticai/Assets/Python/CUSTOM/ActionButton.py,v $: $Name:  $
#	ActionButton for Sid Meier's Civilization 4 Custom Mods.
#	$CopyRight$
#	by SimCutie < simcutie (AT) hanmail.net >
#	-- inspired by TALCHAS's Action Button Utility mod
#	action buttons that shows in lower pane of main interface screen
#		when a Civ 4 unit is selected.
#	This module works with or without SimCutie's SDK GameCore DLL. 
#	This file should reside in  "... /CustomAssets/python/custom" folder.
#	Version: $Revision: 1.1 $ $Date: 2006/09/12 12:43:11 $
#
__author__ = "$Author: the_great_apple $"
__revision__ = "$Id: ActionButton.py,v 1.1 2006/09/12 12:43:11 the_great_apple Exp $"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2006/09/12 12:43:11 $"

"""
HOW TO USE ACTION BUTTON:

First, you should install SimCutie's EventHandler framework.
See "SimCutie-Civ4-Framwork.txt" for details.
Install "ActionButton.py" to "(Custom)Assets/Python/Custom" folder
and install "CvSimcutieMainInterface.py" to your "(Custom)Assets/Python/Screens".
Or copy updateButtons() function of it to your "CvMainInterface.py"
This can be used with or without CIv4CCP SDK GameCore DLL.
If used without the DLL, on-screen tooltip help will not displayed.

You should have two event handlers to use action button.
Use EventHandlerRegister() to set event handlers.

1. CvCustomEvent.EVENT_BUTTONSHOW event handler
	This event is triggered when main interface screen is updating button area.
	Event argsList is tuple of ( bUnitSelection, pSelectedEntity )
	bUnitSelection is boolean that indicates unit selection or city selection.
	pSelectedEntity is selected object. i.e. pSelected_PyUnit or pSelected_PyCity.

	If your buttons are not yet created and inited, you should do that first.
	Button creation and initialization should be done once and only once.
	Next, you set or change button art and help for display.
	You can optionally set iData and strData to pass your data to your 
	EVENT_BUTTONACTION event handler. Use setHandlerData to set custom data.
	then call button.showButton() for each button you display to actually 
	display the buttons on the screen.

2. CvCustomEvent.EVENT_BUTTONACTION event handler
	This event is triggered when player clicks on the button.
	Event argsList is tuple of ( actId, bUnitSelection, pSelectedEntity, bRightClick, iData, strData ).
	actId is button action ID of clicked button.
	bUnitSelection is boolean indicating current selection is an unit or a city.
	pSelectedEntity is selected object. i.e. pSelected_PyUnit or pSelected_PyCity.
	bRightClick is boolean indicating right mouse click or left mouse click.
	iData, strData is data set by the EVENT_BUTTONSHOW event handler.

	You should check that actId is matching with button id you wish to handle.
	If it does not match with id of your button, return immedately with false return value.

##--------------------------------------------------------------------------
# $Source: /cvsroot/civ4geneticai/Assets/Python/CUSTOM/ActionButton.py,v $: $Name:  $
##
##	DEMO Example for SimCutie's ActionButton facility.
##	See "ActionButton.py" for detailed information.
##
#__author__ = "$Author: the_great_apple $"
#__revision__ = "$Id: ActionButton.py,v 1.1 2006/09/12 12:43:11 the_great_apple Exp $"
#__version__ = "$Revision: 1.1 $"
#__date__ = "$Date: 2006/09/12 12:43:11 $"
#
#from CvPythonExtensions import *
#import CvUtil
#import time
#
#gc = CvUtil.gc
#
#import ActionButton
#
#Act_Test1 = None
#Act_Test2 = None
#
#def EventHandlerRegister(evt_mgr, unused = None):
#	"Register two action button event handler"
#	import CvCustomEvent
#
#	CvUtil.pyPrint( __name__ + ": Test Action Button Register OK" )
#	return { CvCustomEvent.EVENT_BUTTONSHOW : onShowButtons, 
#		CvCustomEvent.EVENT_BUTTONACTION : onActionButtons }
#
#def helpFunc_Test2(args):
#	"help generator for test2, updated on each mouse over"
#	actID, data2, option = args[:3]
#	return " Actid = %d, iData = %d\ntime= %s" % ( actID, data2, time.ctime())
#	
#def onShowButtons(argsList):
#	" called when Bottum Button List are updated"
#	" argsList = ( selType, pSelectedUnit or pSelectedCity)"
#	global Act_Test1, Act_Test2
#	bUnitSelection, pSelectedEntity = argsList[:2]
#
#	if not Act_Test1 or not Act_Test2 :
#		# init action buttons
#		Act_Test1 = ActionButton.createButton("PYACTION_TEST1")
#		btn = gc.getCivilizationInfo( gc.getInfoTypeForString("CIVILIZATION_AMERICA" )).getButton()
#		Act_Test1.setButton(btn)
#		Act_Test1.setHelp( "TXT_KEY_DEMO_ACTION_BUTTON_TEST1_HELP" )
#
#		Act_Test2 = ActionButton.createButton("PYACTION_TEST2" )
#		Act_Test2.setButton("CIVIC_HEREDITARY_RULE" )
#		Act_Test2.setHelp(helpFunc_Test2)
#
#	if bUnitSelection :
#		Act_Test1.showButton()
#	else:
#		Act_Test2.setHandlerData( strData=("Test 2 Button SELECTED\n at " + time.ctime())  )
#		Act_Test2.showButton( iData=1234)
#
#
#def onActionButtons( argsList):
#	" called when an action button is pressed. You must check actId (argsList[0])."
#	global Act_Test1, Act_Test2
#	( actId, bUnitSelection, pSelectedEntity, bRightClick, iData, strData ) = argsList[:6]
#
#	if actId == Act_Test1.getID() :
#		CyInterface().addImmediateMessage( "BUTTON TEST1", "")
#		#PyCivSim.MSGDEBUG( "BUTTON TEST1")
#	elif actId == Act_Test2.getID() :
#		CyInterface().addImmediateMessage( strData, "")
#	else :
#		return False
#	return True
###--------------------------------------------------------------------------
"""
############## ACTION BUTTON FACILITY ####################################

from CvPythonExtensions import WidgetTypes, CyArtFileMgr
from PyCivSim import findButton, triggerEvent, hasExtGC, setPythonEntryPoint
import CvUtil

bUseDLL = False
if hasExtGC() :
	try :
		from CvPythonExtensions import CyExtGC, CvPyActionInfo
		bUseDLL = True
	except ImportError :
		pass

from CvUtil import gc
from  CvCustomEvent import EVENT_BUTTONSHOW, EVENT_BUTTONACTION

g_Screen = None
g_bUnitSelection = False
g_pSelectedEntity = None
g_ButtonCount = 0

ST_NOTSET	= -1
ST_HIDE		= 0
ST_DISABLE	= 1
ST_ENABLE	= 2
ST_FLASH	= 3

class ActionButtonInfo:
	InfoDict = {}
	NumEntries = 0

	ACTID_BASE = 1000
	name_prefix = "PYACTION_"

	def __init__(self, name=""):
		"create new action button class"
		# Use name like 'PYACTION_CIVILOPEDIA', 'PYACTION_UNIT_STAT'"
		if bUseDLL :
			if not name :
				name = self.name_prefix + str(self.NumEntries)
			elif not name.upper().startswith( self.name_prefix ):
				name =  self.name_prefix + name
			
			try:
				info = CvPyActionInfo().create(name.upper())
			except Exception:
				assert False, "CvPyActionInfo().create() failed"
				return

			assert info  and info.getID() >= 0, "Info Create failed"
			self.actionID = info.getID()
		else :
			self.actionID = ( self.ACTID_BASE + self.NumEntries)

			self.ButtonArt = ""
			self.InfoDict[self.actionID] = self
			self.handlerData = ""

		self.iData = None
		self.state = ST_ENABLE
		self.__class__.NumEntries += 1
	
	def __del__(self):
		if bUseDLL :
			self.getDLLInfo().destroy()

	def getID(self):
		if bUseDLL :
			return self.getDLLInfo().getID()
		else :
			return self.actionID

	def getDLLInfo(self):
		return CvPyActionInfo.getInfo(self.actionID)

	def setButton(self, buttonArt):
		""" button can be button art path or 
		info type name with button (like "UNIT_WARRIR", "CIVILIZATION_ROMAN")"""

		assert buttonArt
		if bUseDLL :
			self.getDLLInfo().setButton(buttonArt)
		else :
			self.ButtonArt = buttonArt
	
	def setHelp(self, helpobj):
		"""
		Help work only in DLL version.
		help can be normal text string or text key ("TXT_KEY_*")
		or Python function object that returns help text string. 
		( not functon name string, but function object)
		Function Args tuple of ( actionID, iData, bIsUnitSelection ) """

		if bUseDLL :
			if callable(helpobj):
				funcName =  "handleActionButtonHelp_" + self.getDLLInfo().getType()
				setPythonEntryPoint( funcName, helpobj)
				self.getDLLInfo().setHelpHandler(funcName)
			else :
				self.getDLLInfo().setHelpKey(helpobj)

	def setState(self, newState):
		" state is one of ST_HIDE, ST_DISABLE, ST_ENABLE, ST_FLASH, ST_NOTSET"
		if newState != ST_NOTSET :
			self.state = newState

	def setHandlerData(self, iData=0, strData=""):
		"string data passed to button action handler"
		if bUseDLL :
			self.getDLLInfo().setActionHandler(strData)
		else :
			self.handlerData = strData
		
		if iData :
			self.iData = iData

	def showButton(self, state = ST_NOTSET, iData = 0, button="" ):
		global g_Screen, g_ButtonCount

		if button :
			btnArt = button
		elif bUseDLL :
			#btnArt = CvPyActionInfo.getInfo(self.actionID).getButton()
			btnArt = self.getDLLInfo().getButton()
		else :
			btnArt = self.ButtonArt

		assert btnArt, "need button Art name"
		btnPath = findButton( btnArt )
		if not btnPath :
			CvUtil.pyPrint( "Invalid button art = " + str(buttonArt) )
			return

		if state != ST_NOTSET :
			self.setState(state)
			
		if bUseDLL :
			wt = WidgetTypes.WIDGET_ACTION_BUTTON
			if self.iData :
				iData = self.iData
		else :
			wt = WidgetTypes.WIDGET_ACTION
			self.iData = iData

		iRow = 0
		try :
			g_Screen.appendMultiListButton( "BottomButtonContainer", 
				btnPath, iRow, wt, self.getID(), iData, g_bUnitSelection )
		except Exception, err:
			CvUtil.pyPrint( __name__ + ": appendMultiListButton() failed: " + str(err))
			return
	
		if  self.state == ST_DISABLE :
			g_Screen.disableMultiListButton( "BottomButtonContainer",  
				iRow, g_ButtonCount, btnPath )
		elif self.state == ST_FLASH :
			g_Screen.enableMultiListPulse( "BottomButtonContainer", 
				True, iRow, g_ButtonCount )
		g_ButtonCount += 1
		

def createButton( name, button="", helpkey=None ):
	""" Register New action button  on init"
	" Use name like 'PYACTION_CIVILOPEDIA', 'PYACTION_UNIT_STAT'"""
	info = ActionButtonInfo(name)

	if not info :
		CvUtil.pyPrint( "Button creation failed")
	assert info, "Button creation failed"
	if button :
		info.setButton(button)
	if helpkey is not None:
		info.setHelp(helpkey)
	return info
		
def DisplayCustomButtons( screen, iCount, bUnitSelection, pSelected ):
	" used by CvMainInterface.py to init Action Buttons List"
	# Don't call me before main interface screen is fully inited.
	#	Else, CvPyActionInfo().create() will fail.
	
	global g_Screen, g_ButtonCount, g_bUnitSelection, g_pSelectedEntity

	g_Screen = screen
	g_ButtonCount = iCount
	g_bUnitSelection = bUnitSelection
	g_pSelectedEntity = pSelected
	
	triggerEvent( EVENT_BUTTONSHOW, bUnitSelection, pSelected )

	return g_ButtonCount

def handleActionButton(argsList):
	"called from CvGameInterface.handleActionButton() (DLL version)"
	# argsList = ( actId, iData, bUnitSelection, bRightClick, handlerStrData)"
	allArgs = ( argsList[0], argsList[2], g_pSelectedEntity, 
		argsList[3], argsList[1], argsList[4] )

	pInfo = CvPyActionInfo.getInfo(argsList[0])		# actID
	# Event arg = ( actId, bUnitSelection, pSelectedEntity, bRightClick, iData, strData )
	ret = triggerEvent( EVENT_BUTTONACTION, *allArgs )
	if ret :	# update button display again...
		CyInterface().setDirty(InterfaceDirtyBits.SelectionButtons_DIRTY_BIT, True)
		return ret
	return False

def handleActionButtonPython(argsList):
	"called from CvGameInterface.cannotHandleAction() (Python Version)"
	# argsList == ( pPlot, iAction, bTestVisible )
	global g_pSelectedEntity, g_bUnitSelection
	if bUseDLL :
		return False

	actId = argsList[1]
	if ( actId >= ActionButtonInfo.ACTID_BASE and actId in ActionButtonInfo.InfoDict ):
		( pPlot, iAction, bTestVisible ) = argsList
		info = ActionButtonInfo.InfoDict.get(actId)
		# Event arg = ( actId, bUnitSelection, pSelectedEntity, bRightClick, iData, strData ) 
		ret = triggerEvent( EVENT_BUTTONACTION, actId, g_bUnitSelection, 
			g_pSelectedEntity, False, info.iData, info.handlerData )
		if ret :	# update button display again...
			CyInterface().setDirty(InterfaceDirtyBits.SelectionButtons_DIRTY_BIT, True)
			return ret
	return False

