# $Source: /cvsroot/civ4geneticai/Assets/Python/CUSTOM/PyBaseLib.py,v $: $Name:  $
#	Python Base Libray for Sid Meier's Civilization 4 Custom Mods.
#	$CopyRight$
#	by SimCutie < simcutie (AT) hanmail.net >
#	This is general python programing libray that does not depend on CIV4 game.
#	See "CvLibTest.py" to test this module and usage example.
#	This file should reside in  "... /CustomAssets/python/custom" folder.
#	Version: $Revision: 1.1 $ $Date: 2006/09/12 12:43:11 $
#
__author__ = "$Author: the_great_apple $"
__revision__ = "$Id: PyBaseLib.py,v 1.1 2006/09/12 12:43:11 the_great_apple Exp $"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2006/09/12 12:43:11 $"

############ PHYTHON DATA TYPE  ##########################################
#**********************************************
# Python Types utility function
import types

def isNone(aItem):
	return aItem is None

def isInt(aItem):
	" int but not bool"
	return isinstance(aItem, int) and not isinstance(aItem, bool)

def isLong(aItem):
	return isinstance(aItem, long)

def isFloat(aItem):
	return isinstance(aItem, float)

def isNumber(aItem):
	" int, long, float"
	return ((isinstance(aItem, int) and not isinstance(aItem, bool)) 
		or isinstance(aItem, (long, float)))

def isComplex(aItem):
	return isinstance(aItem, complex)

def isBool(aItem):
	return isinstance(aItem, bool)

def isStr(aItem):
	"Ascii/SBCS/MBCS string, but not unicode" 
	return isinstance(aItem, str)

def isUnicode(aItem):
	return isinstance(aItem, unicode)

def isUStr(aItem):
	" Unicode String"
	return isinstance(aItem, unicode)

def isString(aItem):
	" Ascii/SBCS/MBCS string or Unicode string"
	return isinstance(aItem, basestring )

def isChar(aItem):
	"single charcter including unicode char"
	return ( isinstance(aItem, basestring) and len(aItem) == 1 )

def isList(aItem):
	return isinstance(aItem, list )

def isTuple(aItem):
	return isinstance(aItem, tuple )

def isDict(aItem):
	return isinstance(aItem, dict )

def isSetType(aItem):
	return type(aItem) == type(set())

def isClassType(aItem):
	return type(aItem) == types.ClassType

def isInstanceType(aItem):
	return type(aItem) == types.InstanceType

def isException(aItem):
	return type(aItem) == types.ClassType and issubclass (aItem, Exception)

def isFunction(aItem):
	" Function, Method or Builtin function  == callable()"
	return type(aItem) in (types.FunctionType, types.MethodType, types.BuiltinFunctionType)

def isFile(aItem):
	" File object returned by file()"
	return isinstance(aItem, file )

def isCollection(aItem):
	"""  List, Tuple, Dict type only, But Not String or Set 
	(applicable opr: len(), in,  [n] or [key]) """
	return isinstance(aItem, (list, tuple, dict))

def isContainer(aItem):
	""" Container that can contain other element or containers
	List, Tuple, Dict type and Set (Applicables:  len(), in ) """
	return isinstance(aItem, (list, tuple, dict, type(set())))

def isElement(aItem):
	" Oppisite of isContainer(). Individual entities like number, string, object, functions ... "
	return not isinstance(aItem, (list, tuple, dict, type(set())))

def isAtom(aItem):
	" Python basic atomic entity with like None, int, long, float, bool, string"
	return (isinstance(aItem, (int, long, float, basestring)) or aItem is None )

def isSeqStr(aItem):
	" Sequence or string Not dict or Set. (Applicables: len(), in, [n])"
	return isinstance(aItem, (list, tuple, basestring))

def isSeq(aItem):
	" List or tuple only: Sequences except strings (Applicables: len(), in, [n])"
	return isinstance(aItem, (list, tuple))

def isEnumerable(aItem):
	" all Enumerable : Sequence + Str+ Dict and Set (Applicables: len(), in )"
	return isinstance(aItem, (list, tuple, dict, basestring, type(set())))

def isType(aItem, aType):
	" is aItem and aType has same type"
	return type(aItem) == type(aType)

def typeName(aItem):
	""" returns Type name string like 'int', 'float', 'list', 
	'instance:module_name.class_name' """
	aType = type(aItem)
	if aType is types.InstanceType:
		(className, aType, rest) = str(aItem).replace("<","").split(" ", 2)
		aStr = "instance:%s" % className
		return aStr 
	elif aType is types.ClassType:
		return "class:%s" % aItem
	else:
		aStr = str(aType).replace( r"<type '", "").replace( r"'>", "" )
	return aStr

def className(arg):
	""" returns string of class name for class or instance type
		TODO: extend to calss method """
	if isInstanceType(arg):
		return arg.__class__.__name__
	elif isClassType(arg):
		return arg.__name__
	else:
		raise TypeError

def moduleName(arg):
	""" returns string of module name for class or instance type
	TODO:  extend to function and calss method """
	if isInstanceType(arg):
		return arg.__class__.__module__
	elif isClassType(arg):
		return arg.__module__
	else:
		raise TypeError

QNOVALUE = NotImplemented		# just unique const... Used as default value of arguments

############ NUMBER CONVERSION ###########################################
#===============================================================
# Number Test & Conversion Methods ***

#re_float =	re.compile(r"[+-]?(?:(?:(?:\d+\.\d*)|(?:\.\d+))(?:[eE][+-]?\d+)?)|(?:\d+[eE][+-]?\d+)")
#re_hex =	re.compile(r"0[xX][\dA-Fa-f]+")
#re_int =	re.compile(r"[+-]?\d+")
#
#def qnumber(arg, default = 0):
#	""" Convert numeric string to int or float. 
#	leading white space/trailing non-numeric gabage ignored.
#	If it is not number, return 0 or specified default arg value.
#	Ex: "-1234", "  3456 km ",  "0xF7c0", "  -23.4% ", ".334", "23.4e-3" """
#
#	inp = str(arg).strip()
#	if not inp or arg == None:
#		return default
#	mo = re_float.match(inp)
#	if mo :
#		return float(mo.group(0))
#	mo = re_hex.match(inp)
#	if mo :
#		return int(mo.group(0), 16)
##	inp = inp.replace( ",", "").replace( "$", "")
#	mo = re_int.match(inp)
#	if mo :
#		return int(mo.group(0))
#	return default

def qnumber( arg, default = None):
	""" Convert int/hexa/float numeric string to int or float.
	If it is not number, return given default arg value or None if not given. """
	if isString(arg):
		try : 
			return int(arg)
		except ValueError:
			try :
				return float(arg)
			except ValueError:
				try :
					return int(arg, 16 )
				except ValueError:
					return default
	elif isNumber(arg):
		return arg
	else :
		return default

def isnumeric(arg):
	" is the arg is number or number string (decimal, float, hexa)"
	return qnumber( arg, None) is not None

def qint(arg, default = 0):
	""" Convert a String to Integer.
	On conversion error, or <None> input, return given default value or 0 if not given. """
	
	if isString(arg):
		try:
			return int(arg)
		except ValueError:
			try :
				return float(arg)
			except ValueError:
				return default
	elif isInt(arg) or isLong(arg):
		return arg
	elif isFloat(arg) or isBool(arg):
		return int(arg)
	elif arg == None:
		return default
	else:
		raise TypeError

def qxor(a, b):
	" boolean exclusive or "
	return bool(a) != bool(b)

############ HANDY SEQUENCE UTILITY ###########################################

def qcond( cond, on_true, on_false ):
	"""  if cond is true return on_true arg value, else return on_false value 
		Similar to 'cond and on_true or on_false' """
	if cond: 
		return on_true 
	else: 
		return on_false

def qdefault( value, on_false = None ):
	""" if value is true return the value. If value is evaluated as false, 
		Retun given on_flase as default value or None if not specified.
		Same as value or on_false """
	if value :
		return value
	else: 
		return on_false

def qapply( cond, on_true, on_false):
	""" to prevent evaluate on_true, or on_false, 
		use qapply( cond, lambda: on_true(), lambda: on_false() ) """
	if cond :
		if isFunction(on_true):
			return apply(on_true)
		else: 
			return on_true 
	else:
		if isFunction(on_false):
			return apply(on_false)
		else: 
			return on_false

def apply_catch(func, args, exlist = Exception, exval = QNOVALUE ):
	""" apply the function and catch error. On specified exceptions,
		Returns exval if specified. If not, return exception object. 
		args is list of args or single arg for func.
		exlist is list of exceptions to mask/catch (all exceptions by default) 
	ex) apply_catch ( lambda (x) : 100/x, 0, StandardError ) """
	
	assert callable(func)
	try:
		return apply (func, qlist(args))
	except exlist, ex :
		if exval == QNOVALUE :
			return ex
		else:
			return exval

# These are used as lambda argument of some function like map()

def car (ll):
	"first element of sequence"
#	assert isSeq(ll)
	return ll[0]

def cdr (ll):
	"remaining sequence except first element"
#	assert isSeq(ll)
	return ll[1:]

def first(ll):
	"first element of sequence"
	if len(ll) >= 1 :
		return ll[0]
	else :
		return None

def second(ll):
	"second element of sequence"
	if len(ll) >= 2 :
		return ll[1]
	else :
		return None
	
def last(ll):
	"last element of sequence"
	if len(ll) >= 1 :
		return ll[-1]
	else :
		return None

def qbound( value, bound1, bound2 = 0 ):
	""" return constrainted value by maximum and minumum bound
	if secocond bound is not given, 0 is assumed """
	if bound1 > bound2:
		return max( min ( value, bound1), bound2)
	else:
		return max( min ( value, bound2), bound1)

def qrange( value, range1, range2 = 0):
	""" similar to qbound, but upper bound is not included in range.
		value should be integer value """
	assert isinstance(value, int)
	if range1 > range2:
		return max( min ( value, bound1), bound2 -1)
	else:
		return max( min ( value, bound2), bound1 -1)

def isBound( value, bound1, bound2 = 0 ):
	""" check the value is within maximum and minumum bound?
	if secocond range is not given, 0 is assumed """
	if bound1 > bound2:
		return bound2 <= value <= bound1
	else:
		return  bound1 <= value <= bound2

def isRange( value, range1, range2 = 0 ):
	""" similar to isbound, but upper range is not included in range.
	if secocond range is not given, 0 is assumed """
	if range1 > range2:
		return range2 <= value < range1
	else:
		return  range1 <= value < range2

def checkIndex( key, aCollection ):
	" is it valid index or key for this collection?"
	if isinstance(aCollection, ( dict, type(set()))) :
		ok = key in aCollection
	else :
		ok = idx >= 0 and idx < len(aCollection)
	assert ok, "index range/key error"
	return ok

def isStrOK(astr): 
	" is non-empty string?"
	return ( astr and isinstance(astr, basestring) )

def checkStrOK(astr):
	"any non-empty string? and with type check"
	ok = astr and  isinstance(astr, basestring)
	assert ok, "Not string or empty"
	return ok

def qtuple(arg):
	" tuple() except that for string and non-sequence,  qtuple(123) => (123,) "
	if isElement(arg):
		return (arg,)
	else:
		return tuple(arg)

def qlist(arg):
	" list()  except for string and non-sequence, qlist(123) => [123] "
	if isElement(arg):
		return [ arg ]
	else:
		return list(arg)

def qlen(arg):
	""" len() for List, Tuple, Dict and Set arg; 0 for <None> arg, 
	1 for string and all unsized, non-enumerables """
	if isContainer(arg):
		return len(arg)
	elif arg == None: 
		return 0
	else:
		return 1

def qntuple(args, leng , pad = None):
	""" force args to be tuple of given length.
	If longer, chop-off right; If shorter, pad right """
	assert leng >=0
	
	argtup = qtuple(args)
	arglen = len(argtup)
	if leng > arglen :
		return argtup + (pad,) * ( leng - arglen )
	else:
		return argtup [ : leng]

def qfind( colstr_haystack, item_needle, default = -1):
	""" find item_needle in colstr_haystack  (may be list, tuple, string, dict)
		without ValueError exception. Returns index in colstr_haystack.
		If not found, return default (-1 if not specified)"""
	if isinstance(colstr_haystack, (list, basestring)) :
		try:
			found = colstr_haystack.index(item_needle)
		except ValueError:
			return default
		return found
	elif isTuple(colstr_haystack) : # tuple has no index()
		if item_needle in colstr_haystack :
			for ii in range(len( colstr_haystack )) :
				if colstr_haystack[ii] == item_needle :
					return ii
		else :
			return default
	elif isDict(colstr_haystack):
		# search values, not keys. If found, retun key : Expensive operation
		try :
			found = colstr_haystack.values().index(item_needle)
		except ValueError:
			return default
		return colstr_haystack.keys()[found]
	else:
		raise TypeError

def qfindany( colstr_haystack, list_needles, default = -1) :
	""" search colstr_haystack (may be list, tuple, string, dict)
		for any item in list_needles (may be any enumerables)
		returns first matched index in colstr_haystack, -1 if not found """

	if isDict(colstr_haystack) :	# for efficency's shake
		haystack = colstr_haystack.values()
		for needle in list_needles :
			try:
				found =  haystack.index( needle )
			except ValueError:
				continue
			return colstr_haystack.keys()[found]
		return default
		
	for needle in list_needles :
		if needle in colstr_haystack :
			return qfind( colstr_haystack, needle, default )

def qfindall( src, item):
	""" Repeat index() on src string or list to the end, 
	Returns list of index numbers of matched items. [] if nothing matched."""
	
	found = 0 ; found_list = []
	while found < len(src):
		try:
			found = src.index(item, found )
		except ValueError:
			break
		found_list.append(found)
		found += 1

	return found_list

def append_unique( alist, item ):
	"append  uniquely new item to list or tuple"
	if isinstance(alist, list) :
		if item not in alist :
			alist.append(item)
	elif isinstance(alist, tuple ) :
		if item not in alist :
			alist += (item ,)
	else :
		assert False, "Not list or tuple" 
		
def sort_mapped( aList, key_mapper, **kw):
	""" sort list. sort key is extracted by applying key_mapper(item)
	key_mapper is function or lambda of 1 arg. Returns sorted list.
	Ex) To reverse sort numbers: x = [ '765', '8', '4321']
	sort_mapped( x, int, reverse=True) => ['4321', '765', '8'] """

	# Build list of [[mapped_key_1, item1], [mappPed_key_2, item2], ... ]
	keyList = zip ( map (key_mapper, aList), aList )
	keyList.sort(**kw)
	return map( (lambda x: x[1]), keyList)

def dict2list(a_dict):
	" convert dict to list like ((key, value), (key2, value2), ...)"
	assert isDict(a_dict), "Not Dict"
	return zip(a_dict.keys(), a_dict.values())

def unzip( *aList ):
	""" reverse of zip() with multiple args 
	unzip((1, 2), ( 3, 4), (5, 6, 7)) ==> [[1, 3, 5], [2, 4, 6]]
	if used with only one arg, assume aList is sequence of sequence to unzip. """
	if (len(aList)) == 1 :
		aList = aList[0]
	leng = min(map(len, aList))
	return [ [ aItem[ii] for aItem in aList ] for ii in range(leng) ]

def strs( *argstrs, **kw ):
	"""concatenate all argument strings into single string.
	a " " is default delimeter if not specified by 'delim' keyword arg """
	delim = kw.get("delim", " ")
	return delim.join(argstrs)

def strcasecmp(s1, s2):
	"""  compare string ignoring case, 
	return result as  positive, 0, negative, not just True/False """
	return cmp(s1.lower(), s2.lower())

# 
# NOTE : builtin string functions :
#	isalpha(), isalnum(), isdigit(), isspace(), islower(), isupper(), istitle() 

def isident(arg):
	" valid python identifier like _ok123"
	if arg and isinstance(arg, basestring) :
		return ( not arg[0].isdigit() and arg.replace("_", "A").isalnum())
	return False

_ident_map = None

def toident(arg):
	global _ident_map
	if not _ident_map :
		_ident_map = "".join( [ chr(ii).isalnum() and chr(ii) or "_"  for ii in range(256) ])
	
	szTmp = arg.strip()
	return szTmp.translate(_ident_map)
	
_ascii_map = None

def toascii(s):
	" ISO-8859-1 or Windows-1252 (CP1252) to pure ascii approximation."
	global _ascii_map
	if not _ascii_map :
		_ascii_map =  ( "".join(map(unichr, range(128)))
			+ """e ,f,.+#^%S<O Z  ''"".--~Ts>o zY !cL*Y|S"Ca<~-R_`+23'uP.,1^>423?"""
			+ "AAAAAAACEEEEIIIIDNOOOOOxOUUUUYTsaaaaaaaceeeeiiiidnooooo%ouuuuyty" )
	return s.translate(_ascii_map)

def tostr(arg) :
	if isinstance(arg, unicode) :
		return arg.encode('latin-1');
	else :
		return arg
##########################################################################

__builtin_const = { 'None': None, 'True': True, 'False': False, 'chr' : chr }

def raw_const_exp(exp):
	"constant-only evals. good for string to value conversion"
	return eval( exp, {'__builtins__': __builtin_const })

def const_exp(exp, default = None):
	"safer string to value conversion. On error, return given default value or None"
	try:
		assert isString( exp )
		return eval( exp, {'__builtins__': __builtin_const })
	except Exception:
		return default
