# $Source: /cvsroot/civ4geneticai/Assets/Python/CUSTOM/PyCivSim.py,v $: : $Name:  $
#	Python Utility Libray for Sid Meier's Civilization 4 Custom Mods.
#	$CopyRight$
#	by SimCutie < simcutie (AT) hanmail.net >
#	Civ4 related but not MOD-specific utility function/classes
#	See "CvLibTest.py" to test this module and usage example
#	This file should reside in  "... /CustomAssets/python/custom" folder.
#	Version: $Revision: 1.1 $ $Date: 2006/09/12 12:43:11 $
#
__author__ = "$Author: the_great_apple $"
__revision__ = "$Id: PyCivSim.py,v 1.1 2006/09/12 12:43:11 the_great_apple Exp $"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2006/09/12 12:43:11 $"

############## FREQUENTLY USED GLOBALS ###################################
import os

from CvPythonExtensions import ( CyGame, CyArtFileMgr, CyInterface,
	ColorTypes, CommerceTypes, YieldTypes, FontTypes, FontSymbols,
	isWorldWonderClass, isTeamWonderClass, isNationalWonderClass, DomainTypes, 
	TradeData, DenialTypes, IDInfo, PlayerTypes, InterfaceMessageTypes,
	EventContextTypes, PopupStates, CyUnit, InputTypes, plotDistance )

from CvPythonExtensions import getCity as cyGetCity, getUnit as cyGetUnit

import CvUtil, PyHelpers
from PyBaseLib import (isString, isInt, isBool, isStr, isTuple, isInstanceType, typeName, 
	qint, QNOVALUE, qbound, const_exp, toascii, sort_mapped, qxor, toident, tostr )
from PyExtLib import path_join, getRegValue, qPrint, qqAssert, qqDebug
import CvCustomEvent 

from CvUtil import gc, localText
Game = CyGame()
ArtFileMgr = CyArtFileMgr()

from PyHelpers import PyPlayer, PyInfo

# Civ 4 game file installation directory 
CIV4_INSTALLDIR = str(getRegValue(
	r"HKLM/SOFTWARE\Firaxis Games/Sid Meier's Civilization 4/INSTALLDIR" ))

__instdir = CIV4_INSTALLDIR.rsplit("\\", 1)[1]

# Civ4 game user data directory under "My Documents"
CIV4_USERDIR =  str(path_join ( getRegValue( 
	"HKCU/Software/Microsoft/Windows/CurrentVersion/Explorer/Shell Folders/Personal" ), 
	"My Games", __instdir ))

gc.setDefineSTRING("CIV4_INSTALLDIR", CIV4_INSTALLDIR)
gc.setDefineSTRING("CIV4_USERDIR", CIV4_USERDIR)
#	Use gc.getDefineSTRING("CIV4_INSTALLDIR"), gc.getDefineSTRING("CIV4_USERDIR")

# GameCoreDLL custom global context
try : 
	from CvPythonExtensions import CyExtGC
	CyDLLGC = CyExtGC()
	qPrint(__name__, "Custom CvGameCoreDLL detected.")
except ImportError:
	CyDLLGC = None

def EventHandlerRegister( evt_mgr, unused = None ):
	qPrint(__name__, "PyCivSim Register OK.")
	eventDict = { 'OnSave-' : onSaveUtil, 'OnLoad+' : onLoadUtil, }
	if hasExtGC() :
		eventDict[ CvCustomEvent.EVENT_INITEND + "+" ] = InitPyCiv
		eventDict[ "Init" ] = onDLLOptionUpdate
		eventDict[ CvCustomEvent.EVENT_OPTIONBEGIN ] = onDLLOptionDraw
		eventDict[ CvCustomEvent.EVENT_OPTIONEND ] = onDLLOptionUpdate
	else:
		eventDict[ CvCustomEvent.EVENT_GAMEBEGIN + "+" ] = InitPyCiv
	return eventDict

def onSaveUtil(args):
	writeScriptData(Game,__name__, GameStoreDict )
	return ""

def onLoadUtil(args):
	GameStoreDict = readScriptData(Game, __name__, {})

EXTDLLCONF_SECTION = "EXTDLLCONF"
EXTDLLCONF_TXTPREFIX = "TXT_KEY_OPTIONS_EXTDLLCONF_"
EXTDLLCONF_LIST = "EXTDLL_CONFIGS"

def onDLLOptionDraw(argsList):
	" draw Ext DLL config options"
	if not hasExtGC() :
		return
	optscr, canvas, Unused = argsList[:3]
	
#	# Display unit type first when displaying custom unit name.
#	optscr.addGenericCheckbox( canvas, "ExtConfUnitNameCheckbox",
#		"TXT_KEY_OPTIONS_EXTDLLCONF_UNITNAME_TYPEFIRST",
#	"EXTDLLCONF", "unitname_typefirst", True )

	confList = const_exp(gc.getDefineSTRING(EXTDLLCONF_LIST))
	assert confList and isTuple(confList)

	for key in confList :
		# Display Ext DLL config Option check box
		txtkey = EXTDLLCONF_TXTPREFIX + key.upper()
		if localText.getText( txtkey, ()) != txtkey :
			optscr.addGenericCheckbox( canvas, "ExtConf" + key.capitalize() + "Checkbox",
				txtkey, EXTDLLCONF_SECTION, key.lower(), getExtDLLConf( key ) )

def onDLLOptionUpdate(argsList):
	" update Ext DLL config options"
	if not hasExtGC() :
		return
	# add all Ext DLL Config update here..
	useOptionSection(EXTDLLCONF_SECTION)
	confList = const_exp(gc.getDefineSTRING(EXTDLLCONF_LIST))
	assert confList and isTuple(confList)
	newConfList = ()
	if confList :
		# update DLL conf to current
		for key in confList :
			setExtDLLConf( key,
			  getCustomOption(EXTDLLCONF_SECTION, key.lower(), getExtDLLConf( key )))
		
##########################################################################
# This should be called after all system internal data is init'ed but no game is loaded. 
# 
def InitPyCiv(dummy):

	global MyInfoMap
	if MyInfoMap :	# it seems already init done...
		return
	if hasExtGC() :
		qPrint("DLL Build", gc.getDefineSTRING("EXTDLL_BUILD"))
		qPrint("DLL Features", gc.getDefineSTRING("EXTDLL_FEATURES"))
		qPrint("DLL Conf. Options", gc.getDefineSTRING("EXTDLL_CONFIGS"))
	else:
		global _imageSlotBegin, _freeimageSlot, _imageSlotEnd
		_imageSlotBegin = findInfoTypeForString("CUSTOM_IMAGESLOT_BEGIN")
		_imageSlotEnd = findInfoTypeForString("CUSTOM_IMAGESLOT_END")
		
		if _imageSlotBegin > 0 and _imageSlotEnd > 0 :
			_freeimageSlot = _imageSlotBegin +1 # first slot	
		else :
			qPrint("No custom image slot")

	if getActiveMODPath() :
		qPrint ("MOD Path", getActiveMODPath(True))
	
	# Symbolic name of few frequently used colors. Ex) icolor = COLOR.BLUE
	global color
	COLOR.WHITE =		gc.getInfoTypeForString("COLOR_WHITE")
	COLOR.BLACK =		gc.getInfoTypeForString("COLOR_BLACK")
	COLOR.GRAY =		gc.getInfoTypeForString("COLOR_GREY")
	COLOR.RED =			gc.getInfoTypeForString("COLOR_RED")
	COLOR.GREEN =		gc.getInfoTypeForString("COLOR_W_GREEN")
	COLOR.BLUE =		gc.getInfoTypeForString("COLOR_BLUE")
	COLOR.CYAN =		gc.getInfoTypeForString("COLOR_CYAN")
	COLOR.YELLOW =		gc.getInfoTypeForString("COLOR_YELLOW")
	COLOR.MAGENTA =		gc.getInfoTypeForString("COLOR_MAGENTA")
	COLOR.BROWN =		gc.getInfoTypeForString("COLOR_X_BROWN")
	COLOR.LIME =		gc.getInfoTypeForString("COLOR_W_LIME")
	COLOR.MAROON =		gc.getInfoTypeForString("COLOR_W_MAROON")
	COLOR.NAVY =		gc.getInfoTypeForString("COLOR_W_NAVY")
	COLOR.OLIVE =		gc.getInfoTypeForString("COLOR_W_OLIVE")
	COLOR.PURPLE =		gc.getInfoTypeForString("COLOR_W_PURPLE")
	COLOR.SILVER =		gc.getInfoTypeForString("COLOR_W_SILVER")
	COLOR.TEAL =		gc.getInfoTypeForString("COLOR_W_TEAL")
	COLOR.ORANGE =		gc.getInfoTypeForString("COLOR_W_ORANGE")

	MyInfoMap = {
		#	from CvUtil.GlobalInfosMap
		'bonus':		( gc.getNumBonusInfos(),		gc.getBonusInfo ),
		'improvement' : ( gc.getNumImprovementInfos(),	gc.getImprovementInfo ),
		'yield':		( YieldTypes.NUM_YIELD_TYPES,	gc.getYieldInfo ),
		'religion':		( gc.getNumReligionInfos(),		gc.getReligionInfo ),
		'tech':			( gc.getNumTechInfos(),			gc.getTechInfo ),
		'unit':			( gc.getNumUnitInfos(),			gc.getUnitInfo ),
		'civic':		( gc.getNumCivicInfos(),		gc.getCivicInfo ),
		'building':		( gc.getNumBuildingInfos(),		gc.getBuildingInfo ),
		'terrain':		( gc.getNumTerrainInfos(),		gc.getTerrainInfo ),
		'trait':		( gc.getNumTraitInfos(),		gc.getTraitInfo ),
		'feature' :		( gc.getNumFeatureInfos(),		gc.getFeatureInfo ),
		'route':		( gc.getNumRouteInfos(),		gc.getRouteInfo ),
		'promotion':	( gc.getNumPromotionInfos(),	gc.getPromotionInfo ),
		#	My addition.
		'civilization':	( gc.getNumCivilizationInfos(),	gc.getCivilizationInfo ),
		'leader':		( gc.getNumLeaderHeadInfos(),	gc.getLeaderHeadInfo ),
		'specialist' :	( gc.getNumSpecialistInfos(),	gc.getSpecialistInfo ),
		'era':			( gc.getNumEraInfos(),			gc.getEraInfo ),
		'control':		( gc.getNumControlInfos(),		gc.getControlInfo ),
		'command':		( gc.getNumCommandInfos(),		gc.getCommandInfo ),
		'build':		( gc.getNumBuildInfos(),		gc.getBuildInfo ),
		'mission':		( gc.getNumMissionInfos(),		gc.getMissionInfo ),
		'gameoption':	( gc.getNumGameOptionInfos(),	gc.getGameOptionInfo ),
		'playeroption':	( gc.getNumPlayerOptionInfos(),	gc.getPlayerOptionsInfo ),
	}

	# Custom Font Symbol Icon in custom GameFont.tga, GameFont_75.tga.
	global CustomFontIcons
	CustomFontIcons = {
		# The name 'gold' is too confusing, So I name it 'money' (stack of coins)
		'money' :			gc.getCommerceInfo(CommerceTypes.COMMERCE_GOLD).getChar(),
		'research' :		gc.getCommerceInfo(CommerceTypes.COMMERCE_RESEARCH).getChar(),
		'culture' :			gc.getCommerceInfo(CommerceTypes.COMMERCE_CULTURE).getChar(),

		'food' :			gc.getYieldInfo(YieldTypes.YIELD_FOOD).getChar(),
		'production' :		gc.getYieldInfo(YieldTypes.YIELD_PRODUCTION).getChar(),
		# this is a round gold coin. What you see in yield display.
		'commerce' :		gc.getYieldInfo(YieldTypes.YIELD_COMMERCE).getChar(),

		# alias for some 'OtherFontIcons' icons from CvUtil.getIcon() 
		'silverstar' :		Game.getSymbolID( FontSymbols.SILVER_STAR_CHAR ),
		'openborders' :		Game.getSymbolID( FontSymbols.OPEN_BORDERS_CHAR ),
		'defensivepact' :	Game.getSymbolID( FontSymbols.DEFENSIVE_PACT_CHAR ),
		'globe':			Game.getSymbolID( FontSymbols.MAP_CHAR),
	}

	for ii in range(gc.getNumReligionInfos()):
		info = gc.getReligionInfo(ii)
		parts = info.getType().split("_", 1)
		if ( len(parts) == 2 and parts[1] ):
			key =  parts[1].lower()
			CustomFontIcons[ key ] = info.getChar()
			CustomFontIcons[ key + 'holycity' ] = info.getHolyCityChar()
		CustomFontIcons[ 'religion' + str(ii) ] = info.getChar()
		CustomFontIcons[ 'holycity' + str(ii) ] = info.getHolyCityChar()

	for ii in range(gc.getNumBonusInfos()):
		info = gc.getBonusInfo(ii)
		parts = info.getType().split("_", 1)
		if ( len(parts) == 2 and parts[1] ):
			key = parts[1].lower()
			if key == 'gold' :
				key = 'goldbar'
			CustomFontIcons[ key ] = info.getChar()
		CustomFontIcons[ 'bonus' + str(ii) ] = gc.getBonusInfo(ii).getChar()

	addCustomIcon( 'blank', 0 )
	addCustomIcon( 'war', 14)
	addCustomIcon( 'people', 15)

	addCustomIcon( 'noattitude', 17 )
	addCustomIcon( 'furious', 18 )
	addCustomIcon( 'annoyed', 19 )
	addCustomIcon( 'cautious', 20 )
	addCustomIcon( 'pleased', 21 )
	addCustomIcon( 'friendly', 22 )

	if hasExtGC() :
		for key, icon in CustomFontIcons.items() :
			getExtGC().defineIcon( key, icon)

	global ICON		# frequently used icons

	ICON.MONEY			= getIcon('money')
	ICON.RESEARCH		= getIcon('research')
	ICON.CULTURE		= getIcon('culture')
	ICON.FOOD			= getIcon('food')
	ICON.PRODUCTION		= getIcon('production')
	ICON.COMMERCE		= getIcon('commerce')

	ICON.HAPPY			= getIcon('happy')
	ICON.UNHAPPY		= getIcon('unhappy')
	ICON.HEALTHY		= getIcon('healthy')
	ICON.UNHEALTHY		= getIcon('unhealthy')
	ICON.BULLET			= getIcon('bullet')
	ICON.STRENGTH		= getIcon('strength')
	ICON.MOVES			= getIcon('moves')
	ICON.RELIGION		= getIcon('religion')
	ICON.STAR			= getIcon('star')
	ICON.SILVERSTAR		= getIcon('silverstar')
	ICON.TRADE			= getIcon('trade')
	ICON.DEFENSE		= getIcon('defense')
	ICON.GREATPEOPLE	= getIcon('greatpeople')
	ICON.BADGOLD		= getIcon('badgold')
	ICON.BADFOOD		= getIcon('badfood')
	ICON.EATENFOOD		= getIcon('eatenfood')
	ICON.GOLDENAGE		= getIcon('goldenage')
	ICON.ANGRYPOP		= getIcon('angrypop')
	ICON.OPENBORDERS	= getIcon('openborders')
	ICON.DEFENSIVEPACT	= getIcon('defensivepact')
	ICON.MAP			= getIcon('map')
	ICON.OCCUPATION		= getIcon('occupation')
	ICON.POWER			= getIcon('power')

	ICON.GLOBE			= getIcon('globe')

	ICON.BLANK			= getIcon('blank')
	ICON.WAR			= getIcon('war')
	ICON.PEOPLE			= getIcon('people')

#=========================================================================
	global _def_articles #, _ascii_map 
	t1 = localText.getText("TXT_KEY_DEFINITE_ARTICLE", ()).split(":")
	_def_articles = [ (x2.replace("_", " "), len(x2), "," + x2.rstrip("_")) for x2 in t1 ]

#	_ascii_map = ( u"".join(map(unichr, range(192))) 
#		+ u"AAAAAAACEEEEIIIIDNOOOOOxOUUUUYTsaaaaaaaceeeeiiiidnooooo/ouuuuyty" )

	global UnitCategories

	UnitCategories.NO_UNITCAT			= -1 # gc.getInfoTypeForString("NO_UNITCOMBAT")
	UnitCategories.UNITCAT_RECON		= gc.getInfoTypeForString("UNITCOMBAT_RECON")
	UnitCategories.UNITCAT_ARCHER		= gc.getInfoTypeForString("UNITCOMBAT_ARCHER")
	UnitCategories.UNITCAT_MOUNTED		= gc.getInfoTypeForString("UNITCOMBAT_MOUNTED")
	UnitCategories.UNITCAT_MELEE		= gc.getInfoTypeForString("UNITCOMBAT_MELEE")
	UnitCategories.UNITCAT_SIEGE		= gc.getInfoTypeForString("UNITCOMBAT_SIEGE")
	UnitCategories.UNITCAT_GUN			= gc.getInfoTypeForString("UNITCOMBAT_GUN")
	UnitCategories.UNITCAT_ARMOR		= gc.getInfoTypeForString("UNITCOMBAT_ARMOR")
	UnitCategories.UNITCAT_HELICOPTER	= gc.getInfoTypeForString("UNITCOMBAT_HELICOPTER")
	UnitCategories.UNITCAT_NAVAL		= gc.getInfoTypeForString("UNITCOMBAT_NAVAL")
	numCombats = gc.getNumUnitCombatInfos()
	UnitCategories.UNITCAT_AIR			= numCombats
	UnitCategories.UNITCAT_WORKER		= numCombats +1
	UnitCategories.UNITCAT_MISSIONARY	= numCombats +2
	UnitCategories.UNITCAT_GREATPEOPLE	= numCombats +3
	UnitCategories.UNITCAT_ANIMAL		= numCombats +4
	UnitCategories.UNITCAT_SPECIAL		= numCombats +5
	UnitCategories.NUM_UNITCATS			= numCombats +6

	qPrint(__name__, "Init Done." )

KeyBoardMapping = {
	"KB_ESCAPE" : ( InputTypes.KB_ESCAPE, "TXT_KEY_KEYBOARD_ESCAPE" ),
	"KB_0" : ( InputTypes.KB_0, "0" ),
	"KB_1" : ( InputTypes.KB_1, "1" ),
	"KB_2" : ( InputTypes.KB_2, "2" ),
	"KB_3" : ( InputTypes.KB_3, "3" ),
	"KB_4" : ( InputTypes.KB_4, "4" ),
	"KB_5" : ( InputTypes.KB_5, "5" ),
	"KB_6" : ( InputTypes.KB_6, "6" ),
	"KB_7" : ( InputTypes.KB_7, "7" ),
	"KB_8" : ( InputTypes.KB_8, "8" ),
	"KB_9" : ( InputTypes.KB_9, "9" ),
	"KB_MINUS" : ( InputTypes.KB_MINUS, "-" ),		# - on main keyboard
	"KB_A" : ( InputTypes.KB_A, "A" ),
	"KB_B" : ( InputTypes.KB_B, "B" ),
	"KB_C" : ( InputTypes.KB_C, "C" ),
	"KB_D" : ( InputTypes.KB_D, "D" ),
	"KB_E" : ( InputTypes.KB_E, "E" ),
	"KB_F" : ( InputTypes.KB_F, "F" ),
	"KB_G" : ( InputTypes.KB_G, "G" ),
	"KB_H" : ( InputTypes.KB_H, "H" ),
	"KB_I" : ( InputTypes.KB_I, "I" ),
	"KB_J" : ( InputTypes.KB_J, "J" ),
	"KB_K" : ( InputTypes.KB_K, "K" ),
	"KB_L" : ( InputTypes.KB_L, "L" ),
	"KB_M" : ( InputTypes.KB_M, "M" ),
	"KB_N" : ( InputTypes.KB_N, "N" ),
	"KB_O" : ( InputTypes.KB_O, "O" ),
	"KB_P" : ( InputTypes.KB_P, "P" ),
	"KB_Q" : ( InputTypes.KB_Q, "Q" ),
	"KB_R" : ( InputTypes.KB_R, "R" ),
	"KB_S" : ( InputTypes.KB_S, "S" ),
	"KB_T" : ( InputTypes.KB_T, "T" ),
	"KB_U" : ( InputTypes.KB_U, "U" ),
	"KB_V" : ( InputTypes.KB_V, "V" ),
	"KB_W" : ( InputTypes.KB_W, "W" ),
	"KB_X" : ( InputTypes.KB_X, "X" ),
	"KB_Y" : ( InputTypes.KB_Y, "Y" ),
	"KB_Z" : ( InputTypes.KB_Z, "Z" ),
	"KB_EQUALS" : ( InputTypes.KB_EQUALS, "=" ),
	"KB_BACKSPACE" : ( InputTypes.KB_BACKSPACE, "TXT_KEY_KEYBOARD_BACKSPACE" ),
	"KB_TAB" : ( InputTypes.KB_TAB, "TAB" ),
	"KB_LBRACKET" : ( InputTypes.KB_LBRACKET, "[" ),
	"KB_RBRACKET" : ( InputTypes.KB_RBRACKET, "]" ),
	"KB_RETURN" : ( InputTypes.KB_RETURN, "TXT_KEY_KEYBOARD_ENTER" ),		# Enter on main keyboard
	"KB_LCONTROL" : ( InputTypes.KB_LCONTROL, "TXT_KEY_KEYBOARD_LEFT_CONTROL_KEY" ),
	"KB_SEMICOLON" : ( InputTypes.KB_SEMICOLON, ";" ),
	"KB_APOSTROPHE" : ( InputTypes.KB_APOSTROPHE, "'" ),
	"KB_GRAVE" : ( InputTypes.KB_GRAVE, "`" ),		# accent grave
	"KB_LSHIFT" : ( InputTypes.KB_LSHIFT, "TXT_KEY_KEYBOARD_LEFT_SHIFT_KEY" ),
	"KB_BACKSLASH" : ( InputTypes.KB_BACKSLASH, "\\" ),
	"KB_COMMA" : ( InputTypes.KB_COMMA, "," ),
	"KB_PERIOD" : ( InputTypes.KB_PERIOD, "." ),
	"KB_SLASH" : ( InputTypes.KB_SLASH, "/" ),
	"KB_RSHIFT" : ( InputTypes.KB_RSHIFT, "TXT_KEY_KEYBOARD_RIGHT_SHIFT_KEY" ),
	"KB_NUMPADSTAR" : ( InputTypes.KB_NUMPADSTAR, "TXT_KEY_KEYBOARD_NUM_PAD_STAR" ),
	"KB_LALT" : ( InputTypes.KB_LALT, "TXT_KEY_KEYBOARD_LEFT_ALT_KEY" ),
	"KB_SPACE" : ( InputTypes.KB_SPACE, "TXT_KEY_KEYBOARD_SPACE_KEY" ),
	"KB_CAPSLOCK" : ( InputTypes.KB_CAPSLOCK, "TXT_KEY_KEYBOARD_CAPS_LOCK" ),
	"KB_F1" : ( InputTypes.KB_F1, "F1" ),
	"KB_F2" : ( InputTypes.KB_F2, "F2" ),
	"KB_F3" : ( InputTypes.KB_F3, "F3" ),
	"KB_F4" : ( InputTypes.KB_F4, "F4" ),
	"KB_F5" : ( InputTypes.KB_F5, "F5" ),
	"KB_F6" : ( InputTypes.KB_F6, "F6" ),
	"KB_F7" : ( InputTypes.KB_F7, "F7" ),
	"KB_F8" : ( InputTypes.KB_F8, "F8" ),
	"KB_F9" : ( InputTypes.KB_F9, "F9" ),
	"KB_F10" : ( InputTypes.KB_F10, "F10" ),
	"KB_NUMLOCK" : ( InputTypes.KB_NUMLOCK, "TXT_KEY_KEYBOARD_NUM_LOCK" ),
	"KB_SCROLL" : ( InputTypes.KB_SCROLL, "TXT_KEY_KEYBOARD_SCROLL_KEY" ),
	"KB_NUMPAD7" : ( InputTypes.KB_NUMPAD7, "TXT_KEY_KEYBOARD_NUMPAD_NUMBER", 7 ),
	"KB_NUMPAD8" : ( InputTypes.KB_NUMPAD8, "TXT_KEY_KEYBOARD_NUMPAD_NUMBER", 8 ),
	"KB_NUMPAD9" : ( InputTypes.KB_NUMPAD9, "TXT_KEY_KEYBOARD_NUMPAD_NUMBER", 9 ),
	"KB_NUMPADMINUS" : ( InputTypes.KB_NUMPADMINUS, "TXT_KEY_KEYBOARD_NUMPAD_MINUS" ),
	"KB_NUMPAD4" : ( InputTypes.KB_NUMPAD4, "TXT_KEY_KEYBOARD_NUMPAD_NUMBER", 4 ),
	"KB_NUMPAD5" : ( InputTypes.KB_NUMPAD5, "TXT_KEY_KEYBOARD_NUMPAD_NUMBER", 5 ),
	"KB_NUMPAD6" : ( InputTypes.KB_NUMPAD6, "TXT_KEY_KEYBOARD_NUMPAD_NUMBER", 6 ),
	"KB_NUMPADPLUS" : ( InputTypes.KB_NUMPADPLUS, "TXT_KEY_KEYBOARD_NUMPAD_PLUS" ),
	"KB_NUMPAD1" : ( InputTypes.KB_NUMPAD1, "TXT_KEY_KEYBOARD_NUMPAD_NUMBER", 1 ),
	"KB_NUMPAD2" : ( InputTypes.KB_NUMPAD2, "TXT_KEY_KEYBOARD_NUMPAD_NUMBER", 2 ),
	"KB_NUMPAD3" : ( InputTypes.KB_NUMPAD3, "TXT_KEY_KEYBOARD_NUMPAD_NUMBER", 3 ),
	"KB_NUMPAD0" : ( InputTypes.KB_NUMPAD0,  "TXT_KEY_KEYBOARD_NUMPAD_NUMBER", 0 ),
	"KB_NUMPADPERIOD" : ( InputTypes.KB_NUMPADPERIOD, "TXT_KEY_KEYBOARD_NUMPAD_PERIOD" ), 
	"KB_F11" : ( InputTypes.KB_F11, "F11" ),
	"KB_F12" : ( InputTypes.KB_F12, "F12" ),
	"KB_NUMPADEQUALS" : ( InputTypes.KB_NUMPADEQUALS, "TXT_KEY_KEYBOARD_NUMPAD_EQUALS" ),
	"KB_AT" : ( InputTypes.KB_AT, "@" ),
	"KB_UNDERLINE" : ( InputTypes.KB_UNDERLINE, "_" ),
	"KB_COLON" : ( InputTypes.KB_COLON, ":" ),
	"KB_NUMPADENTER" : ( InputTypes.KB_NUMPADENTER, "TXT_KEY_KEYBOARD_NUMPAD_ENTER_KEY" ),
	"KB_RCONTROL" : ( InputTypes.KB_RCONTROL, "TXT_KEY_KEYBOARD_RIGHT_CONTROL_KEY" ),
	"KB_VOLUMEDOWN" : ( InputTypes.KB_VOLUMEDOWN, "TXT_KEY_KEYBOARD_VOLUME_DOWN" ),
	"KB_VOLUMEUP" : ( InputTypes.KB_VOLUMEUP, "TXT_KEY_KEYBOARD_VOLUME_UP" ),
	"KB_NUMPADCOMMA" : ( InputTypes.KB_NUMPADCOMMA, "TXT_KEY_KEYBOARD_NUMPAD_COMMA" ),
	"KB_NUMPADSLASH" : ( InputTypes.KB_NUMPADSLASH, "TXT_KEY_KEYBOARD_NUMPAD_SLASH" ),
	"KB_SYSRQ" : ( InputTypes.KB_SYSRQ, "TXT_KEY_KEYBOARD_SYSRQ" ),
	"KB_RALT" : ( InputTypes.KB_RALT, "TXT_KEY_KEYBOARD_RIGHT_ALT_KEY" ),
	"KB_PAUSE" : ( InputTypes.KB_PAUSE, "TXT_KEY_KEYBOARD_PAUSE_KEY" ),
	"KB_HOME" : ( InputTypes.KB_HOME, "TXT_KEY_KEYBOARD_HOME_KEY" ),
	"KB_UP" : ( InputTypes.KB_UP, "TXT_KEY_KEYBOARD_UP_ARROW" ),
	"KB_PGUP" : ( InputTypes.KB_PGUP, "TXT_KEY_KEYBOARD_PAGE_UP" ),
	"KB_LEFT" : ( InputTypes.KB_LEFT, "TXT_KEY_KEYBOARD_LEFT_ARROW" ),
	"KB_RIGHT" : ( InputTypes.KB_RIGHT, "TXT_KEY_KEYBOARD_RIGHT_ARROW" ),
	"KB_END" : ( InputTypes.KB_END, "TXT_KEY_KEYBOARD_END_KEY" ),
	"KB_DOWN" : ( InputTypes.KB_DOWN, "TXT_KEY_KEYBOARD_DOWN_ARROW" ),
	"KB_PGDN" : ( InputTypes.KB_PGDN, "TXT_KEY_KEYBOARD_PAGE_DOWN" ),
	"KB_INSERT" : ( InputTypes.KB_INSERT, "TXT_KEY_KEYBOARD_INSERT_KEY" ),
	"KB_DELETE" : ( InputTypes.KB_DELETE, "TXT_KEY_KEYBOARD_DELETE_KEY" ),
}

def getKeyBoardMapping( key ) :
	""" key is key tag like "KB_F1" or key code like InputTypes.KB_F1
	Result is triple of ( key tag, key code, key name text key) 
	ex) ( "KB_LEFT", InputTypes.KB_LEFT, "TXT_KEY_KEYBOARD_LEFT_ARROW" ) """
	if isString( key ) :
		mapped = KeyBoardMapping.get(key)
		if mapped is not None :
			return (key,) + mapped
	elif isInt( key ) or isinstance(key, InputTypes ) :
		for item in KeyBoardMapping.items() :
			if int(key) == int(item[1][0]) :
				return ( item[0],) + item[1]

	return None

def keyCode( keytag ) :
	""" translate string keycode name to InputTypes keycode ex) "KB_F1" => KB_F1 """ 
	mapping = getKeyBoardMapping( keytag )
	if mapping :
		return mapping[1]

MouseEventTypes = { 
	"LBUTTONDOWN" : 1,	"LCBUTTONDBLCLICK" : 2,		"RBUTTONDOWN" : 3,
	"WHEELBACK" : 4,	"WHEELFORWARD" : 5,
	"LMOUSE" : 1,	"LCLICK" : 1,	"LBUTTON" : 1,	"DBLCLICK" : 2,
	"RMOUSE" : 3,	"RCLICK" : 3,	"RBUTTON" : 3,
	"MOUSE" : 999,	"CLICK" : 999,	"BACK" : 4,		"FORWARD" : 5, }

def checkKeyInput( inputArgsList, szKeyDesc, bKeyDownEvent = True ):
	""" check keyboard/mouse event handler args List if it matches string key description.
	szKeyDesc is like 'A', 'backspace, 'shift kb_f1', 'CTL alt-D', 'CTRL-KB_NUMPADENTER'
	For mouse event, use event name listed above ex) 'RBUTTONDOWN', 'alt lbutton' ... """

	( isKeyDownEvent, theKey, mx, my, px, py, 
		( keyModifier, bShift, bCtrl, bAlt), Unused ) = inputArgsList
#	( isLButtonDownEvent, eventType, mx, my, px, py, 
#		( keyModifier, bShift, bCtrl, bAlt), Unused )
	keywords = szKeyDesc.upper().replace("-", " ").split()
	modifier = set(keywords[:-1])
	keyTag = keywords[-1]

	if ( qxor( bShift, "SHIFT" in modifier )
		or qxor( bCtrl, modifier.intersection(["CONTROL", "CTL", "CTRL"]))
		or qxor( bAlt, "ALT" in modifier ) or qxor( keyModifier, modifier )) :
		return False

	mouseEventID = MouseEventTypes.get( keyTag, -1 )
	if mouseEventID > 0 :		# theKey is mouse eventType
		return ( mouseEventID == int(theKey)
				or ( mouseEventID == 999 and int(theKey) in ( 1, 2 ,3) ))
	if qxor( isKeyDownEvent, bKeyDownEvent ) :
		return False

	if not keyTag.startswith("KB_" ) :
		keyTag = "KB_" + keyTag
	if int(theKey) != int(keyCode( keyTag )) :
		return False
	return True

def getHotkey( modName, command, default ):
	"""get hokey information for checkKeyInput()
	ex) HOTKEY_CAST = getHotkey( "MAGIC_MOD", "CAST_MAGIC", "Control-Shift-C" ) """
	useOptionSection( "MOD_HOKEY" ) 
	option = toident( "mod_" + modName + "_hotkey_" + command )
	return getCustomOption( "MOD_HOKEY", option, default)

# FUNCTION ScreenRegister() from "CvScreenUtils.py"
# 1) to get receive input event (key press/mouse click) and screen event ( redraw...)  
#	in new screen ( in handleInput() or update(), onClose(), leftMouseUp(), .... )
# 2) to overide/replace existing stock screen to new subclassed screen instance.
#	See "CvScreenUtil.py" for detailed documentation.
#
def ScreenRegister(screenInst, screenEnum, inputEventList = None, Unused = None , **kwargs):
	"Register screen for input handling. See 'CvScreenUtils.py' for details"
	import CvScreenUtilsInterface
	qqAssert ( ( isInstanceType(screenInst) and isInt( screenEnum)), "ScreenRegister arg error")
	ret = CvScreenUtilsInterface.getScreenUtils().ScreenRegister(
		screenInst, screenEnum, inputEventList, Unused )
	if ret == None:
		qPrint( "ScreenRegister failed. Enum clash?", typeName(screen), screenEnum)

def triggerEvent(*argsList):
	import  CvEventInterface
	CvEventInterface.onEvent( argsList + (0,0,0,0,0,0))

def setPythonEntryPoint( funcName, funcDef):
	""" Add Python Entry poin function to CvGameInterface module.
		Added function can be called from SDK/"CvExtension.cpp/callGameEntryPoint() """
	import CvGameInterface
	assert funcName and isString(funcName) and callable(funcDef)
	setattr(CvGameInterface, funcName, funcDef ) 

############## GAME RESOUCE UTILS ########################################

def TXT_KEY(key, *args, **kw):
	""" simpler localText.getText(). Use text key with "TXT_KEY_" prefix removed.
	Ex) TXT_KEY("MISSIONARY_DEMAND", religion, unitname, cityname) 
	If text is not found, return value givebn by 'default' keyword ( "" if not given)
	It accepts 'color', 'font' keyword. ex) color="RED", font ="4b" """
	assert key and isStr(key)
	
	if not key.startswith("TXT_KEY_" ):
		key = "TXT_KEY_" + key.upper()

	loctxt = localText.getText( key, args)
	if loctxt == key :
		loctxt = kw.get('default', "")
		if qqDebug() > 1 :
			qPrint("Untranslated text", key )

	if kw.get('case') == "u" :
		loctxt = loctxt.upper()
	
	color = kw.get('color')
	if color is not None :
		loctxt = localText.changeTextColor(loctxt, getColor(color)) 
		
	font = kw.get('font', '')
	if font :
		loctxt = u"<font=%s>%s</font>" % (str(font), loctxt )
	return loctxt

def getText( key, default = u"", args = () ) :
	""" like localtext.getText(). argument is optional. (assumes ()).
	If text does not found, return default ( "" if not given) """
	return TXT_KEY( key, default=default, *args )

# Popup defines (TODO: Expose these from C++)
JUST_C = CvUtil.FONT_CENTER_JUSTIFY
JUST_R = CvUtil.FONT_RIGHT_JUSTIFY
JUST_L = CvUtil.FONT_LEFT_JUSTIFY

FONT_TITLE = FontTypes.TITLE_FONT
FONT_GAME = FontTypes.GAME_FONT
FONT_SMALL = FontTypes.SMALL_FONT

def setTextEntry( key, text ):
	"Add or set text entry to Game text table."
	assert ( key and text and hasExtGC())
	if hasExtGC():
		getExtGC().setTextEntry( key, unicode(text) )

############## MESSAGING/LOGGING UTILS ###################################

def MSGDEBUG( msg ):
	CyInterface().addImmediateMessage(msg, "")

LOG_NONE = InterfaceMessageTypes.MESSAGE_TYPE_DISPLAY_ONLY	# no log, screen display only
LOG_TEMP = InterfaceMessageTypes.MESSAGE_TYPE_INFO			# transient log
LOG_PERM = InterfaceMessageTypes.MESSAGE_TYPE_MAJOR_EVENT	# permanent log

# *TODO* Add more sounds 
_sound_list = [ "",   "AS2D_CHAT", "A2D_ERROR", "AS2D_UNIT_GREATPEOPLE", ]

# addImmediateMessage(...) - void (wstring szString, string szSound) 
# addMessage(...) - void (int /*PlayerTypes*/ ePlayer, bool bForce, int iLength, wstring szString,
#	string szSound = NULL, int /*InterfaceMessageTypes*/ eType = MESSAGE_TYPE_INFO, 
#	string szIcon = NULL, ColorTypes eFlashColor = NO_COLOR, int iFlashX = -1, int iFlashY = -1,
#	bool bShowOffScreenArrows = false, bool bShowOnScreenArrows = false) 
#
#	InterfaceMessageTypes
#		NO_MESSAGE_TYPE
#		MESSAGE_TYPE_INFO
#		MESSAGE_TYPE_DISPLAY_ONLY
#		MESSAGE_TYPE_MAJOR_EVENT
#		MESSAGE_TYPE_MINOR_EVENT
#		MESSAGE_TYPE_CHAT
#		MESSAGE_TYPE_COMBAT_MESSAGE
#		NUM_INTERFACE_MESSAGE_TYPES

def MsgAdd(message, msgType = LOG_NONE, bUrgent = True, color = None, sound = 0,  iDuration = 0):
	""" 
	bUrgent = Immediate or queued, 
	msgType; InterfaceMessageTypes or LOG_NONE, LOG_TEMP, LOG_PERM
	color : integer index out of the colors from "CIV4ColorVals.xml"(2 = white, 8 = green,..) 
			or color name string like "COLOR_CYAN","COLOR_X_ORANGE". Defaut is white.
	sound : integer index from predefined sound from _sound_list[]; Default is 0, (No sound)
		or Sound Script ID string like "AS2D_CHAT" in "Audio2DScripts.xml"
	iDuration: number of seconds to display message. default = 10 sec.
	"""
	iColor = getColor(color)
	if isInt(sound):
		sound = _sound_list[ qbound( sound, len(_sound_list)-1) ]
	if iDuration == 0 :
		iDuration = 10 + len(message) // 10

	CyInterface().addMessage( Game.getActivePlayer(), bUrgent, iDuration, message,
		sound, msgType, None, iColor , -1, -1, False, False)

def MsgAnnounce(message, **kwargs):
	MsgAdd ( message, msgType = LOG_PERM, sound = 1, **kwargs)

def MsgInform(message, **kwargs):
	MsgAdd ( message, msgType = LOG_TEMP, color=COLOR.LIME, sound = 1, **kwargs)

def MsgDisplay( message, **kwargs):
	MsgAdd ( message, msgType = LOG_NONE, **kwargs)

def MsgPost( message, bUrgent=False, **kwargs):
	MsgAdd ( message, msgType = LOG_NONE, bUrgent=bUrgent, **kwargs)

#
#	To create popup event, use PyCivSim.createPopup( applyHandler, headKey = "", *userData ).
#
#	createPopup() will create popup and show popup to the Player.
#	When player clicks OK buton or close button, the popup is closed and the appyHandler will be called with arguments ( playerID, userData, popupReturn) by CvSimCutieEventManager
#
#	Example) 
#		def ShowCityNamePopup(city):
#			popup = PyCivSim.createPopup( CityNamePopupApply, "CITY NAME", 11, 222, "Good" )
#			popup.setBodyString("Enter City Name")
#			popup.popup.createEditBox(city.getName())
#			launchPopup(popup, POPUPSTYLE_OK)
#
#		def CityNamePopupApply( playerID, userData, popupReturn):
#			# popup closed and returned. Process result.
#			# Data from ShowPopup ( 11, 222, "Good" )
#			(userdata1, userdata2, userdata3) = userData 
#
#			cityName = popupReturn.getEditBoxString(0)
#			.....

def createPopup( applyHandler, headKey = "", *userData ):
	" create popup and set apply event handler"

	import CvEventInterface
	import Popup as PyPopup
	assert callable( applyHandler )

	event_mgr = CvEventInterface.getEventManager()
	eventID = event_mgr.addPopupEvent(applyHandler)
	popup = PyPopup.PyPopup(eventID, EventContextTypes.EVENTCONTEXT_ALL)
	if headKey :
		if isStr(headKey) :
			headStr = localText.getText( headKey, ())
		else :
			headStr = headKey
		popup.setHeaderString(headStr)
	if userData:
		popup.setUserData(userData)
	return popup

class PopupStyles :
	NO_POPUPSTYLE				= 0
	POPUPSTYLE_OK				= 1
	POPUPSTYLE_OKCANCEL			= 2
	POPUPSTYLE_YESNO			= 3
	POPUPSTYLE_YESNOCANCEL		= 4

def	launchPopup( popup, ePopupStyle = PopupStyles.POPUPSTYLE_OK, 
	eState = PopupStates.POPUPSTATE_IMMEDIATE ):

	txtOK	= localText.getText("TXT_KEY_MAIN_MENU_OK", ())
	txtCancel = localText.getText("TXT_KEY_POPUP_CANCEL", ())
	txtYes = localText.getText("TXT_KEY_POPUP_YES", ())
	txtNo = localText.getText("TXT_KEY_POPUP_NO", ())

	if ePopupStyle == PopupStyles.POPUPSTYLE_OK :
		popup.addButton(txtOK)
	elif ePopupStyle == PopupStyles.POPUPSTYLE_OKCANCEL :
		popup.addButton(txtOK)
		popup.addButton(txtCancel)
	elif ePopupStyle == PopupStyles.POPUPSTYLE_YESNO :
		popup.addButton(txtYes)
		popup.addButton(txtNo)
	elif ePopupStyle == PopupStyles.POPUPSTYLE_YESNOCANCEL :
		popup.addButton(txtYes)
		popup.addButton(txNo)
		popup.addButton(txtCancel)

	popup.launch( False, eState )

class WidgetGroup:
	def __init__(self, szPrefix, bNoGroup = False):
		assert szPrefix
		self.groupPrefix = szPrefix
		self.bNoGroup = bNoGroup
		self.clear()

	def clear(self):
		self.widgetCount = 0
		self.widgetList = []
	
	def count(self):
		return self.widgetCount

	def list(self):
		return self.widgetList
	
	def next(self):
		name = self.groupPrefix + str(self.widgetCount)
		if self.bNoGroup :
			name += "X"
		self.widgetList.append(name)
		self.widgetCount += 1
		return name

MyInfoMap = {}

def findInfoTypeForString(infoTag):
	" works like findInfoTypeForString()"
	if hasExtGC():
		return getExtGC().findInfoTypeForString(infoTag)
	else:
		return gc.getInfoTypeForString(infoTag)

def matchInfo( index, numInfo, infoname, getter, useTag):
	"helper function for findInfo"
	if index < numInfo :
		info = getter(index)
		if useTag :
			name = info.getTag()
		else :
			name = info.getType()		
		if name == infoname :
			return info
	return None

def findInfo( infoName , infoType = "" ):
	""" searches all infoType for game info structure by string name
	Similar to CvUtil.getInfo() but more general.
	returns triple CvInfo obj
	if infoType is not given, all infoType is seached """

	if hasExtGC() :
		return getExtGC().findInfo(infoName)

	infoNameType = infoName.upper()
	index =  gc.getInfoTypeForString(infoNameType)
	if index < 0 :
		return
#	infoNameDesc = infoName.capitalize()
	infoType = infoType.lower()
	
	if not infoType :
		parts = infoName.split("_", 1)
		if len(parts) == 2 and parts[0].lower() in MyInfoMap :
			infoType = parts[0].lower()

	if not infoType :	# search all the dictionaries...
		for numInfo, getter in MyInfoMap.values() :
			found = matchInfo( index, numInfo, infoNameType, getter, False)
			if found :
				return found
	elif infoType in MyInfoMap :
		numInfo, getter = MyInfoMap[ infoType ]
		return matchInfo( index, numInfo, infoNameType, getter, False)
		
	return None

def getColor(color):
	""" get color by name string. Returns integer ColorType.
	Ex) getColor("COLOR_YELLOW"), getColor("white") """

	if isString(color):
		if not color.startswith("COLOR_" ):
			color = "COLOR_" + color.upper()
		color = gc.getInfoTypeForString( color )

	iColor = ColorTypes( isInt(color) and color or 2)	# 2 = 100% white
	return iColor

# Symbolic name of few frequently used colors. Ex) icolor = COLOR.BLUE
class COLOR:	# inited later"
	pass

def textColor(iColor):
	""" convert color ID or RGBA 32 bit color to GFC color string 
	like "<color=255,96,128,255>" """

	if ( 0 <= iColor <= 255 ):	# color index
		color = gc.getColorInfo(iColor).getColor()
		(cr, cg, cb, ca ) = (int(color.r * 255), int(color.g * 255),
			int(color.b * 255), int(color.a * 255))
	else :
		# ( 0x000000ff <= iColor <= 0xffffffff ): # RGBA
		(cr, cg, cb, ca ) = ( iColor >> 24 & 0xff, iColor >> 16 & 0xff, 
			iColor >> 8 & 0xff, iColor & 0xff )

	return "<color=%d,%d,%d,%d>" % (cr, cg, cb, ca )	# GFC style

def textPlayerColor(iPlayer):
	" get player text color as GFC color string"
	pPlayer = gc.getPlayer(iPlayer)
	rgba = ( pPlayer.getPlayerTextColorR(), pPlayer.getPlayerTextColorG(),
		pPlayer.getPlayerTextColorB(), pPlayer.getPlayerTextColorA())
	return u"<color=%d,%d,%d,%d>" % rgba

def getPlayerColor(iPlayer, kind = 1):
	"""	kind == 0 ==> Text player color
		kind == 1 ==> Primary player color
		kind == 2 ==> Secondary player color """
	
	colorInfo = gc.getPlayerColorInfo(gc.getPlayer(iPlayer).getPlayerColor())
	if kind == 1 :
		icolor = colorInfo.getColorTypePrimary()
	elif kind == 2 :
		icolor = colorInfo.getColorTypeSecondary()
	elif kind == 0 or kind == 3:
		icolor = colorInfo.getColorTypeText()
	else :
		icolor = 0
	return iColor

def addCustomIcon(charEntry, offset ):
	""" attach symbolic name to custom font symbol icon.
	iCodeVal is ordial int value of unicode char.
	if offset < 25, it is postion of the symbol in last row of GameFont.tga.
	leftmost symbol position index = 0, rightmost = 24 """
	global CustomFontIcons 

	assert 0 <= offset < 25
	CustomFontIcons[charEntry.lower()] = Game.getSymbolID(FontSymbols.HAPPY_CHAR)+ 25 + offset

from htmlentitydefs import entitydefs
CustomFontIcons = {}

def getIcon(charEntry ):
	""" Like CvUtil.getIcon(), Returns unicode character of 
	custom font symbols icon or HTML named Latin-1 entity special character
	HTML characters are named like "&bull;" (case-sensitive)
	Unlike CvUtil.getIcon(), it works well in non-English locale. """

	if charEntry[0] ==  "&" and charEntry[-1] == ";" :
		charEntry = charEntry[1:-1]
	iconEntry = charEntry.lower()

	if CustomFontIcons.has_key( iconEntry ) :
		if hasExtGC():
			return getExtGC().getIcon(iconEntry)
		else :
			return unichr(CustomFontIcons.get( iconEntry ))
	elif CvUtil.FontIconMap.has_key( iconEntry ) :
		assert iconEntry != 'gold', "Too confusing name 'gold'. Use 'money' or 'goldbar'."
		return CvUtil.getIcon( iconEntry )
	# case sensitive...
	elif entitydefs.has_key( charEntry ) and entitydefs.get(charEntry) < 256: 
		return 	unichr( entitydefs.get(charEntry))
	else :
		return u"?"

class ICON :	# init'ed later.
	pass

_ArtInfoMap = {
	"UNIT_":			(gc.getNumUnitArtInfos,			gc.getUnitArtInfo ),
	"BUILDING_":		(gc.getNumBuildingArtInfos,		gc.getBuildingArtInfo ),
	"BONUS_":			(gc.getNumBonusArtInfos,		gc.getBonusArtInfo ),
	"IMPROVEMENT_":		(gc.getNumImprovementArtInfos,	gc.getImprovementArtInfo ),
	"CIVILIZATION_":	(gc.getNumCivilizationArtInfos,	gc.getCivilizationArtInfo ),
	"LEADER_":			(gc.getNumLeaderheadArtInfos,	gc.getLeaderheadArtInfo ),
	"TERRAIN_":			(gc.getNumTerrainArtInfos,		gc.getTerrainArtInfo ),
	"FEATURE_":			(gc.getNumFeatureArtInfos,		gc.getFeatureArtInfo ),
	}

def findArtInfo(art_tag , index = None):
	""" find image info structure for given art_tag.
	ART_DEF_ prefix may be omitted"""
	tag = art_tag.upper()

	if hasExtGC() :
		info = getExtGC().findArtInfo(art_tag)
		return info

	if index is None :
		for prefix in _ArtInfoMap:
			if tag.startswith(prefix) :
				tag = "ART_DEF_" + tag
				break
		index = gc.getInfoTypeForString(art_tag)

	if index < 0 :
		return None	
	for ( numInfo, getter) in _ArtInfoMap.values():
		info = matchInfo( index, numInfo(), art_tag, getter, True ) 
		if info :
			return info

	info = matchInfo( index, gc.getNumInterfaceArtInfos(), art_tag, gc.getInterfaceArtInfo, True ) 
	if info :
		return info

	info = matchInfo( index, gc.getNumMiscArtInfos(), art_tag, gc.getMiscArtInfo, True ) 
	if info :
		return info

	return None

def findButton(art_tag , notFound = QNOVALUE ):
	"""  get image button or path from various art/resource name string
	( Ex) "CIVILIZATION_AMERICA", "CIVIC_HEREDITARY_RULE" )
	using CyArtFileMgr().getXXXArtInfo.getbutton()/getpath() 
	or gc.getXXXInfo.getButton() (like gc.getReligionInfo/gc.getTechInfo).
	If it fails to get proper button, return dummy button (red X button) 
	or returns notFound arg if the arg given.
	Dont call me before XML is loaded and ArtFileMgr is ready to service. """
	
#	assert (art_tag and ArtFileMgr and not ArtFileMgr.isNone())
#	# make sure XML resources are loaded..
#	assert( ArtFileMgr.getInterfaceArtInfo("INTERFACE_BUTTONS_RED_X") )
	
	if art_tag.find("Art/") >= 0 : # art_tag seems to be the file path itself.
		return art_tag

	if hasExtGC() :
		button = getExtGC().findButton(art_tag)
		if button :
			return button
	else :
		index = gc.getInfoTypeForString(art_tag)
		if index >= 0 :
			info = matchInfo( index, gc.getNumInterfaceArtInfos(), art_tag, gc.getInterfaceArtInfo, True )
			if info :
				return info.getPath()

			info = matchInfo( index, gc.getNumMiscArtInfos(), art_tag, gc.getMiscArtInfo, True )
			if info :
				return info.getPath()

			info = findArtInfo(art_tag, index)
			if info :
				return info.getButton()

			info = findInfo(art_tag)
			if info :
				return info.getButton()

	# Not found....
	if notFound == QNOVALUE : # "Art/Interface/Screens/TechTree/X.dds"
		return ArtFileMgr.getInterfaceArtInfo("INTERFACE_BUTTONS_RED_X").getPath()
	else:
		return notFound

_imageSlotBegin = 0
_freeimageSlot = 0		# top of waterline
_imageSlotEnd = 0

def defineImagePath( art_tag, image_path, rebuild = True):
#	""" Define new image or redefine path of existing image.
#	art_tag is tag of new image to define or old image to redefine.
#	image_path is art path ( ex: "Art/Interface/TeamColor/FlagDECAL_Laurels.dds"
# 
#	Defining new image requires special "CIV4ArtDefines_Interface.xml"
#	After adding new tags, be sure to call CyArtFileMgr().buildArtFileInfoMaps().
#	""" 
	global _imageSlotBegin, _freeimageSlot, _imageSlotEnd
	assert art_tag and image_path
	
	art_tag = art_tag.upper()
	assert image_path.find("/") >= 0
	
	if hasExtGC() :
		ok = getExtGC().defineImagePath( art_tag, image_path, rebuild )
		return ok

	# No DLL version
	info = findArtInfo(art_tag)
	if not info :		# new tag / image
		if _freeimageSlot >= _imageSlotEnd :
			return None
		
		info = gc.getInterfaceArtInfo(_freeimageSlot) 
		if not info : 
			return -1
		info.setTag(art_tag)
		info.setPath(image_path)
		_freeimageSlot += 1
	else :
		info.setPath(image_path)
	if bRebuild :
		ArtFileMgr.buildArtFileInfoMaps()
	
	return True

# Path of selected mod in relative path like "Mods\<mod-name>\"
# or full path like "C:\Documents and Settings\<User>\My Documents\My Games\
# Sid Meier's Civilization 4\Mods\<mode-name>\"
# set by SDK DLL or CvSimCutieEventManager.EventHandlerSetup()
def getActiveMODPath(bFull = False):
	if hasExtGC() :
		if bFull :
			path = gc.getDefineSTRING("MOD_FULLPATH")
		else:
			path = gc.getDefineSTRING("MOD_PATH")
	else :
		# CAVEAT: This may work incorrectly some times without DLL support.
		#	If some python file in MODS folder has same file name with files 
		#	in (Custom)Assets/Python folder, Python import is confused with mod file 
		#	in MODS folder,then unselected mod is incorrectly reported as active MOD

		import CvEventInterface
		path = CvEventInterface.getEventManager().ActiveMODPath
		if path and not bFull :
			p1 =  path.lower().replace( "\\" , "/" ).find( "/mods/" )
			path = path[p1+1:]
	return path

def AssetsPath(arg = None):
	" Returns full path of CustomAssets or MODS/<mod-name>/assets folder."
	active_mod_path = getActiveMODPath(True)

	if arg and os.path.isabs(arg) and arg.lower().find("assets") > 0:
		return arg[0: arg.lower().find("assets") + 6 ]
	elif arg and  ( 0 <= arg.find("MODS") <= (len(arg) - 6 ) ):
		return path_join( CIV4_USERDIR, "MODS", arg[arg.find("MODS"):], "assets" )
	elif not arg and active_mod_path :	# more educated guess 
		return path_join(  active_mod_path, "assets" )
	else:
		return path_join( CIV4_USERDIR, "CustomAssets" )

############## FROM PEDIA HELP ###########################################
#====================================================================

_def_articles = [ "The " ]

def normalizeName(desc):
	" Remove definite article and map to ascii uppercase letter."
	global _def_articles
	for art in _def_articles :
		if desc.startswith(art[0]):
			key = desc[art[1]:] + art[2]
			break
	else:
		key = desc
	# Map letter to ascii uppercase.
#	normalized = toascii(key[0]).upper() + key[1:]
	normalized = toascii(key).upper().strip()
	return normalized

def getInfoList( numInfos, infogetter ):
	' returned a unsorted list of ( "description", info ID)'
	infoList = [ (infogetter(ii).getDescription(), ii)
		for ii in range( numInfos )
		if (not infogetter(ii).isGraphicalOnly()) ]
	return infoList

def sortNameList( alist ):
	" first element of each item is text desc used as sort key. Case ignored"
	return sort_mapped( alist, ( lambda x : normalizeName( x[0])))

class WonderClasses :
	NO_WONDERCLASS = 2
	WONDERCLASS_NATIONAL = 4
	WONDERCLASS_TEAM = 8
	WONDERCLASS_WORLD = 16

	WONDERCLASS_WONDER = WONDERCLASS_WORLD | WONDERCLASS_TEAM | WONDERCLASS_NATIONAL
	WONDERCLASS_ALL = WONDERCLASS_WONDER | NO_WONDERCLASS
	
def getBuildingWonderClass( eBuilding ):
	""" get Wonder Class for building.
	2: Regular building, 4: NationalWoner 8: TeamWonder, 16: WorldWonder """

	buildingClassType = gc.getBuildingInfo(int(eBuilding)).getBuildingClassType()
	wonder_calss = 0
	if isWorldWonderClass(buildingClassType):
		wonder_calss |= WonderClasses.WONDERCLASS_WORLD
	if isTeamWonderClass(buildingClassType):
		wonder_calss |= WonderClasses.WONDERCLASS_TEAM
	if isNationalWonderClass(buildingClassType):
		wonder_calss |= WonderClasses.WONDERCLASS_NATIONAL
	if wonder_calss == 0:
		wonder_calss = WonderClasses.NO_WONDERCLASS
	return wonder_calss
	
def getBuildingList( wonderClassMask = WonderClasses.WONDERCLASS_ALL ):
	""" returned unsorted list of ( "description", info ID) of buildings'
		wonderClasMask is bit-mask of class WonderClasses  or bool True/False"""
	if isBool(wonderClassMask) :
		wonderClassMask = ( wonderClassMask and WonderClasses.WONDERCLASS_WONDER 
							or WonderClasses.NO_WONDERCLASS )
	
	listBuildings = [ (gc.getBuildingInfo(iBuilding).getDescription(), iBuilding)
		for iBuilding in range(gc.getNumBuildingInfos())
		if ( wonderClassMask & getBuildingWonderClass(iBuilding)
			and not gc.getBuildingInfo(iBuilding).isGraphicalOnly()) ]
	return listBuildings

class UnitCategories :
	" Super set of UnitCombatTypes"
	NUM_UNITCATS = 0

#	UnitCategories.NO_UNITCAT
#	UnitCategories.UNITCAT_RECON
#	UnitCategories.UNITCAT_ARCHER
#	UnitCategories.UNITCAT_MOUNTED
#	UnitCategories.UNITCAT_MELEE
#	UnitCategories.UNITCAT_SIEGE
#	UnitCategories.UNITCAT_GUN
#	UnitCategories.UNITCAT_ARMOR
#	UnitCategories.UNITCAT_HELICOPTER
#	UnitCategories.UNITCAT_NAVAL
# -------------------------------------
#	UnitCategories.UNITCAT_AIR
#	UnitCategories.UNITCAT_WORKER
#	UnitCategories.UNITCAT_MISSIONARY
#	UnitCategories.UNITCAT_GREATPEOPLE
#	UnitCategories.UNITCAT_ANIMAL
#	UnitCategories.UNITCAT_SPECIAL
#	UnitCategories.NUM_UNITCATS	

def getUnitCategory( argUnit ):
	" Get UnitCategories. argUnit may be pCyUnit or iUnitType "

	assert UnitCategories.NUM_UNITCATS > 0, "UnitCategories not inited"
	if isinstance( argUnit, CyUnit ):
		info = gc.getUnitInfo( argUnit.getUnitType())
		cType = argUnit.getUnitCombatType()
	else :
		info =  gc.getUnitInfo( argUnit)
		cType = info.getUnitCombatType()
	
	if cType in range( gc.getNumUnitCombatInfos()) :
		uCat = cType
	elif info.isAnimal() :
		uCat = UnitCategories.UNITCAT_ANIMAL
	elif info.isGoldenAge() :
		uCat = UnitCategories.UNITCAT_GREATPEOPLE
	elif ( info.getPrereqReligion() >=0  
		and info.getReligionSpreads(info.getPrereqReligion())) :
		uCat = UnitCategories.UNITCAT_MISSIONARY
	elif info.getDomainType() == DomainTypes.DOMAIN_AIR :
		uCat = UnitCategories.UNITCAT_AIR
	elif info.getWorkRate() > 0 or info.isFoodProduction() :
		uCat = UnitCategories.UNITCAT_WORKER
	elif info.getSpecialUnitType() >= 0 : # != SpecialUnitCategories.NO_SPECIALUNIT
		uCat = UnitCategories.UNITCAT_SPECIAL
	else :
		assert False, "Uncategorized unit type"
		uCat = UnitCategories.NO_UNITCAT
	
	return uCat

#====================================================================
########## CIV4CCP EXT DLL I/F ###########################################

def hasExtGC():
	" Do I have SimCutie's Civ4CCP Extension DLL?"
	return bool(CyDLLGC)

def getExtGC():
	"Get Ext DLL global context"
	return CyDLLGC

def getExtDLLConf(key):
	"get Ext DLL configuration option value"
	return bool(gc.getDefineINT( "EXCF_" + key.upper() ))

def setExtDLLConf(key, bNewvalue, bSaveConf = False):
	"""set Ext DLL configuration option value.
		If bSaveConf is True, bNewvalue value will be saved. """
	gc.setDefineINT( "EXCF_" + key.upper(), int(bNewvalue))
	if bSaveConf :
		setCustomOption( EXTDLLCONF_SECTION, key.lower(), bool(bNewvalue))

def activateGameEntryPoint( entrypointName ):
	""" Activate CvGameIntarface entry point for '12Moneys' SDK DLL enttrypoint"
	See CvGameUtils.py for details on each entry points."""
	if not hasExtGC() :
		return False

	import CvGameInterface

	if entrypointName == 'doUnitCommand' :
		entrypointName = 'willUnitDoCommand'
	elif entrypointName == 'unitUpgradePrice' :
		entrypointName = 'calculateUnitUpgradePrice'
	elif entrypointName == 'spreadReligionToCity' :
		entrypointName = 'willSpreadReligionToCity'
	elif entrypointName == 'razeCity' :
		entrypointName = 'willPlayerRazeCity'
	elif entrypointName == 'playerFoundReligion' :
		entrypointName = 'willPlayerFoundReligion'
	elif entrypointName == 'groupStartMission' :
		entrypointName = 'willGroupStartMission'

	elif entrypointName == 'new' :
		CvGameInterface.gameUtils().OldInterface(False);
		return True
	elif entrypointName == 'old' :
		CvGameInterface.gameUtils().OldInterface(True);
		return True

	if hasattr(CvGameInterface, entrypointName ) :
		return getExtGC().activateGameEntryPoint(entrypointName)
	else :
		return False

############## MISC GAME UTILS #############################################

def isActiveHumanPlayer(iPlayer):
	" is the integer iPlayer currently active, alive, human player? "
	pPlayer = gc.getActivePlayer()
	return iPlayer ==  pPlayer.getID() and pPlayer.isHuman() and pPlayer.isAlive()

def isCivPlayer(iPlayer):
	"is Live and non-babarian Civ player ?"
	pPlayer = gc.getPlayer(iPlayer)
	return ( not pPlayer.isNone() and pPlayer.isAlive() 
			and not pPlayer.isBarbarian() and not pPlayer.isMinorCiv() )

def getPlayerList():
	""" returns list of current players as list of tuple ( 'leader_name', player_id )
	Dead, minor, barbarian civs are excluded """
	plist = [ ( gc.getPlayer(iPlayer).getName(), iPlayer )
				for iPlayer in range(gc.getMAX_PLAYERS())
				if isCivPlayer(iPlayer) ]
	return plist	

def isHasMetPlayer(iPlayer, jPlayer):
	" Have the iPlayer and jplayer met each other before?" 
	return gc.getTeam(gc.getPlayer(iPlayer).getTeam()).isHasMet(
		gc.getPlayer(jPlayer).getTeam())

#def getActivePlayer():
#	"get Active Player ID"
#	return Game.getActivePlayer()

#def getCyTeam(iPlayer):
#	return gc.getTeam(gc.getPlayer(iPlayer).getTeam())

def getCyPlayerTeam(iPlayer):
	" returns CyPlayer, CyTeam of eplayer"
	pPlayer = gc.getPlayer(iPlayer)
	pTeam =  gc.getTeam(pPlayer.getTeam())
	return ( pPlayer, pTeam )

def getGameTurn():
	" get Current game turn"
	return Game.getGameTurn()

def getGameYear( turn ):
	"Convert game turn to years number, Return negative number for BC."
	timeStr = CyGameTextMgr().getTimeStr(turn, false).split(' ', 1)
	years   = int(timeStr[0])
	if timeStr[1] in ('BC', 'av. J-C.', 'v. Chr.', 'a. C.' ):
		years = - years
	return years

def getXY(pEntity):
	" return tuple of (X, Y) coodinate." 
	return ( pEntity.getX(), pEntity.getY())

def isEqualXY( pEntity1, pEntity2 ):
	" is enity 1 is same plot posistion with entity 2?"
	return ( pEntity1.getX() == pEntity2.getX() 
		and pEntity1.getY() == pEntity2.getY())

def distanceTo( pPlot, qPlot) :
	" get plot to plot distance in plot cood unit (not step distance)."
	assert pPlot and qPlot and not pPlot.isNone() and not qPlot.isNone()
	return plotDistance(pPlot.getX(), pPlot.getY(), qPlot.getX(), qPlot.getY())

def isEqualID(pEntity1, pEntity2):
	""" check equality of  enity_1 is to entity_2. (like City, Unit, ...)
	Be careful when comparing Units during/through upgrade. """
	return ( pEntity1.getOwner() == pEntity2.getOwner() 
			and pEntity1.getID() == pEntity2.getID())

def getIDInfo( arg1, entityID = None):
	""" get IDInfo(eOwner, iID ). 
	Arg1 is entity like pUnit, pCity or tuple like (eOwner, entityID)."""

	entityIDInfo = IDInfo()
	if entityID is not None :
		assert isInt(arg1) and isInt(entityID)
		entityIDInfo.eOwner = PlayerTypes(arg1)
		entityIDInfo.iID = entityID
	elif isTuple( arg1 ) and len(arg1) == 2 :
		entityIDInfo.eOwner = arg1[0]
		entityIDInfo.iID = arg1[1]
	elif hasattr( arg1, "getID" ) :
		entityIDInfo.eOwner = PlayerTypes(arg1.getOwner())
		entityIDInfo.iID = pEntity.getID()
	else :
		raise TypeError
	return entityIDInfo

def parseIDInfo( arg1, entityID = None ):
	" args may be (eOwner, entytyID), IDInfo, tuple(eOwner, entytyID)"

	if entytyID is not None :				# arg1 =iPlayer, arg2 = entityID
		eOwner = arg1
	elif isinstance(arg1, IDInfo) :			# arg1 = IDInfo
		( eOwner, entityID ) = ( arg1.eOwner, arg1.iID )
	elif isTuple(arg1) and len(arg1) == 2 :	# arg1 = tuple(iPlayer, unitID)
		( eOwner, entityID ) = arg1
	else :
		raise TypeError
	assert isInt(eOwner) and isInt(entityID)
	return ( eOwner, entityID )

def getUnit(arg1, arg2 = None):
	" args may be (eOwner, unitID), IDInfo, tuple(eOwner, unitID) or UID"
	if isinstance(arg1, IDInfo) :			# arg1 = IDInfo
		return cyGetUnit(arg1)
	elif arg2 is None and isInt(arg1) :		# UID
		return getUnitbyUID(arg1)
	
	( iPlayer, unitID) = parseIDInfo( arg1, arg2 )	
	return gc.getPlayer(iPlayer).getUnit(unitID)

def getCity(arg1, arg2 = None):
	" args may be (eOwner, cityID), IDInfo, tuple(eOwner, cityID)"
	if isinstance(arg1, IDInfo) :			# arg1 = IDInfo
		return cyGeCity(arg1)
	( iPlayer, cityID) = parseIDInfo( arg1, arg2 )
	return gc.getPlayer(iPlayer).getCity(cityID)

def getPlayerUnitList( iPlayer = None ):
	" get list of all Unit belongs to a player, default is current active player."
	if iPlayer == None :
		iPlayer = Game.getActivePlayer()
	pyPlayer = PyPlayer(iPlayer)
	if pyPlayer :
		return pyPlayer.getUnitList()
	else :
		return []

def getPlayerCityList( iPlayer = None ):
	" get list of all City belongs to a player, default is current active player."
	if iPlayer == None :
		iPlayer = Game.getActivePlayer()
	pyPlayer = PyPlayer(iPlayer)
	if pyPlayer :
		return pyPlayer.getCityList()
	else :
		return []

#	enum TradeableItems:
#	TRADE_GOLD,		TRADE_GOLD_PER_TURN,	TRADE_MAPS,		TRADE_OPEN_BORDERS,
#	TRADE_DEFENSIVE_PACT,	TRADE_PERMANENT_ALLIANCE,		TRADE_PEACE_TREATY,
#
#	TRADE_TECHNOLOGIES,	TRADE_RESOURCES,	TRADE_CITIES,	TRADE_PEACE,
#	TRADE_WAR,	TRADE_EMBARGO,	TRADE_CIVIC,	TRADE_RELIGION,

def checkTradeableItem(iPlayer, iTargetPlayer, tradeType, tradeItem = -1 ):
	""" Check iPlayer can trade with iTargetPlayer or not.
	tradeType is TradeableItems type like TradeableItems.TRADE_TECHNOLOGIES,
	tradeItem is index (ID) in the item types like 1 for TECH_MEDITATION """
 	pPlayer = gc.getPlayer(iPlayer)
	tradeData = TradeData()
	tradeData.ItemType = tradeType
	if tradeItem >= 0 :
		tradeData.iData = tradeItem
	return ( pPlayer.canTradeItem( iTargetPlayer, tradeData, False) 
		and pPlayer.getTradeDenial(iTargetPlayer, tradeData) == DenialTypes.NO_DENIAL )

############## DATA STORE UTILS ##########################################

# CustomProfile USAGE SUMMARY: 
#	See CvCustomOption.py for usage example 
#	See "PyDataStore.py" ConfigMgr class for detail
#	from PyCivSim import useOptionSection, getCustomOption, setCustomOption
#	useOptionSection ( szSection )
#		-- Create or load the section if not yet created or loaded
#	getCustomOption ( szSection, szOptionName, [default = ""] )
#		-- get option value. Return value is String, bool or int/float. 
#		-- If option is not set but default value if given, set it as new value.
#	setCustomOption ( szSection, szOptionName, value)

try :
	import CvCustomOption
	getCustomOption = CvCustomOption.CustomProfile.getOption
	setCustomOption = CvCustomOption.CustomProfile.setOption
	useOptionSection	= CvCustomOption.CustomProfile.useSection
except ImportError:
	pass

import cPickle as pickle

GameStoreDict = {}	# dict of stores to auto-save/load

def readScriptData( civObj, dataKey, default = None ):
	try:
		cyTable = pickle.loads( civObj.getScriptData() )
		return cyTable[ dataKey ]
	except :
		return default

def writeScriptData( civObj, dataKey, data):
	try:
		cyTable = pickle.loads( civObj.getScriptData() )
	except:
		cyTable = dict()
	cyTable[ dataKey ] = data
	civObj.setScriptData( pickle.dumps(cyTable) )

def deleteScriptData( civObj, dataKey):
	try:
		cyTable = pickle.loads( civObj.getScriptData() )
		del cyTable[ dataKey ]
		civObj.setScriptData( pickle.dumps(cyTable) )
	except:
		pass

def getObjVar( civObj, groupName, itemName, default = None ):
	return readScriptData(civObj, groupName + "." + itemName, default = None )

def setObjVar( civObj, groupName, itemName, data):
	writeScriptData( civObj, groupName + "." + itemName, data)

def deleteObjVars(civObj, groupName):
	try:
		cyTable = pickle.loads( civObj.getScriptData() )
		for key in cyTable.keys():
			if key.startswith( groupName + "." ) or groupName == "*" :
				del cyTable[ key  ]
		civObj.setScriptData( pickle.dumps(cyTable) )
	except:
		pass

def addupObjVar( civObj, groupName, itemName, addrand):
	""" same as setObjVar( getObjVar(..) + addrand)
	group, item should be existing variable.
	addrand can be any data type with valid "+" operator 
	like number, string or list """
	dataKey = groupName + "." + itemName
	cyTable = pickle.loads( civObj.getScriptData() )
	value = cyTable[ dataKey ] + addrand
	cyTable[ dataKey ] = value 
	civObj.setScriptData( pickle.dumps(cyTable) )
	return value

import PyDataStore

def CvStoreCreate(szStoreName, bPersistent = False):
	""" create data dictionary to store mod data into game save/load file.
	Use your mod name as Store name key. See PyDataStore.py for details. """

	if bPersistent :
		if szStoreName in GameStoreDict:
			storeObj = GameStoreDict[szStoreName] 
		else:
			storeObj = PyDataStore.PyStore(szStoreName)
			GameStoreDict[szStoreName] = storeObj
	else:
		storeObj = PyDataStore.PyStore(szStoreName)
		storeObj.reader = (lambda stroreName : 
			readScriptData( Game, stroreName ))
		storeObj.writer = (lambda stroreName, data : 
			writeScriptData( Game, stroreName, data))
	return storeObj

#-------------------------------------------------------------------------
# NOTIFY_CLICKED => 0
# NOTIFY_DBL_CLICKED => 1
# NOTIFY_FOCUS => 2
# NOTIFY_UNFOCUS => 3
# NOTIFY_CURSOR_MOVE_ON => 4
# NOTIFY_CURSOR_MOVE_OFF => 5
# NOTIFY_CHARACTER => 6
# NOTIFY_SCROLL_UP => 7
# NOTIFY_SCROLL_DOWN => 8
# NOTIFY_NEW_HORIZONTAL_STOP => 9
# NOTIFY_NEW_VERTICAL_STOP => 10
# NOTIFY_LISTBOX_ITEM_SELECTED => 11
# NOTIFY_FLYOUT_ITEM_SELECTED => 12
# NOTIFY_MOUSEMOVE => 12		<=== ??????????????
#
# NOTIFY_MOUSEWHEELUP => 14
# NOTIFY_MOUSEWHEELDOWN => 15
#
#
# NOTIFY_LINKEXECUTE => 18
# NOTIFY_MOVIE_DONE => 19
# NOTIFY_SLIDER_NEWSTOP => 20
# NOTIFY_TABLE_HEADER_SELECTED => 21
#
############## ADD MORE HERE #############################################
