# $Source: /cvsroot/civ4geneticai/Assets/Python/CUSTOM/PyDataStore.py,v $: $Name:  $
#	Python Libray for game data stroge and access.
#	$Copyright$
#	by SimCutie < simcutie (AT) hanmail.net >
#	Module to store / load User Config/Profile data, and Entity dictionary data. 
#	See "CvLibTest.py" and "PyCivSim.py"to test this module and usage example.
#	This file should reside in  "... /CustomAssets/python/custom" folder.
#	Version: $Revision: 1.1 $ $Date: 2006/09/12 12:43:11 $
#
__author__ = "$Author: the_great_apple $"
__revision__ = "$Id: PyDataStore.py,v 1.1 2006/09/12 12:43:11 the_great_apple Exp $"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2006/09/12 12:43:11 $"

####################### CUSTOM PROFILE ####################################
#	class ConfigMgr to load/saves user options in ".ini" file.
#	See "CvCustomOption.py" for more details.
from ConfigParser import SafeConfigParser
import re
from PyBaseLib import ( isInt, isString, isTuple, isDict, isClassType, isident,
	isInstanceType, const_exp, qlen, QNOVALUE )
from PyExtLib import path_isfile, randomHex, simpleConstValue, attr_dict

class ConfigMgr(SafeConfigParser):
	""" Read and writes custom config in "ini" file.
	For more details, See Python ConfigParser lib. module.
	Old bug of stripping comments is fixed """

	def resetConfig(self, szConfig = None):
		" reset me for new user config or use current config"
		self.myFileName = ""
		if szConfig :
			self.myConfig = szConfig
		else:
			self.myConfig = self.getCurrentConfigName()
		
		if self.myConfig :
			self.myFileName = self.getConfigFileName(self.myConfig)
		self.fileLoaded = False
		
	def readFile (self):
		" read current config file."
		self.OrigLines = []
		if self.myFileName and path_isfile(self.myFileName):
			fo = file(self.myFileName, "r")		# may generate IOError..
			if fo:
				self.OrigLines = fo.readlines()
				fo.seek(0)
				self.readfp(fo, self.myFileName)
				self.fileLoaded = True
	
	def useSection(self, szSection):
		""" Load file and check section name. If the section does not exist, create it. 
		Call me before get or set option, to ensure existance of the section """
		
		assert szSection
		szSection = szSection.upper()
		
		if not self.fileLoaded :
			self.readFile()
		if not self.has_section(szSection):
			self.add_section(szSection)
			
	def setOption(self, szSection, szOptionName, value):
		" Set option value. The section must be exist before set."
		assert szSection and szOptionName
		szSection = szSection.upper()
		if self.has_section(szSection) and value != None :
			self.set(szSection, szOptionName, str(value)) 
	
	def getOption(self, szSection, szOptionName, default = QNOVALUE ):
		""" Retreives option value. The section must exist before getting value.
		If the section or option is not found or at any error, 
		it returns "" (empty string) or default arg value if it is given.
		If given option does not exist, but default value is given,
		new option is created and set to given default value.
		Return value is string type by default. 
		But if it is "True/false/Yes/No", type is bool. 
		If value is valid numeric string, return type is int or float.
		For integer/ hex number, ("1234", "0x34Fa"), type is int.
		For floating point number ("23.45", "3.14e6" ), type is float. """
		
		# If option doesn't exist, but no default value is given, return "" 
		if default != QNOVALUE :
			defaultValue = default
		else :
			defaultValue = ""
		
		assert szOptionName and szSection
		szSection = szSection.upper()
		if not self.has_section(szSection) :
			# qPrint(__name__ + ": No such section. Did you loaded config?", szSection )
			return defaultValue

		if not self.has_option(szSection, szOptionName):
			if default != QNOVALUE :
				self.set(szSection, szOptionName, str(defaultValue))
			return defaultValue
		
		optvalue = self.get(szSection, szOptionName)
		
		if optvalue == None :
			return defaultValue
		eValue = simpleConstValue(optvalue)
		if eValue != None:
			return eValue
		else:
			return optvalue
	
	def write(self, fp):
		"""Write an .ini-format representation of the configuration state."""

		def flushSection(section):
			# dump remaining data
			if attrDict :
				for pair in attrDict.items() :
					fp.write( "%s = %s\n" % pair )
			sectionList.remove(section)
		
		sectionList = self.sections()
		sectionList.append("DEFAULT")
		sectionName = "DEFAULT"
		attrDict = dict(self.items(sectionName))
		for line in self.OrigLines:
			if re.match( r"\s*[#;]" , line ):	# empty or comment line 
				fp.write(line)
				continue 
			mo =  re.match( r"\s*\[([\w_.-]+)]", line )
			if mo :			#	new section name
				flushSection(sectionName)
				sectionName = mo.group(1)
				attrDict = dict(self.items(sectionName))
				fp.write(line)
				continue
			mo = re.match( r"\s*([\w_.-]+( [\w_.-]+)*)\s*=", line )
			if mo :			# option = value ....
				attr = mo.group(1)
				key = attr.lower()
				value = attrDict.get(key)
				if value != None :
					fp.write( "%s = %s\n" % (attr, value ))
					del attrDict[key]
				else :		# deleted value ?
					pass
			else :
				fp.write(line)
		
		flushSection(sectionName)
		for sectionName in sectionList :
			fp.write( "[%s]\n" % sectionName )
			for pair in self.items(sectionName) :
				fp.write( "%s = %s\n" % pair )
			
		fp.close()

	def saveConfig(self):
		" Save current profile blindly."
		if self.myFileName :
			self.write(file(self.myFileName, "w"))
	
	def reloadConfig(self, szConfig = None):
		" Reload named or current custom profile"
		self.resetConfig(szConfig)
		self.readFile()
	
#	*TODO*
	def deleteConfig(szConfig):
		# call me before calling UserProfile.loadProfileFileNames()
		pass

##	Following two methods are  pure virtual method, so class user must supply it
##	( by setattr/method assignnment or subclass overriding )

#	def getCurrentConfigName():
#		" return current config name string"
#		return ""
#
#	def getConfigFileName(configName):
#		"return full path of the'.ini' file for given config name"  
#		return ""

############## CUSTOM DATA ################################################

# import re
import cPickle as pickle

#_re_varName = re.compile( r"^[_A-Za-z][_A-Za-z0-9]*$")
#
#def checkVarName(varName):
#	" Check vality of variable identifier ([_A-Za-z_][_A-Za-z0-9]*)"
#	assert varName and isString(varName)
#	return bool( _re_varName.match(varName))

"""
This service will let moder can save/load game related data into game save file using get/setScriptData(). Similar to old SD toolkit, but it is much fast and easier to use.
This service do not require SDK DLL.
Library used: PyBaseLib.py, PyExtLib.py, PyDataStore.py, PyCivSim.py

-   It does not retreive/save data from ScriptData whenever you change data value.
    It is stored/retreived to/from script data automatically on game load/save time only.
    So it is much more fast than sdToolkit. 
-   It uses much friendier syntax of python attribute value access or dictionary value acess.
    Old sdToolkit example) 
	sdEntityInit( 'MY_MOD', 'Unit1', { 'killCounter': 0} )
	val = sdGetVal( 'MY_MOD', 'Unit1', 'killCounter' )
	val = val +1
	sdSetVal( 'MY_MOD', 'Unit1', 'killCounter', val )

    New example)
	myDB = PyCivSim.CvStoreCreate('MY_MOD', True)
	myDB.addEnt('Unit1', { 'killCounter': 0})
	myDB.Unit1.killCounter += 1

    alternative A)
	Unit1 = myDB.addEnt('Unit1', { 'killCounter': 0})
	Unit1.killCounter += 1
    alternative B)
	Unit1['killCounter'] += 1

More details later. See "PyDataStore.py" for details.
"""
#-------------------------------------------------------------------------

class PyEntity(attr_dict):
	""" dictionary of values with class syntax form.
	See "CvLibTest.py" for usage example."""

	__slots__ = [ "__name__" , "_entityId" ]

	def __init__( self, szEntityName, obj = {} ):

		attr_dict.__init__(self, obj)
		
		self._entityId =  randomHex(16)
		if  szEntityName :
			self.__name__ = str(szEntityName)
		elif hasattr(obj, "__name__"):
			self.__name__ = obj.__name__
		else :
			self.__name__ = "_Unnamed"
	
	def __getstate__ ( self ):
		return  str( { "__name__" : self.__name__, "_entityId" : self._entityId }) 

	def __str__ (self):
		return "{ '%s', '%s', %s }" % ( self.__name__, self._entityId, attr_dict.__str__(self))
	
	# Syntactic salt...
	def hasVar( self, varName ):
		" Have I the variable?"
		return varName in self
	
	def getVar( self,  varName, default = QNOVALUE):
			return self[ varName, default ]
	
	def setVar( self, varName, value = QNOVALUE ):
		"set value of variable" 
		if isident(varName):
			self[ varName ] = value
		else:
			raise NameError(varName)
	
	def delVar( self, varName ):
		"delete existing variable "
		if varName in self:
			del self[ varName ]
	
	def defVar( self,  varName, default = QNOVALUE):
		""" get the varible. If the variable is not yet set, 
		set it with given default value """
		if varName in self:
			return self[ varName ]
		elif default != QNOVALUE:
			self[ varName ] = default
			return default
	
#	def copy( self):
#		" Copy me (shallow copy) including 
#		return dict.copy()
#
#=========================================================================

class PyStore(attr_dict):
	" Dictionary of PyEntity's with save / load method "

	__slots__ = [ "reader", "writer", "__name__" ]

	def __init__ ( self, szStoreName ):
		szStoreName = szStoreName.upper()
		if not isident( szStoreName ):
			raise ValueError(szStoreName)
		self.__name__ = szStoreName
		self["_"] = PyEntity("global", {})	# global dict.
	
	@staticmethod
	def getNamefromArgs(args):
		if isTuple(args):
			name = args[0]
		else :
			name = args
		if isInt(name):
			return "_INT_" + str(name)
		return name
	
	def __getitem__( self, args):

		entName = self.getNamefromArgs(args)
		if not isTuple(args):
			return dict.__getitem__( self, entName )

		ac = len(args) 
		if ac == 2 :
			return dict.__getitem__( self, entName )[ args[1] ]
		elif ac == 3:
			return dict.__getitem__( self, entName )[ args[1], args[2] ]
		else:
			return dict.__getitem__( self, entName )
	
	def __setitem__( self, args, value ):

		entName = self.getNamefromArgs(args)
		if  isTuple( args ) and len(args) == 2 :	# db [ ent, var] = value form.
			dict.__getitem__( self, entName)[ args[1]] = value 
			 
		else :										# db [ ent ] = value form
			if isinstance(value, PyEntity) or args in self.__slots__ :
				entValue = value
			else:
				entValue = PyEntity(args, value)

			dict.__setitem__(self, entName, entValue )

	def __getstate__ ( self ):
		return  str(  self.__name__ ) 

	def __setstate__ ( self, state ):
		self.__init__( state )

	def addEnt(self, entName, obj = {} ):
		""" Add new entity. ValueError is raised 
			if entName is not valid name."""
		if isident(entName) and entName not in self:
			ent = PyEntity(entName, obj)
			dict.__setitem__(self, entName, ent )
			return ent
		else:
			raise ValueError(entName)
	
	def hasEnt(self, entName ):
		return entName in self
	
	def getEnt(self, entName ):
		return self[ entName ]
	
	def getVar(self, entName, varName, default = None ):
		"get value of variable from existing entity"
		assert isString(varName)
		try:			
			return self[ entName ][ varName, default ]
		except KeyError:
			pass
		return default
	
	def setEnt(self, entName, entValue ):
		" Set existing entity, not new entity"
		if  ( isident(entName) and entName in self
			and isinstance( entValue, dict) ):
			self[ entName ] = entValue
		else:
			raise ValueError(entName)
	
	def setVar(self, entName, varName, value ):
		# set value of variable of the existing dict.
		# variable can be created but entity must exist.
		if entName in self :
			self[ entName ][ varName ] = value
		else:
			raise ValueError(entName)
	
	def delEnt(self, entName ):
		"delete existing dict"
		if entName in self:
			del self[ entName ]
	
#	def CopyEnt(self, szDestEnt, szSrcEnt):
#		if szSrcEnt in self:
#			if szDestEnt not in self:
#				self.addEnt(szDestEnt)
#			self[szDestEnt] = self[ szSrcEnt ]
#		else:
#			raise AttributeError
	
	def load(self):
		serializedStr = self.reader( self.__name__ )
		data = pickle.loads(serializedStr)
		dict.__init__(self, data)
	
	def save(self):
		self.writer( self.__name__ , pickle.dumps(self))
	
#	Following two methods are virtual method, so class user must supply it
#	( by setattr/method assignnment or subclass overriding )
#	This is just for testing
#	
#	def reader(self, storeName ):
#		print "called reader", storeName
#		global _testStore
#		return  _testStore[storeName]
#	
#	def writer(self, storeName, data):
#		global _testStore
#		print "called writer", storeName
#		_testStore [storeName] = data
#		return True

#-------------------------------------------------------------------------
############################# XMLItems ###################################

import warnings
warnings.filterwarnings("ignore", "The xmllib module is obsolete.", DeprecationWarning, "xmllib")
# Yes, but the new xml.sax is not included in Civ4. So I have to use it.

from xmllib import XMLParser, Error as xmllibError

class XMLItems(XMLParser):

	Error = xmllibError

	# virtual function, This handler should be overided by subclass
	def handle_item(self, item_id, item_title, item_text, item_extra ):
		"We got an item, precess it"
		print "<<<id=[%s], title='%s', extra='%s'>>>" % (item_id, item_title, item_extra)
		print item_text

	def __init__(self, handle_item):
		XMLParser.__init__(self)
		self.data = self.tag = ""
		self.handle_item = handle_item

	def unknown_starttag(self, tag, attrs):
		if tag in ( "ITEMS", "ITEM", "ID", "TITLE", "EXTRA", "TEXT" ) :
			self.tag = tag
			if tag == "ITEM" :
				self.reset_data()
			elif tag == "TEXT" :
				self.setliteral() 

	def handle_data(self, data):
		if self.tag:
			self.data += data.strip()

	def unknown_endtag(self, tag):

		if tag == "ID" :
			self.item_id = self.data
		elif tag == "TITLE" :
			self.item_title = self.unescape(self.data)
		elif tag == "EXTRA" :
			self.item_extra = self.unescape(self.data)
		elif tag == "TEXT" :
			self.item_text = self.unescape(self.data)
		elif tag == "ITEM" :
			if self.item_id and self.item_title and self.item_text :
				if self.item_extra :
					extra = self.item_extra 
				else :
					extra = self.extra_default
				self.handle_item( self, self.item_id, self.item_title, self.item_text, extra )

			self.reset_data()
		elif tag == "ITEMS" :
			pass
		else:
			pass
		self.data = self.tag = ""
	
#	def syntax_error(self, message):
#		raise Error('Syntax error at line %d: %s' % (self.lineno, message))

	def reset_data(self):
		self.item_id = self.item_title = self.item_extra = self.item_text = ""
	
	@staticmethod
	def unescape(s):
		# Civ4 post processing eat up '<' and '[', unescape '@{'and '@[' "
		s = s.replace("@{", " &#60;")
		s = s.replace("@[", "&#91;")
		s = s.replace("@_", "&#32;")
		return s

