# $Source: /cvsroot/civ4geneticai/Assets/Python/CUSTOM/PyExtLib.py,v $: $Name:  $
#	Python Extended Libray for Sid Meier's Civilization 4 Custom Mods.
#	$CopyRight$
#	by SimCutie < simcutie (AT) hanmail.net >
#	This is general python programing libray that does not depend on CIV4 game.
#	See "CvLibTest.py" to test this module and usage example.
#	This file should reside in  "... /CustomAssets/python/custom" folder.
#	Version: $Revision: 1.1 $ $Date: 2006/09/12 12:43:11 $
#
__author__ = "$Author: the_great_apple $"
__revision__ = "$Id: PyExtLib.py,v 1.1 2006/09/12 12:43:11 the_great_apple Exp $"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2006/09/12 12:43:11 $"

import re, random
from PyBaseLib import (isString, isDict, isTuple, isContainer, isSeq, isInstanceType, 
		isident, className, qnumber, const_exp, QNOVALUE )

######### C/C++ LIKE STRING HANDLING #####################################

_esc_map = { " ": " " , "\\" : "\\\\", "\n" : "\\n", "\r" : "\\r", "\t": "\\t", 
			"\f": "\\f", "\e" :  "\\e", }
def escape_special(src, passchrs ="" ):
	""" escape all non-alphanumeric, non-white special chars except given in 
	passchrs string. passchrs can include backslash char. ("\")
	But it may be bad idea and you must know what you are doing. """
	assert isString(src)
	outstr = ""
	for ch in src:
		if ch.isalnum() or ch in passchrs:
			outstr += ch
		elif ch in _esc_map:
			outstr += _esc_map[ch]
		elif 0x00 <= ord(ch) <= 0x1f :
			outstr += "\\x%02X" % ord(ch)
		else:
			outstr += "\\" + ch 
	
	return outstr

def respn( src, spnchrs ):
	""" get length of span, in begining of src string, which consist 
	entirely of chars from (or not from) the spnchrs string. 
	Similar to strspn() and strcspn() of "C" language library. 
	Returns length of matched span. (src[0:n]) Returns 0 if not found.
	Span chars is like char. range pattern of regular expresson([...]).
	Ex: "abcd" "a-zXYZ", "0-9.+-", "\\/.*?"(  backslash is not special )
	To find span consits of chars NOT from the spnchrs like strcspn(), 
	prepend '^' in begining of spnchars arg ( like "^A-Z" ) """
	
	assert isString(src)
	#  escape all special char except  '^' and '-'
	patt = re.escape( spnchrs ).replace("\\^", "^").replace("\\-", "-")
	patt = "([" + patt + "]*)"
	mo = re.match( patt , src )
	return mo.end(1)

def	rebrk( src, brkchrs = " " ):
	""" Find first occurence, in src string, of any char. from 
	( or not from) brkchrs string. Similar to strpbrk() of "C" language.
	Returns index of first found brk char. or -1 if not found.
	For brkchrs string format, see spnchrs range pattern ([...]) of respn(). """
	
	assert isString(src)
	#  escape all special char except  '^' and '-'
	patt = re.escape( brkchrs ).replace("\\^", "^").replace("\\-", "-")
	patt = "([" + patt + "])"
	mo = re.search( patt , src )
	if mo:
		return mo.start(1)
	else:
		return -1

def resplit( src, sepchrs = " ", empty_tok = False, maxsplit = 0 ):
	""" Find tokens delimeted by any char. from sepchrs.
	Unlike string.split(), separator is set of characters, not substring.
	Returns split list of tokens. If empty_tok arg is <True>, empty token ('') 
	between two successive separator will be added, too.
	For sepchrs string format, see spnchrs range pattern ([...]) of respn()."""
	
	assert isString(src)
	#  escape all special char except  '^' and '-'
	patt = re.escape( sepchrs ).replace("\\^", "^").replace("\\-", "-")
	if empty_tok : 
		patt = "[" + patt + "]"
	else:
		patt = "[" + patt + "]+"
	return re.split( patt , src, maxsplit )

def reextract( src, prefix, target = ".*?", suffix = "" ):
	" Extract pattern between two patterns. Returns None if not found"
	assert isString(src)
	patt = "%s*(?P<_EX_>%s)%s" % ( prefix, target, suffix )
	mo = re.search( patt, src )
	if mo :
		return mo.group("_EX_")
	else :
		return None

def splitwords(src, maxsplit = 0):
	"""split string to list of words enreirely consisted with
	Ascii/Latin-1 alphabets + number letters + '_' only """
	resplit(src, "^A-Za-z0-9_\xc0-\xff", False, maxsplit )

def strwrap(text, width = 80):
	""" A word-wrap function that preserves existing line breaks and most spaces 
	in the text. Expects that existing line breaks are posix newlines (\n). """
	return reduce(lambda line, word, width=width: '%s%s%s' % (line, 
		' \n'[(len(line)-line.rfind('\n')-1 + len(word.split('\n',1)[0] ) >= width)],
		word), text.split(' '))

def formatExtent( before, sLabel, sValue, delim1=u"", delim2=u"", delim3= u""):
	""" Returns concated string of delim1, sLabel, delim2, sValue delim3 
	if and only if sValue is non-empty. if <before> is empty, omit delim1, too. """
	szBuf = ""
	if sValue :
		if before :
			szBuf +=  delim1
		szBuf += sLabel + delim2 + sValue + delim3
	return szBuf

def formatListItem( header, item, separator, bFirst):
	""" Used to build string list of items like "header item1, item2, ..., items" """
	if bFirst :
		return header + item
	else :
		return separator + item

def mixed2Under(x):
	" mixed case/capwords to lowercase underscore notation"
	return re.sub(r'(?<=[a-z])[A-Z]|(?<!^)[A-Z](?=[a-z])', r"_\g<0>", x).lower()

def mixed2Upper(x): 
	" mixed case/capwords case to upper case underscore notation"
	return mixed2Under(x).upper()

def under2Mixed(x): 
	" underscore to mixed case notation"
	return re.sub(r'_([a-z])', lambda m: (m.group(1).upper()), x.lower())

def under2Cap(x): 
	" underscore to capwords notation"
	s = under2Mixed(x)
	return s[0].upper()+s[1:]

def randomHex( numhexdigits = 16):
	"""Return a string representing random hexa digits. ( ~1024 bits = 256 digits )"""
	# create a numhexdigits digit string from random source
	reg = 0L
	for i in range(0, numhexdigits):
		reg = long(random.randint(0, 15)) + reg << 4

	# removes leading '0x' and trailing 'L'
	rnd_hex = hex(reg)[2:-1]
	return(rnd_hex)

import string

class Translator:
	"""
This class handles the three most common cases where I find myself having to stop and think about how to use translate:

1) Keeping only a given set of characters.
>>> trans = Translator(keep=string.digits)
>>> trans('Chris Perkins : 224-7992')
'2247992'

2) Deleting a given set of characters.
>>> trans = Translator(delete=string.digits)
>>> trans('Chris Perkins : 224-7992')
'Chris Perkins : -'

3) Replacing a set of characters with a single character.
>>> trans = Translator(string.digits, '#')
>>> trans('Chris Perkins : 224-7992')
'Chris Perkins : ###-####'

This last one is admittedly a bit of a special case, but it is something I seem to need to do once in a while.

Note that the delete parameter trumps the keep parameter if they overlap:
>>> trans = Translator(delete='abcd', keep='cdef')
>>> trans('abcdef')
'ef'
This is an arbitrary design choice - makes as much sense as anything, I suppose.
"""
	allchars = string.maketrans('','')
	def __init__(self, frm='', to='', delete='', keep=None):
		if len(to) == 1:
			to = to * len(frm)
		self.trans = string.maketrans(frm, to)
		if keep is None:
			self.delete = delete
		else:
			self.delete = self.allchars.translate(self.allchars, keep.translate(self.allchars, delete))
	def __call__(self, s):
		return s.translate(self.trans, self.delete)

#def addArticle( name ):
#		if ( name[0].lower() in "aeiou" ):
#			return "an " + name
#		else:
#			return "a " + name

def simpleConstValue(arg):
	""" string to simple constant value. 
	Only number(int, float), bool(false/true/yes/No), None will be regognized 
	Else every other thing is treated as string """
	value = qnumber(arg, None)
	if value != None :
		return value
	strarg = arg.lower().strip()
	if strarg in ( "true",  "yes" ):
		return True
	elif strarg in ( "false",  "no" ):
		return False
	elif strarg == "none":
		return None
	else:
		return arg

##_re_csv = re.compile( r"^[\w_+-.=/]+(,[\w_+-.=/]+)+$" )	
##	arg.find(",") > 0 and _re_csv.match(arg) :
##		return arg.split(",")
##	else :
##		return const_eval(arg, default)

######### PYTHON HACKS / DEBUGGING #######################################
import sys, os
import dis

#=============== Safe evaluator ===========================================
# Python constant only eval() and constant-only expression eval()
#	only "safe" expressions will be evaluted
#	From http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/286134

_const_codes = map(dis.opmap.__getitem__, [
	'POP_TOP','ROT_TWO','ROT_THREE','ROT_FOUR','DUP_TOP',
	'BUILD_LIST','BUILD_MAP','BUILD_TUPLE',
	'LOAD_CONST','RETURN_VALUE','STORE_SUBSCR'
	])

_expr_codes = _const_codes + map(dis.opmap.__getitem__, [
	'UNARY_POSITIVE','UNARY_NEGATIVE','UNARY_NOT',
	'UNARY_INVERT','BINARY_POWER','BINARY_MULTIPLY',
	'BINARY_DIVIDE','BINARY_FLOOR_DIVIDE','BINARY_TRUE_DIVIDE',
	'BINARY_MODULO','BINARY_ADD','BINARY_SUBTRACT',
	'BINARY_LSHIFT','BINARY_RSHIFT','BINARY_AND','BINARY_XOR',
	'BINARY_OR',
	])

def _get_opcodes(codeobj):
	"""_get_opcodes(codeobj) -> [opcodes]

	Extract the actual opcodes as a list from a code object

	>>> c = compile("[1 + 2, (1,2)]", "", "eval")
	>>> _get_opcodes(c)
	[100, 100, 23, 100, 100, 102, 103, 83]
	"""
	i = 0
	opcodes = []
	s = codeobj.co_code
	while i < len(s):
		code = ord(s[i])
		opcodes.append(code)
		if code >= dis.HAVE_ARGUMENT:
			i += 3
		else:
			i += 1
	return opcodes		

def test_expr(expr, allowed_codes):
	"""test_expr(expr) -> codeobj

	Test that the expression contains only the listed opcodes.
	If the expression is valid and contains only allowed codes,
	return the compiled code object. Otherwise raise a ValueError
	"""
	try:
		c = compile(expr, "", "eval")
	except:
		raise ValueError, "%s is not a valid expression", expr
	codes = _get_opcodes(c)
	for code in codes:
		if code not in allowed_codes:
			raise ValueError, "opcode %s not allowed" % dis.opname[code]
	return c

def const_eval(expr, default = QNOVALUE ):
	"""const_eval(expression) -> value

	Safe Python constant evaluation

	Evaluates a string that contains an expression describing
	a Python constant. Strings that are not valid Python expressions
	or that contain other code besides the constant raise ValueError.

	>>> const_eval("10")
	10
	>>> const_eval("[1,2, (3,4), {'foo':'bar'}]")
	[1, 2, (3, 4), {'foo': 'bar'}]
	>>> const_eval("1+2")
	Traceback (most recent call last):
	...
	ValueError: opcode BINARY_ADD not allowed
	"""
	if default == QNOVALUE :
		c = test_expr(expr, _const_codes)
	else :
		try:
			c = test_expr(expr, _const_codes)
		except ValueError:
			return default
	return eval(c)

def expr_eval(expr, default = QNOVALUE):
	"""expr_eval(expression) -> value

	Safe Python expression evaluation

	Evaluates a string that contains an expression that only
	uses Python constants. This can be used to e.g. evaluate 
	a numerical expression from an untrusted source.

	>>> expr_eval("1+2")
	3
	>>> expr_eval("[1,2]*2")
	[1, 2, 1, 2]
	>>> expr_eval("__import__('sys').modules")
	Traceback (most recent call last):
	...
	ValueError: opcode LOAD_NAME not allowed
	"""
	if default == QNOVALUE :
		c = test_expr(expr, _expr_codes)
	else :
		try:
			c = test_expr(expr, _expr_codes)
		except ValueError:
			return default
	return eval(c)

#=============== Argument handling utils ==================================

def expecting():
	" Returns number of values caller is expecting " 
	f = sys._getframe().f_back.f_back
	i = f.f_lasti + 3
	bytecode = f.f_code.co_code
	instruction = ord(bytecode[i])
	while True:
		if instruction == dis.opmap['DUP_TOP']:
			if ord(bytecode[i + 1]) == dis.opmap['UNPACK_SEQUENCE']:
				return ord(bytecode[i + 2])
			i += 4
			instruction = ord(bytecode[i])
			continue
		if instruction == dis.opmap['STORE_NAME']:
			return 1
		if instruction == dis.opmap['UNPACK_SEQUENCE']:
			return ord(bytecode[i + 1])
		return 0

def varargs( *argsList ):
	""" Return tuple/list with exact numbers of elemnts 
	expected by return value, regardless of actual argsList length.
	Ex)	( arg1, arg2 ) = varargs((1, 2, 3, 4)) 
		( arg1, arg2 ) = varargs(1, 2, 3, 4)
	"""
	exp_args = expecting()
	if len(argsList) == 1 and isSeq(argsList[0]) :
		args = argsList[0]
	else:
		args = argsList
	
	arglen = len(args)
	if exp_args > arglen :
		return args + (None,) * ( exp_args - arglen )
	else:
		return args [ : exp_args]

#========= DEBUGGING / TRACING/ ERROR  UTILS ==============================

def import_module(modName):
	try:
		module = __import__(modName)
		return module
	except ImportError:
		return None

# This doesn't work in Civ4
def findModuleFile( modName ):
	" find named Python module. Returns file object on success; <None> on failure."
	import imp
	try:
		fo, path, suff = imp.find_module( modName )
		return fo	# fo.name has module path value.
	except ImportError:
		return None

def getFuncName(bWithFile =True):
	""" Retuens string of function name of caller with optional file name prefix
		ex) "caller", "my_module.py:caller" """
	co = sys._getframe(1).f_code
	if bWithFile :
		return os.path.split(co.co_filename)[1] + ":" + co.co_name
	else :
		return co.co_name

def frameSrcLoc(upframe = 0):
	""" retuns current caller's frame source location info as tuple of 
	(module file full path, function name, first line # of function (int)) """
	co = sys._getframe(upframe + 1).f_code
	if co :
		return (co.co_filename, co.co_name, co.co_firstlineno)
	else:
		return ( "", "", 0 )

def qPrint(stuff, *stuff2 ):
	"Print to stdout with word-wrap (similar to CvUtil.pyPrint)"
	if not isinstance(stuff, (str, unicode)):
		stuff = str(stuff)
	if stuff2 :
		stuff2 = map( unicode, stuff2 )
		stuff = stuff + ": " + ", ".join(stuff2)
	stuff = strwrap ( stuff + "\n" , 80 )
	try:
		sys.stdout.write(stuff)
	except:
		pass

qqDebugLevel = 0

def qqDebug(lev = QNOVALUE):
	" get or set qqDebugLevel for qqPrint(), qqAssert() "
	global qqDebugLevel
	if lev != QNOVALUE:
		qqDebugLevel =int(lev)
	return qqDebugLevel

def qqPrint(stuff1, *suff2 ):
	"Print with source location for debugging, only if qqDebugLevel is set"
	if qqDebugLevel > 0:
		stuff1 = "QQ:[%s %s(%d)] %s" % frameSrcLoc(1) + ( stuff1,)
		qPrint(stuff1, *stuff2)

def qqAssert(cond, msg):
	""" If qqDebugLevel > 0, check assert condition and generate assert error,
	Else, just print error to stderr on cond failure """
	if not cond:
		msg = str(frameSrcLoc(1)) + ": " + msg
		if qqDebugLevel > 0:
			sys.stderr.write(msg)
		else :
			assert False, msg

def reportError(*msg):
	" Report error to stderr with class name (for class/instance) or source location"
	if isInstanceType(msg[0]) or isClassType(msg[0]):
		msg2 = msg[1:]
		qPrint("Error in %s" % className(msg[0]), *msg2)
	else:
		(path, name, line) = frameSrcLoc(1)
		qPrint( "Error in %s: %s() begining line %d " % ( path, name, line), *msg ) 

######## OS ACCESS UTILS ################################################

# join multiple file path name parts to single path"
path_join = os.path.join

def path_isfile( *paths ):
	" is the path is normal ordinary file?"
	return os.path.isfile ( os.path.join( *paths ) )

def path_listdir ( *paths ):
	" return all directory enties in the path "
	return os.listdir ( os.path.join( *paths ) )
	
def path_normalize( *paths):
	""" Normalize path for comparison. To all lowercase path with "/" separator.
	 '/' or '\' at the end of path will be removed. """
	apath = os.path.join( *paths ).lower().replace("\\","/")
	if len(apath) > 1 and  apath[-1] == "/" :
		return apath[:-1]
	else :
		return apath

def getRegValue(key):
	""" Retrive window registry value. key must start like "HKLM/SOFTWARE/...".
	If key end with "/" ( like ".../install/" ) , retrive unnamed value of the key.
	Returns <None> on all error."""
	import _winreg
	_regroot = { "HKLM":	_winreg.HKEY_LOCAL_MACHINE,
		"HKCU":	_winreg.HKEY_CURRENT_USER,	"HKCR":	_winreg.HKEY_CLASSES_ROOT,
		"HKU":	_winreg.HKEY_USERS,			"HKCC":	_winreg.HKEY_CURRENT_CONFIG, }

	compos = key.replace("/", "\\" ).split("\\")
	if compos[0] not in _regroot:
		return None
	try:
		reg = _winreg.OpenKey( _regroot[compos[0]], "\\".join(compos[1:-1]) )
		if reg :
			if compos[-1]:
				return _winreg.QueryValueEx( reg, compos[-1])[0]
			else:
				return _winreg.QueryValue( reg, compos[-1])
	except EnvironmentError:
		return None

# *TODO* Do more sequenc utils
#def strsubs( src, sub_list ):
#	""" multiple string substitution. Beware of re-substitution mutiple times
#	sub_list is '[[ "patt1", "replace1"], [ "patt2", "replace2" ], ... ] form """
#	for sub in sub_list:
#		src = src.replace( sub[0], sub[1])
#	return src
#
#def qindex( seq , index, default = None):
#	""" return index of matching sequence (zero-based) or dict, 
#		return default if index is negative or out or range 
#		or not found in dict. it does not generate IndexError """
#	
#	if isSequence( seq ):
#		if 0 <= index < len(seq):
#			return index
#		else:
#			return default
#	elif isDict(seq):
#		if index in seq :
#			return seq[index]
#		else:
#			return default
#	elif index == 0:	# special case!!
#		return seq
#	else:
#		raise TypeError

#def multisubs( src, sub_list, resub = False):
#	""" multiple string/list substitution. Returns new substituted string/list.
#	if resub is <True>, search over substituted string and resubstitute. 
#	sub_list is  '[[ patt1, replace1], [ patt2, replace2 ], ... ] form """
#	
##	outlist = []
##	subs = map(car, sub_list)
##	for item in src:
##		try:
##			found = subs.index(item)
##			outlist.append(sub_list[found][1])
##		except ValueError:
##			outlist.append (item)
#	outlist = list(src)	# copy it.
#	for sub in sub_list
#		matches = mutifind( qq( resub, outlist, src , sub[0] ))
#		for pos in matches
#			if IsString(sub[0]):
#				outlist[pos : pos + len(sub[0])] = sub[1]
#			else:
#				outlist[pos] = sub[1]
#			
#	return outlist

#def dict_merge(d1, d2):
#	" Merge two dicts. for same key, overwrite d1[key] with d2[key]"
#	assert isDict(d1) and isDict(d2)
#	for key, value in d2.items():
#		d1[key] = value
#	return d1

#-------------------------------------------------------------------------
#	A dict class with attribute syntax
#	ex) att_dic.keyAttr  <==> att_dic["keyAttr"]
#	You can use dic.keyAttr = item, or item = dic.keyAttr form to set and get item.
#	It can be init'ed from other dict or list or tuple or set.
#
#	Inherited method from dict type:
#	clear(self), copy(self), get(self, key, failobj=None), has_key(self, key) 
#	items(self), keys(self), update(self, dict), values(self)
#
class attr_dict(dict):
	" dictionary of values with class attribute syntax form."
	__slots__ = []

	def __init__( self, init_data = {} ):
		""" ini_data may be dict, tuple, list, set.
			if value part is not applicable like list, fill <None>"""

		if isDict(init_data):
			init_dict = init_data
		elif isContainer(init_data):  # tuple, list, set
			init_dict = dict(zip(list(init_data), [None]* len(init_data)))
		elif init_data :
			raise ValueError(init_data)
		dict.__init__(self, init_dict )
			
	def __getitem__( self, args ):
		" get value with default"
		if isTuple(args) and len(args) == 2:
			(attr, default) = args
		else:
			(attr, default) = ( args, QNOVALUE)
		try:
			return dict.__getitem__(self, attr )
		except KeyError:
			if default == QNOVALUE :
				raise
		return default
	
	def __getattr__( self, attr):
		try:
			return self[ attr ]
		except KeyError:
			raise AttributeError, attr

	def __setattr__( self, attr, value ):
		if attr in self.__slots__ :
			dict.__setattr__(self, attr, value)
		else:
			try :
				self [ attr ] = value
			except KeyError:
				raise AttributeError, attr
	
	def __delattr__( self, attr ):
		del self[ attr ]
	
	def __getstate__ ( self ):
		return  "" 

	def __setstate__ ( self, state ):
		if state :
			dict.__init__(self, const_exp(state, {}))
	
	@staticmethod
	def isident(name):
		" is the name valid identifier for python attribute? ([_A-Za-z][_A-Za-z0-9]*)"
		return isident(name)

#=========================================================================
