## Sid Meier's Civilization 4
## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Implementaion of miscellaneous game functions

import CvUtil
from CvPythonExtensions import *

# START | Enhanced Game Utils | 12monkeys | v1.0 | 29-Apr-2006 
import sys
# END | Enhanced Game Utils | 12monkeys | v1.0

# globals
gc = CyGlobalContext()

class CvGameUtils:
	"Miscellaneous game functions"
	def __init__(self): 
		pass

#====== ADDED BY SIMCUTIE ===============================================
#	HOW TO USE EXTENSION DLL INTERFACE ENTRYPOINT
#
#	To enable new game intarface, call  PyCivSim.activateGameEntryPoint("new" )
#	To enable old game intarface, call  PyCivSim.activateGameEntryPoint("old" )
#	To activate particular call to interface entrypoints from Civ4CCP Extension DLL,
#	call PyCivSim.activateGameEntryPoint("entryPointName" ) in "OnInit" event time. 
#
#	Example) 
#	In "YourMOD.py":
#		def OnInit(argsList):
#			"onInit event Handler - activate custom extension entry Point"
#			import PySivSim
#			PyCivSim.activateGameEntryPoint( "new" )
#			PyCivSim.activateGameEntryPoint( "canPlayerContact" )
#
#	In "CvGameUtils.py":
#		def canPlayerContact(self, argsList):
#			( pPlayer, ePlayer, bcanContactResult) = argsList[:3]
#			bcanContact = .......	# decision logic of your MOD.
#			return bcanContact
#
#	In C++ SDK:
#	bool CyExtGC::activateGameEntryPoint(LPCSTR szPyFunc):
#		Thic function activates Python game entry point from DLL.
#
#======= END BY SIMCUTIE =================================================

##################################################################################
# START | Enhanced Game Utils | 12monkeys | v1.0 | 29-Apr-2006 
	
	# ---------------------------------------------------------------
	# called by :
	#	MonkeyTools.cpp dllDebug()
	# 
	# Function to write debug information to screen and/or PythonDbg.log file
	# 
	def dllDebug(self, argsList):
		printToScr = True
		printToLog = True
		message = argsList[0]
		if (printToScr):
			CyInterface().addImmediateMessage(message,"")
		if (printToLog):
			message = 'DLL:' + message + "\n"
			sys.stdout.write(message)
		return 0

	# ---------------------------------------------------------------
	# called by :
	#	CvPlayer.cpp CvPlayer::canContact()
	#
	# called when : 
	# 	decides if a player can get into contact with another one. 
	#	The function is called after final decision of SDK.
	# 	The value bcanContactResult is the results of the original function of CvPlayer::canContact. If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: contact is allowed
	# 	FALSE 	: contact is NOT allowed
	#
	def canPlayerContact(self, argsList):
		pPlayer = argsList[0] # current player
		ePlayer = argsList[1] # player to be contact
		bcanContactResult = argsList[2] # the SDKs decision
		return bcanContactResult
		
	# ---------------------------------------------------------------
	# called by :
	#	CvPlayer.cpp CvPlayer::canTradeItem()
	#
	# called when : 
	# 	decides if a player can trade an item swith onother player. 
	#	The function is called after final decision of SDK.
	# 	The value bcanTradeItemResult is the results of the original function of CvPlayer::canTradeItem. If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: trade that item is allowed 
	# 	FALSE 	: trade that item is NOT allowed
	#
	def canPlayerTradeItem(self, argsList):
		pPlayer = argsList[0] # current player
		ePlayer = argsList[1] # player the item should be get
		eTradeItem = argsList[2] # the trade item (check the TradableItems constants)
		bTestDenial = argsList[3]
		bcanTradeItemResult = argsList[4] # the SDKs decision
		return bcanTradeItemResult
		
	# ---------------------------------------------------------------
	# called by :
	#	CvPlayer.cpp CvPlayer::canTradeNetworkWith()
	#
	# called when : 
	# 	this function is called to check if a player is connected via trade network to another player.
	#	The function is called after final decision of SDK.
	# 	The value bcanTradeNetworkWithResult is the results of the original function of CvPlayer::canTradeNetworkWith. If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	# 	TRUE	: trade via trade network is possible
	# 	FALSE 	: trade via trade network is NOT possible
	# 
	def canPlayerTradeNetworkWith(self, argsList):
		pPlayer = argsList[0] # current player
		ePlayer = argsList[1] # player to who the network is checked
		bcanTradeNetworkWithResult = argsList[2] # the SDKs decision
		return bcanTradeNetworkWithResult
		
	# ---------------------------------------------------------------
	# called by :
	#	CvPlayer.cpp CvPlayer::canRaze()
	#
	# called when : 
	# 	decides if a city canbe razed or not. 
	#	The function is called after final decision of SDK.
	# 	The value bcanRazeResult is the results of the original function of CvPlayer::canRaze. If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: raze is allowed 
	# 	FALSE 	: raze is NOT allowed
	#
	def canPlayerRazeCity(self, argsList):
		pPlayer = argsList[0] # current player
		pCity = argsList[1] # city to be razed
		bcanRazeResult = argsList[2] # the SDKs decision
		return bcanRazeResult
		
	# ---------------------------------------------------------------
	# called by :
	#	CvPlayer.cpp CvPlayer::raze()
	#
	# called when : 
	# 	just before a city is razed.
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: raze is performed (default)
	# 	FALSE 	: raze is NOT performed
	#
	def razeCity(self, argsList):
		pPlayer = argsList[0] # current player
		pCity = argsList[1]
		return true
		
	# ---------------------------------------------------------------
	# called by :
	#	CvPlayer.cpp CvPlayer::canFound()
	#
	# called when : 
	# 	decides if a city can be found on a certain plot. 
	#	The function is called after final decision of SDK.
	# 	The value bcanFoundResult is the results of the original function of CvPlayer::canFound. If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: a city can be found on that plot 
	# 	FALSE 	: a city can NOT be found on that plot
	#
	def canPlayerFoundCity(self, argsList):
		pPlayer = argsList[0] # current player
		pPlot = argsList[1] # the plot where the city should be built
		bTestVisible = argsList[2] 
		bcanFoundResult = argsList[3] # the SDKs decision
		return bcanFoundResult
		
	# ---------------------------------------------------------------
	# called by :
	#	CvPlayer.cpp CvPlayer::canConvert()
	#
	# called when : 
	# 	this function decides if a player can convert to a religion or not. It is used for diplomatic purposes, 
	# 	trade of religions, religion advisor screen, aso..
	#	The function is called after final decision of SDK.
	# 	The value bcanConvertResult is the results of the original function of CvPlayer::canConvert. If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: conversion is allowed 
	# 	FALSE 	: conversion is NOT allowed
	# 
	def canPlayerConvert(self, argsList):
		pPlayer = argsList[0]
		eReligionType = argsList[1]
		bcanConvertResult = argsList[2] # the SDKs decision
		return bcanConvertResult

	# ---------------------------------------------------------------
	# called by :
	#	CvPlayer.cpp CvPlayer::foundReligion()
	#
	# called when : 
	# 	this function is called just before a religion is founded. 
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: found religion is allowed (default)
	# 	FALSE 	: found religion is NOT allowed
	# 
	def playerFoundReligion(self, argsList):
		pPlayer = argsList[0]
		eReligionType = argsList[1]
		return true			
		
	# ---------------------------------------------------------------
	# called by :
	#	CvSelectionGroup.cpp CvSelectionGroup::canStartMission()
	#
	# called when : 
	# 	decides if a mission can be done by a group. EG: Is relevant for the display of the action buttons.
	#	The function is called after final decision of SDK.
	# 	The value bcanStartMissionResult is the results of the original function of CvPlayer::canStartMission. If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: mission is allowed
	# 	FALSE 	: mission is NOT allowed
	# 
	def canGroupStartMission(self, argsList):
		pSelectionGroup = argsList[0]
		pPlot = argsList[1]
		eMissionType = argsList[2]
		iData1 = argsList[3]
		iData2 = argsList[4]
		bTestVisible = argsList[5]
		bcanStartMissionResult = argsList[6] # the SDKs decision
		return bcanStartMissionResult
	
	# ---------------------------------------------------------------
	# called by :
	#	CvSelectionGroup.cpp CvSelectionGroup::startMission()
	#
	# called when : 
	# 	before a unit/group mission is started
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: mission is done (perform SDK decision)
	# 	FALSE 	: mission is NOT done
	# 
	# remark : 
	#	When return FALSE, the mission remains on mission queue. 
	# 	It can be deleted from queue using popMission() command.
	#
	def groupStartMission(self, argsList):
		pSelectionGroup = argsList[0]
		eMissionType = argsList[1]
		iData1 = argsList[2]
		iData2 = argsList[3]
		return true	
	
	# ---------------------------------------------------------------
	# called by :
	#	CvUnit.cpp CvUnit::doCommand()
	#
	# called when : 
	# 	before a unit command is performed
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: command is allowed
	# 	FALSE 	: command is NOT allowed
	# 
	def doUnitCommand(self, argsList):
		pUnit = argsList[0]
		eCommandType = argsList[1]
		iData1 = argsList[2]
		iData2 = argsList[3]
		return true	

	# ---------------------------------------------------------------
	# called by :
	#	CvUnit.cpp CvUnit::canSpread()
	#
	# called when : 
	# 	this function decides if a missionary can spread a religion to specific city (at plot).
	#	It is also used from the AI to send a missinary and to decide whether a "spread" action button is
	#	displayed or not.
	# 	The value bcanSpreadResult is the results of the original function of CvUnit::canSpread(). If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: spread is allowed
	# 	FALSE 	: spread is NOT allowed
	# 
	def canUnitSpread(self, argsList):
		pUnit = argsList[0]
		pPlot = argsList[1]
		eReligionType = argsList[2]
		bTestVisible = argsList[3]
		bcanSpreadResult = argsList[4] # the SDKs result
		return bcanSpreadResult

	# ---------------------------------------------------------------
	# called by :
	#	CvUnit.cpp CvUnit::canMoveInto()
	#
	# called when : 
	# 	This functions decides if a unit (incl. Animals, player and AI units) can move into a plot or not.
	#	The function is called after final decision of SDK. So be carful if you don't want that the a lion 
	# 	walk across water. 
	# 	The value bcanMoveIntoResult is the results of the original function of CvUnit::canMoveInto(). If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: move is allowed
	# 	FALSE 	: move is NOT allowed
	#
	def canUnitMoveInto(self, argsList):
		pUnit = argsList[0]
		pPlot = argsList[1]
		bAttack = argsList[2]
		bDeclareWare = argsList[3]
		bIgnoreLoad = argsList[4]
		bcanMoveIntoResult = argsList[5] 	# the SDKs result
		return bcanMoveIntoResult
				
	# ---------------------------------------------------------------
	# called by :
	#	CvUnit.cpp CvUnit::canFortify()
	#
	# called when : 
	# 	This functions decides if a unit can be fortified on a plot or not.
	#	The function is called after final decision of SDK.
	# 	The value bcanFortifyResult is the results of the original function of CvUnit::canFortify(). If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: unit can fortify 
	# 	FALSE 	: unit can't fortify 
	#
	def canUnitFortify(self, argsList):
		pUnit = argsList[0]
		pPlot = argsList[1]
		bcanFortifyResult = argsList[2] # the SDKs result
		return bcanFortifyResult
		
	# ---------------------------------------------------------------
	# called by :
	#	CvUnit.cpp CvUnit::canHeal()
	#
	# called when : 
	# 	This functions decides if a unit can be healed on a plot or not.
	#	The function is called after final decision of SDK.
	# 	The value bcanHealResult is the results of the original function of CvUnit::canHeal(). If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: unit can heal 
	# 	FALSE 	: unit can't heal 
	#
	def canUnitHeal(self, argsList):
		pUnit = argsList[0]
		pPlot = argsList[1]
		bcanHealResult = argsList[2] # the SDKs result
		return bcanHealResult

	# ---------------------------------------------------------------
	# called by :
	#	CvUnit.cpp CvUnit::canNukeAt()
	#
	# called when : 
	# 	This functions decides if a nuke can be fired at a certain plot.
	#	The function is called after final decision of SDK.
	#	Return TRUE to allow to nuke your friends or yourself.
	# 	The value bcanNukeAtResult is the results of the original function of CvUnit::canHeal(). If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: the nuke can be fired at that plot
	# 	FALSE 	: the nuke can't be fired at that plot
	#
	def canUnitNukeAt(self, argsList):
		pUnit = argsList[0]
		pPlot = argsList[1]
		bcanNukeAtResult = argsList[2] # the SDKs result
		return bcanNukeAtResult
		
	# ---------------------------------------------------------------
	# called by :
	#	CvUnit.cpp CvUnit::upgradePrice()
	#
	# called when : 
	# 	This functions calculates the upgrade price for a certain unit.
	# 	The value iPrice is the results of the original function of CvUnit::upgradePrice(). If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	iPrice 	: the returned value is the new upgrade price
	#
	def unitUpgradePrice(self, argsList):
		pUnit = argsList[0]
		eUnitType = argsList[1]
		iPrice = argsList[2] # the SDKs result
		return iPrice

	# ---------------------------------------------------------------
	# called by :
	#	CvUnit.cpp CvUnit::isInvisible()
	#
	# called when : 
	# 	This functions decide wheather a unit is invisible or not. 
	#	The function is called after final decision of SDK.
	# 	The value bisInvisibleResult is the results of the original function of CvUnit::isInvisible(). If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: the unit is invisible 
	# 	FALSE 	: the unit is NOT invisible 
	#
	def isUnitInvisible(self, argsList):
		pUnit = argsList[0]
		eTeam = argsList[1]
		bDebug = argsList[2]
		bisInvisibleResult = argsList[3] # the SDKs result
		return bisInvisibleResult
		
	# ---------------------------------------------------------------
	# called by :
	#	CvUnit.cpp CvUnit::canJoinGroup()
	#
	# called when : 
	# 	This functions decide wheather a unit can enter a group or not. It is called after all plausibility checks are done.
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: the unit can join the group (default).
	# 	FALSE 	: the unit can't join the group
	#
	def canUnitJoinGroup(self, argsList):
		pUnit = argsList[0]
		pPlot = argsList[1]
		pSelectionGroup = argsList[2]
		return true
		
	# ---------------------------------------------------------------
	# called by :
	#	CvCity.cpp CvCity::doReligion()
	#
	# called when : 
	# 	this function is called just before a religion is spread randomly to city.
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: religion spread will be performed (perform SDK decision)
	# 	FALSE 	: religion spread will NOT be performed
	# 
	# remark :
	#	there is another function "CvGameUtils.doReligion()" in this file here, which came with the standard API.
	#	The "doReligion()" is called when the SDK starts to examine if a random spread should be done or not. At that moment its is 
	#	not finally decided if a spread will be done, to what city and what religion. 
	#	The function "spreadReligionToCity()" is called when all of this is already decided and just before the spread is performed.
	#
	def spreadReligionToCity(self, argsList):
		pCity = argsList[0]
		eReligionType = argsList[1]
		return true	
		
	# ---------------------------------------------------------------
	# called by :
	#	CvCity.cpp CvCity::canWork()
	#
	# called when : 
	# 	this function is called to decide if a plot is a cities working plot or not.
	#	The function is called after final decision of SDK.
	# 	The value bcanWorkResult is the results of the original function of CvCity::canWork(). If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: plot is a working plot
	# 	FALSE 	: plot is NOT a working plot
	#
	def canCityWorkOnPlot(self, argsList):
		pCity = argsList[0]
		pPlot = argsList[1]
		bcanWorkResult = argsList[2] # the SDKs result
		return bcanWorkResult	
		
	# ---------------------------------------------------------------
	# called by :
	#	CvCity.cpp CvCity::canHurry()
	#
	# called when : 
	# 	this function is called to decide if a city can hurry production or not
	#	The function is called after final decision of SDK.
	# 	The value bcanHurryResult is the results of the original function of CvCity::canHurry(). If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: city can hurry production
	# 	FALSE 	: city can NOT hurry production
	#
	def canCityHurry(self, argsList):
		pCity = argsList[0]
		eHurryType = argsList[1]
		bTestVisible = argsList[2]
		bcanHurryResult = argsList[3] # the SDKs result
		return bcanHurryResult	
		
	# ---------------------------------------------------------------
	# called by :
	#	CvCity.cpp CvCity::canConscript()
	#
	# called when : 
	# 	this function is called to decide if a city can hurry production or not
	#	The function is called after final decision of SDK.
	# 	The value bcanConscriptResult is the results of the original function of CvCity::canConscript(). If you don't want
	#	to change something on the SDKs decision, this value should be returned (default)
	#	The AI uses this as well.
	#
	# returns :
	#	TRUE 	: city can conscript 
	# 	FALSE 	: city can NOT conscript
	#
	def canCityConscript(self, argsList):
		pCity = argsList[0]
		bcanConscriptResult = argsList[1] # the SDKs result
		return bcanConscriptResult	

	# ---------------------------------------------------------------
	# called by :
	#	CvCity.cpp CvCity::calculateDistanceMaintenance()
	#
	# called when : 
	# 	this function is called to calculate the maintenance cost of a city caused by its plot distance to the capital city.
	#	The AI uses this as well.
	#
	# returns :
	#	min(iWorstCityMaintenance, iBestCapitalMaintenance) : the cities distance maintenance costs
	#
	# remark :
	# 	the SDK calculates the two values iWorstCityMaintenance and iBestCapitalMaintenance. The smaller 
	#	one is used as maintenance value.
	#
	def calculateCityDistanceMaintenance(self, argsList):
		pCity = argsList[0]
		iWorstCityMaintenance = argsList[1]
		iBestCapitalMaintenance = argsList[2]
		return min(iWorstCityMaintenance, iBestCapitalMaintenance)
		
	# ---------------------------------------------------------------
	# called by :
	#	CvCity.cpp CvCity::calculateNumCitiesMaintenance()
	#
	# called when : 
	# 	this function is called to calculate the maintenance cost of a city caused by the number of cities of the civ.
	#	The AI uses this as well.
	#
	# returns :
	#	iNumCitiesMaintenance : the cities num cities maintenance costs
	#
	# remark :
	# 	the SDK calculates one value iNumCitiesMaintenance, which is passed here for info purposes. 
	#
	def calculateCityNumCitiesMaintenance(self, argsList):
		pCity = argsList[0]
		iNumCitiesMaintenance = argsList[1]
		return iNumCitiesMaintenance
		
# END | Enhanced Game Utils | 12monkeys | v1.0

################################################################################## 

	def isVictoryTest(self):
		if ( gc.getGame().getElapsedGameTurns() > 10 ):
			return True
		else:
			return False

	def isPlayerResearch(self, argsList):
		ePlayer = argsList[0]
		return True

	def createBarbarianCities(self):
		return False
		
	def createBarbarianUnits(self):
		return False
		
	def skipResearchPopup(self,argsList):
		ePlayer = argsList[0]
		return False
		
	def showTechChooserButton(self,argsList):
		ePlayer = argsList[0]
		return True

	def getFirstRecommendedTech(self,argsList):
		ePlayer = argsList[0]
		return TechTypes.NO_TECH

	def getSecondRecommendedTech(self,argsList):
		ePlayer = argsList[0]
		eFirstTech = argsList[1]
		return TechTypes.NO_TECH

	def skipProductionPopup(self,argsList):
		pCity = argsList[0]
		return False
		
	def showExamineCityButton(self,argsList):
		pCity = argsList[0]
		return True

	def getRecommendedUnit(self,argsList):
		pCity = argsList[0]
		return UnitTypes.NO_UNIT

	def getRecommendedBuilding(self,argsList):
		pCity = argsList[0]
		return BuildingTypes.NO_BUILDING

	def updateColoredPlots(self):
		return False

	def isActionRecommended(self,argsList):
		pUnit = argsList[0]
		iAction = argsList[1]
		return False

	def cannotHandleAction(self,argsList):
		pPlot = argsList[0]
		iAction = argsList[1]
		bTestVisible = argsList[2]
		return False

	def cannotSelectionListMove(self,argsList):
		pPlot = argsList[0]
		bAlt = argsList[1]
		bShift = argsList[2]
		bCtrl = argsList[3]
		return False

	def cannotSelectionListGameNetMessage(self,argsList):
		eMessage = argsList[0]
		iData2 = argsList[1]
		iData3 = argsList[2]
		iData4 = argsList[3]
		iFlags = argsList[4]
		bAlt = argsList[5]
		bShift = argsList[6]
		return False

	def cannotDoControl(self,argsList):
		eControl = argsList[0]
		return False

	def canResearch(self,argsList):
		ePlayer = argsList[0]
		eTech = argsList[1]
		bTrade = argsList[2]
		return False

	def cannotResearch(self,argsList):
		ePlayer = argsList[0]
		eTech = argsList[1]
		bTrade = argsList[2]
		return False

	def canDoCivic(self,argsList):
		ePlayer = argsList[0]
		eCivic = argsList[1]
		return False

	def cannotDoCivic(self,argsList):
		ePlayer = argsList[0]
		eCivic = argsList[1]
		return False
		
	def canTrain(self,argsList):
		pCity = argsList[0]
		eUnit = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		return False

	def cannotTrain(self,argsList):
		pCity = argsList[0]
		eUnit = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		return False

	def canConstruct(self,argsList):
		pCity = argsList[0]
		eBuilding = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		bIgnoreCost = argsList[4]
		return False

	def cannotConstruct(self,argsList):
		pCity = argsList[0]
		eBuilding = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		bIgnoreCost = argsList[4]
		return False

	def canCreate(self,argsList):
		pCity = argsList[0]
		eProject = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		return False

	def cannotCreate(self,argsList):
		pCity = argsList[0]
		eProject = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		return False

	def canMaintain(self,argsList):
		pCity = argsList[0]
		eProcess = argsList[1]
		bContinue = argsList[2]
		return False

	def cannotMaintain(self,argsList):
		pCity = argsList[0]
		eProcess = argsList[1]
		bContinue = argsList[2]
		return False

	def AI_chooseTech(self,argsList):
		ePlayer = argsList[0]
		bFree = argsList[1]
		return TechTypes.NO_TECH

	def AI_chooseProduction(self,argsList):
		pCity = argsList[0]
		return False

	def AI_unitUpdate(self,argsList):
		pUnit = argsList[0]
		return False

	def AI_doWar(self,argsList):
		eTeam = argsList[0]
		return False

	def AI_doDiplo(self,argsList):
		ePlayer = argsList[0]
		return False

	def calculateScore(self,argsList):
		ePlayer = argsList[0]
		bFinal = argsList[1]
		bVictory = argsList[2]
		
		iPopulationScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getPopScore(), gc.getGame().getInitPopulation(), gc.getGame().getMaxPopulation(), gc.getDefineINT("SCORE_POPULATION_FACTOR"), True, bFinal, bVictory)
		iLandScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getLandScore(), gc.getGame().getInitLand(), gc.getGame().getMaxLand(), gc.getDefineINT("SCORE_LAND_FACTOR"), True, bFinal, bVictory)
		iTechScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getTechScore(), gc.getGame().getInitTech(), gc.getGame().getMaxTech(), gc.getDefineINT("SCORE_TECH_FACTOR"), True, bFinal, bVictory)
		iWondersScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getWondersScore(), gc.getGame().getInitWonders(), gc.getGame().getMaxWonders(), gc.getDefineINT("SCORE_WONDER_FACTOR"), False, bFinal, bVictory)
		return int(iPopulationScore + iLandScore + iWondersScore + iTechScore)

	def doHolyCity(self):
		return False

	def doHolyCityTech(self,argsList):
		eTeam = argsList[0]
		ePlayer = argsList[1]
		eTech = argsList[2]
		bFirst = argsList[3]
		return False

	def doGold(self,argsList):
		ePlayer = argsList[0]
		return False

	def doResearch(self,argsList):
		ePlayer = argsList[0]
		return False

	def doGoody(self,argsList):
		ePlayer = argsList[0]
		pPlot = argsList[1]
		pUnit = argsList[2]
		return False

	def doGrowth(self,argsList):
		pCity = argsList[0]
		return False

	def doProduction(self,argsList):
		pCity = argsList[0]
		return False

	def doCulture(self,argsList):
		pCity = argsList[0]
		return False

	def doPlotCulture(self,argsList):
		pCity = argsList[0]
		bUpdate = argsList[1]
		return False

	def doReligion(self,argsList):
		pCity = argsList[0]
		return False

	def doGreatPeople(self,argsList):
		pCity = argsList[0]
		return False

	def doMeltdown(self,argsList):
		pCity = argsList[0]
		return False

##########################################################################
# $Source: /cvsroot/civ4geneticai/Assets/Python/CvGameUtils.py,v $: $Name:  $
#	Custom  GameInterface entry points.
#	$CopyRight$
#	by SimCutie < simcutie (AT) hanmail.net >
#	Based on Civ4 Update 1.61
#	This file should reside in  "... /CustomAssets/python/" folder.
#	Version: $Revision: 1.1 $ $Date: 2006/09/12 12:43:11 $
global __author__, __author__, __version__, __version__
__author__ = "$Author: the_great_apple $"
__revision__ = "$Id: CvGameUtils.py,v 1.1 2006/09/12 12:43:11 the_great_apple Exp $"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2006/09/12 12:43:11 $"

class CvSimCutieGameUtils(CvGameUtils):

	def __init__(self):
		self.parent = CvGameUtils
		self.Old_GameUtils = True
		pass

	def OldInterface(bOld):
		self.Old_GameUtils = bOld

	# CvPyActionInfo
	# CvPyActionInfo::doAction() of CvExtension.cpp failed to call Python handler function.
	#	Override this to handle the situation in your code.	
	#	This requires Civ4CCP SDK DLL.
	def cannotHandlePyAction(self, argsList):
		" CvPyActionInfo::doAction() failed to execute 'PyAction' Python action handler"
		( actionID, iData2, bOpt, bRightClick, handlerName ) = argsList[:5]
		CvUtil.pyPrint("CvPyActionInfo: Can't handle: %s (ID:%d)" % (handlerName, actionID ) )
		return True

	def testGameInterface(self, argsList):
		CvUtil.pyPrint( __name__ + "/testGameInterface(): argsList = " + str(argsList))
		return argsList[0]

	# ---------------------------------------------------------------
	#	canUnitDoCommand()
	#	called by : CvPlayer.cpp: CvPlayer::canDoCommand()
	#	called to decides if the unit can execute the command or not.
	#	bcanDoCommandResult is result of original CvPlayer::canDoCommand() of SDK DLL..
	#	If it returns False, Unit can not execute the command.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#	
	def canUnitDoCommand(self, argsList):
		( pyUnit, eCommandType, iData1, iData2, bTestVisible, bcanDoCommandResult ) = argsList[:6]
		return bcanDoCommandResult

	# ---------------------------------------------------------------
	#	willUnitDoCommand()
	#		Works same as doUnitCommand()
	#	called by : CvUnit.cpp: CvUnit::doCommand()
	#	called before a unit is about to perform the command
	#	if it returns False, Unit will not execute the command.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	# 
	def willUnitDoCommand(self, argsList):
		( pUnit, eCommandType, iData1, iData2 )  = argsList[:4]
		if self.Old_GameUtils :
			return self.parent.doUnitCommand(argsList)
		return True	

	# ---------------------------------------------------------------
	#	canUnitFortify():
	#	called by : CvUnit.cpp CvUnit::canFortify()
	#	called to decide if a unit can be fortified or not.
	#	called after all plausibility checks are done.
	#	If it returns False, Unit can not fortify.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canUnitFortify(self, argsList):
		pUnit = argsList[0]
		if self.Old_GameUtils :
			return self.parent.canUnitFortify(( pUnit, pUnit.plot(), True))
		return True

	# ---------------------------------------------------------------
	#	canUnitHeal():
	#	called by : CvUnit.cpp CvUnit::canHeal()
	#	called to decide if a unit can heal or not.
	#	called after all plausibility checks are done.
	#	If it returns False, Unit can not heal.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canUnitHeal(self, argsList):
		pUnit = argsList[0]
		if self.Old_GameUtils :
			return self.parent.canUnitHeal(( pUnit, pUnit.plot(), True))
		return True

	# ---------------------------------------------------------------
	#	calculateUnitHealRate():
	#	called by : CvUnit.cpp CvUnit::canHeal()
	#	called to calculate Unit heal rate ( damage/turn )
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def calculateUnitHealRate(self, argsList):
		(pyUnit, pyPlot, lHealRateResult ) = argsList[:3]
		return lHealRateResult

	# ---------------------------------------------------------------
	#	canNukeAt():
	#	called by : CvUnit.cpp CvUnit::canNukeAt()
	#	called to decide if a unit can nuke at the plot or not.
	#	called after all plausibility checks are done.
	#	If it returns False, Unit can not lauch nuke to the plot.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canUnitNukeAt(self, argsList):
		( pUnit, pTargetPlot ) = argsList[:2]
		if self.Old_GameUtils :
			return self.parent.canUnitNukeAt((pUnit, pTargetPlot, True ))
		return True

	# ---------------------------------------------------------------
	#	calculateUnitUpgradePrice():
	#		Works same as unitUpgradePrice()
	#	called by : CvUnit.cpp: CvUnit::upgradePrice()
	#	called when calculating the upgrade price for a unit.
	#	upgradTo_eUnitType is UnitType to which the Unit grade to.
	# 	iPrice is the results of the original SDK of CvUnit::upgradePrice().
	#		(default return value)
	#	It returns iPrice : the returned value is the new upgrade price
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def calculateUnitUpgradePrice(self, argsList):
		( pUnit, upgradTo_eUnitType, iPrice )  = argsList[:3]
		if self.Old_GameUtils :
			return self.parent.upgradePrice(argsList)
		return iPrice

	# ---------------------------------------------------------------
	#	isUnitInvisible():
	#	called by : CvUnit.cpp CvUnit::isInvisible()
	#	called to decide if a unit is visible to the team or not
	#	called after basic visibility checks are done.
	#	If it returns True, Unit is invisible to the team.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def isUnitInvisible(self, argsList):
		(pUnit, eTeam, bisInvisibleResult)  = argsList[:3]
		if self.Old_GameUtils :
			return self.parent.isUnitInvisible(( pUnit, eTeam, False, bisInvisibleResult ))
		return bisInvisibleResult

	# ---------------------------------------------------------------
	#	isUnitInvisible():
	#	called by : CvUnit.cpp CvUnit::canJoinGroup()
	#	called to decide if a unit can join the selection group or not.
	#	called after all plausibility checks are done.
	#	If it returns False, Unit can not join the group.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canUnitJoinGroup(self, argsList):
		( pUnit, pSelectionGroup ) = argsList[:2]
		if self.Old_GameUtils :
			return self.parent.canUnitJoinGroup(( pUnit, pUnit.plot(), pSelectionGroup ))
		return bcanJoinGroupResult

	# ---------------------------------------------------------------
	#	canCityWorkOnPlot():
	#	called by : CvCity.cpp CvCity::canWork()
	#	called to decide if the City can work on the plot or not.
	#	called after all plausibility checks are done.
	#	If it returns False, the City can not work on the plot.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canCityWorkOnPlot(self, argsList):
		( pCity, pPlot ) = argsList[:2]
		if self.Old_GameUtils :
			return self.parent.canCityWorkOnPlot(( pCity, pPlot, True ))
		return True

	# ---------------------------------------------------------------
	#	canCityHurry():
	#	called by : CvCity.cpp CvCity::canHurry()
	#	called to decide if the City can hurry production or not.
	#	called after all plausibility checks are done.
	#	If it returns False, the City can not hurry production.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canCityHurry(self, argsList):
		( pCity, eHurryType ) = argsList[:2]
		if self.Old_GameUtils :
			return self.parent.canCityHurry(( pCity, eHurryType, False, True))
		return True	

	# ---------------------------------------------------------------
	#	canCityConscript():
	#	called by : CvCity.cpp CvCity::canConscript()
	#	called to decide if the City can draft army or not.
	#	called after all plausibility checks are done.
	#	If it returns False, the City can not draft army.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canCityConscript(self, argsList):
		pCity = argsList[:1]
		if self.Old_GameUtils :
			return self.parent.canCityConscript(( pCity, True))
		return True	

	# ---------------------------------------------------------------
	#	calculateCityDistanceMaintenance():
	#	called by : CvCity.cpp CvCity::calculateDistanceMaintenance()
	#	called to to calculate the maintenance cost of a city caused 
	#		by its plot distance to the capital city.
	#	Deafult lcalculateDistanceMaintenanceResult is 
	#		min(iWorstCityMaintenance, iBestCapitalMaintenance)
	#
	def calculateCityDistanceMaintenance(self, argsList):
		( pCity, iWorstCityMaintenance, iBestCapitalMaintenance, 
			lcalculateDistanceMaintenanceResult ) = argsList[:4]
		if self.Old_GameUtils :
			return self.parent.calculateCityDistanceMaintenance(
			( pCity, iWorstCityMaintenance, iBestCapitalMaintenance ) )

		return lcalculateDistanceMaintenanceResult

	# ---------------------------------------------------------------
	#	willSpreadReligionToCity()
	#		Works similar to spreadReligionToCity()
	#	called by : CvCity.cpp: CvCity::doReligion()
	#	called before a religion is about to spread to city.
	#		This is called when rolling dice to decide
	#		if the religion will be actually spread to city.
	#
	#	dice probability is (iRandThreshold/getDefineINT("RELIGION_SPREAD_RAND"))
	#	bRandomResult is outcome of boolean dice.
	#	if it returns False, religion will not spread.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#	See Remark in spreadReligionToCity() above on random religion spread occasion.
	#	NOTE that this behave differently from spreadReligionToCity() in that
	#		this will be invoked regardless of random dice value.
	#
	def willSpreadReligionToCity(self, argsList):
		( pCity, eReligion, iRandThreshold, bRandomResult ) = argsList[:4]
		if self.Old_GameUtils and bRandomResult :
			return self.parent.spreadReligionToCity(( pCity, eReligion ))
		return bRandomResult

	# ---------------------------------------------------------------
	#	canPlayerContact():
	#	called by : CvPlayer.cpp CvPlayer::canContact()
	#	called to decide if the Player can contact the target player or not.
	#	called after all plausibility checks are done.
	#	If it returns False, the Player can not contact the target player.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canPlayerContact(self, argsList):
		( pPlayer, pPlayerTarget ) = argsList[:2]
		if self.Old_GameUtils :
			return self.parent.canPlayerContact(
				( pPlayer, pPlayerTarget.getID(), True))
		return True

	# ---------------------------------------------------------------
	#	canPlayerTradeItem():
	#	called by : CvPlayer.cpp CvPlayer::canTradeItem()
	#	called to decide if the Player can trade item with the target player or not.
	#	called after all plausibility checks are done.
	#	If it returns False, the Player can not trade item with the target player.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canPlayerTradeItem(self, argsList):
		( pyPlayer, pyPlayerTarget, tradeType, tradeItem, 
			bTestDenial, bcanTradeItemResult ) = argsList[:6]
		if self.Old_GameUtils :
			return self.parent.canPlayerTradeItem(
				( pyPlayer, pyPlayerTarget.getID(), tradeType, bTestDenial, bcanTradeItemResult ))
		return bcanTradeItemResult

	# ---------------------------------------------------------------
	#	canPlayerTradeNetworkWith():
	#	called by : CvPlayer.cpp CvPlayer::canTradeNetworkWith()
	#	called to decide if the Player can establish trade route to the target player.
	#	called after all plausibility checks are done.
	#	If it returns False, the Player can not trade item with the target player.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canPlayerTradeNetworkWith(self, argsList):
		( pyPlayer, pyPlayerTarget ) = argsList[:2]
		if self.Old_GameUtils :
			return self.parent.canPlayerTradeNetworkWith(
				( pyPlayer, pyPlayerTarget.getID(), True ))
		return True

	# ---------------------------------------------------------------
	#	canPlayerRazeCity():
	#	called by : CvPlayer.cpp CvPlayer::canRaze()
	#	called to decide if the Player can raze the city or not.
	#	called after all plausibility checks are done.
	#	If it returns False, the Player can not raze the city.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canPlayerRazeCity(self, argsList):
		( pPlayer, pCity ) = argsList[:2] # current player
		if self.Old_GameUtils :
			self.parent.canPlayerRazeCity(( pPlayer, pCity, True ))
		return True

	# ---------------------------------------------------------------
	#	willPlayerRazeCity()
	#		Works like razeCity()
	#	called by : CvPlayer.cpp: CvPlayer::raze()
	#	called when player is about to raze the city.
	#	If it returns False, the player will not raze the city.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#	
	def willPlayerRazeCity(argsList):
		" Player is about to raze the city"
		(pPlayer, pCity) = argsList[:2]
		if self.Old_GameUtils :
			self.parent.razeCity(argsList)
		return True

	# ---------------------------------------------------------------
	#	canPlayerFoundCity():
	#	called by : CvPlayer.cpp CvPlayer::canFound()
	#	called to decide if the Player can found the city or not.
	#	called after all plausibility checks are done.
	#	If it returns False, the Player can not found the city.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canPlayerFoundCity(self, argsList):
		( pPlayer, pPlot ) = argsList[:2] # current player
		if self.Old_GameUtils :
			self.parent.canPlayerFoundCity((pPlayer, pPlot, False, True ))
		return bcanFoundResult

	# ---------------------------------------------------------------
	#	canPlayerConvert():
	#	called by : CvPlayer.cpp CvPlayer::canConvert()
	#	called to decide if the Player can convert to the religion or not.
	#	called after all plausibility checks are done.
	#	If it returns False, the Player can not convert to the religion.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canPlayerConvert(self, argsList):
		( pPlayer, eReligionType )  = argsList[:2]
		if self.Old_GameUtils :
			self.parent.canPlayerConvert(( pPlayer, eReligionType, True ))
		return True

	# ---------------------------------------------------------------
	#	willPlayerFoundReligion()
	#		Works same as playerFoundReligion()
	#	called by : CvPlayer.cpp: CvPlayer::foundReligion()
	#	called before a player is about to found the religion.
	#	If it returns False, religion will not be found.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def willPlayerFoundReligion(self, argsList):
		( pPlayer, eReligionType ) = argsList[:2]
		if self.Old_GameUtils :
			self.parent.playerFoundReligion(argsList)
		return True		

	# ---------------------------------------------------------------
	#	canGroupStartMission()
	#	called by : CvSelectionGroup.cpp: CvSelectionGroup::canStartMission()
	#	called to decide the selected group can start the mission or not.
	#
	#	bcanStartMissionResult is result of original CvSelectionGroup::canStartMission() of SDK DLL.
	#	If it returns False, the group can not start the mission. 
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def canGroupStartMission(self, argsList):
		( pyPlayer, eMission, iData1, iData2, 
			pyPlot, bTestVisible, bcanStartMissionResult ) = argsList[:7]
		if self.Old_GameUtils :
			self.parent.canGroupStartMission(( pSelectionGroup, pPlot, 
			eMissionType, iData1, iData2, bTestVisible, bcanStartMissionResult ))
		return bcanStartMissionResult
	
	# ---------------------------------------------------------------
	#	willGroupStartMission()
	#		Works like willGroupStartMission()
	#	called by : CvSelectionGroup.cpp: CvSelectionGroup::startMission()
	#	called before selected group is about to start the mission.
	#
	#	If it returns False, the group will not start the mission.
	#	and the mission will be removed from front of the mission queue.
	#	This requires Civ4CCP SDK DLL. AI player uses this as well.
	#
	def willGroupStartMission(self, argsList):
		" player is about to start the mission."
		( pSelectionGroup, eMissionType, iData1, iData2 ) = argsList[:4]
		if self.Old_GameUtils :
			self.parent.groupStartMission(argsList)
		return True

