## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
# $Source: /cvsroot/civ4geneticai/Assets/Python/CvScreenUtils.py,v $: $Name:  $
#	Custom input event dispatch CvScreenUtils Mod.
#	$CopyRight$
#	by SimCutie < simcutie (AT) hanmail.net >
#	Based on Civ4 Update 1.61
#	Redirect screen input to registered screen input event handler
#	Associated Files: CvScreensInterface, CvScreenUtilsInterface
#	This file should reside in  "... /CustomAssets/python" folder.
#
#	Version: $Revision: 1.1 $ $Date: 2006/09/12 12:43:11 $
#
# 	InputEventDict = (
#	'handleInput',	'update',			'keyInput',			'leftMouseUp',
#	'rightMouseUp',	'leftMouseDown',	'rightMouseDown',	'mouseOverPlot',
#	'updateScreen',	'redraw',			'minimapClicked',	'movieDone',	'onClose', )
#
# 1) to get receive input event (key press/mouse click) and screen event ( redraw...)  
#	in new screen ( in handleInput() or update(), onClose(), leftMouseUp(), .... )
# 2) to overide/replace existing stock screen to new subclassed screen instance .
#	See CvScreenUtil.py for detailed documentation.
#
#	def ScreenRegister( screen, screenEnum, inputEventList = None, Unused = None ):
#		defined in "PyCivSim.py"
#
#	Replace existing screen instance with new screen instance.
#	Or Register new/existing screen to receive various input event or screen event.  
#	- screen : screen class instance which will handle the inputs.
#	- screenEnum : screen id #. Unique integer or from "CvScreenEnums.py".
#	- inputEvent : input event the screen will handle. 
#		It is a tuple of input events. Event maybe string like 'handleInput'
#		ex : ( 'handleInput', 'leftMouseUp', 'redraw')
#		Then these 3 events will be redirected to the screen.
#	On the input event, respective screen class method of same name will be invoked.
#	inputEvent is string value listed in InputEventDict above.
#	If screenEnum is same as original game screen enum in "CvScreenEnums.py", 
#	the screen instance is replaced with new screen in arg.
#	In such case, and if inputEventList is None, no custom input handling is applied.
#
#	example 1) 
#	# To replace maininterface screen instance with new subclassed instance.  
#	def EventHandlerRegister( evt_mgr, unused = None ):
#		import CvCustomEvent, PyCivSim
#		mainInterface = CvSimCutieMainInterface()
#		# replace original CvMainInterface instance
#		PyCivSim.ScreenRegister( mainInterface, CvScreenEnums.MAIN_INTERFACE )
#		qPrint(__name__, "MainInterfaceScreen Register OK.")
#
#	example 2)
#	MYSCREEN_ENUM = 555
#	class CvMyScreen():
#		def __init__(self, scrEnum):
#			import PyCivSim
#			screen = CyGInterfaceScreen("MyScreen", MYSCREEN_ENUM)
#			PyCivSim.ScreenRegister( screen, MYSCREEN_ENUM, ( 'handleInput', 'leftMouseUp' ))
#
#		def handleInput(self, argsList):
#			.....
#
#		def leftMouseUp (self, argsList):
#			.....

# CAVEATS:
#	Left/Right Mouse Down event does not work. Use mouse up event only.
#	Event system does not generates mouse down event at all,
#	except in WorldBuilder Screen. I don't know why..
#

__author__ = "$Author: the_great_apple $"
__revision__ = "$Id: CvScreenUtils.py,v 1.1 2006/09/12 12:43:11 the_great_apple Exp $"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2006/09/12 12:43:11 $"

from CvPythonExtensions import NotifyCode, MouseFlags
from PyBaseLib import isInstanceType, isInt, isString, isSeq
from PyExtLib import qPrint, reportError

InputEventDict = (
	'handleInput',
	'update',
	'keyInput',
	'leftMouseUp',
	'rightMouseUp',
	'leftMouseDown',
	'rightMouseDown',
	'mouseOverPlot',
	'updateScreen',
	'redraw',
	'minimapClicked',
	'movieDone',
	'onClose',
)

from CvScreenEnums import *

CvScreensInterfaceScreenMap = {  MAIN_INTERFACE : 'mainInterface',
					DOMESTIC_ADVISOR : 'domesticAdvisor',
					RELIGION_SCREEN : 'religionScreen',
					CIVICS_SCREEN : 'civicScreen',
					TECH_CHOOSER : 'techChooser',
					FOREIGN_ADVISOR : 'foreignAdvisor',
					FINANCE_ADVISOR : 'financeAdvisor',
					MILITARY_ADVISOR : 'militaryAdvisor',
					DAWN_OF_MAN : 'dawnOfMan',
					WONDER_MOVIE_SCREEN : 'wonderMovie',
					ERA_MOVIE_SCREEN : 'eraMovie',
					INTRO_MOVIE_SCREEN : 'introMovie',
					OPTIONS_SCREEN : 'optionsScreen',
					INFO_SCREEN : 'infoScreen',
					TECH_SPLASH : 'techSplashScreen',
					REPLAY_SCREEN : 'replayScreen',
					VICTORY_SCREEN : 'victoryScreen',
					TOP_CIVS : 'topCivs',
					HALL_OF_FAME : 'hallOfFameScreen',
					VICTORY_MOVIE_SCREEN : 'victoryMovie',
					
					PEDIA_MAIN : 'pediaMainScreen',
					PEDIA_TECH : 'pediaMainScreen',
					PEDIA_UNIT : 'pediaMainScreen',
					PEDIA_BUILDING : 'pediaMainScreen',
					PEDIA_PROMOTION : 'pediaMainScreen',
					PEDIA_PROJECT : 'pediaMainScreen',
					PEDIA_UNIT_CHART : 'pediaMainScreen',
					PEDIA_BONUS : 'pediaMainScreen',
					PEDIA_IMPROVEMENT : 'pediaMainScreen',
					PEDIA_TERRAIN : 'pediaMainScreen',
					PEDIA_FEATURE : 'pediaMainScreen',
					PEDIA_CIVIC : 'pediaMainScreen',
					PEDIA_CIVILIZATION : 'pediaMainScreen',
					PEDIA_LEADER : 'pediaMainScreen',
					PEDIA_RELIGION : 'pediaMainScreen',
					PEDIA_HISTORY : 'pediaMainScreen',
					WORLDBUILDER_SCREEN : 'worldBuilderScreen',
					WORLDBUILDER_DIPLOMACY_SCREEN : 'worldBuilderDiplomacyScreen',
					
					DEBUG_INFO_SCREEN : 'debugInfoScreen',
					
				# add new screens here
				}

#	This is *NOT* subclass of CvScreenUtils
class CvScreenUtils:
	CustomHandleInputMap = { }	# don't touch.
	
	def CvScreensInterfaceScreenCheck(self, screen, screenEnum ):
		""" check the screen enum is original stock screen,
			Then replace origial instance with new screen insatnce"""
		import CvScreensInterface
	
		if screenEnum not in CvScreensInterfaceScreenMap:
			return 0
		
		old_screen = CvScreensInterface.HandleInputMap[screenEnum ]
		setattr( CvScreensInterface, CvScreensInterfaceScreenMap[screenEnum], screen)
		
		CvScreensInterface.HandleInputMap[screenEnum ] = screen
		if screenEnum in CvScreensInterface.HandleNavigationMap :
			CvScreensInterface.HandleNavigationMap[screenEnum ] = screen
		return old_screen
	
	def ScreenRegister( self, screen, screenEnum, inputEventList = None, Unused = None ):
		"""	Replace existing screen instance with new screen instance.
		Or Register new/existing screen to receive various input event or screen event.  
		See documentation in CvScreenUtils.py"""
		ERROR = None
		if ( not screen or not isInstanceType( screen ) or not isInt(screenEnum)):
			reportError(self, "Invalid screen or screen Enum", screenEnum )
			return ERROR
		old_screen =  self.CvScreensInterfaceScreenCheck( screen, screenEnum )
		if old_screen and not inputEventList : # no custom input handling.
			return old_screen
		
		if not inputEventList:		# default event
			inputEventList = ( 'handleInput' ,)
		elif isString( inputEventList ) :
			inputEventList = inputEventList.split(",")
		elif not isSeq( inputEventList ) :
			reportError(self, "Invalid inputEvent", inputEvent )
			return ERROR
		
		for inputEvent in inputEventList :
			if inputEvent == 'handleInput':
				key = screenEnum	# for map compatibility...
			else:		# Misc inpt events
				if inputEvent not in InputEventDict:
					reportError(self, "Unknown inputEvent", inputEvent )
					continue
				key = ( screenEnum, inputEvent )
			
			self.CustomHandleInputMap[ key ] = screen
		return screen
	
	def update(self, argsList):
		screen = self.CustomHandleInputMap.get(( argsList[0], 'update' ))
		if screen :
			return screen.update( argsList[1] )
	
	def handleInput (self, argsList):
		' handle input is called when a screen is up '
		screenEnum, input = argsList	#	input = inputClass
		if not input :
			return 0
		if self.CustomHandleInputMap.has_key(screenEnum) :
			# get the screen that is active from the CustomHandleInputMap Dictionary
			screen = self.CustomHandleInputMap.get( screenEnum ) # input.getPythonFile() )
			# call handle input on that screen
			if screen is None :
				return 0

			ret = screen.handleInput(input)
			if ret:				# return 1 to consume input
				return ret
		
		notiy_code = input.getNotifyCode()
		if notiy_code == NotifyCode.NOTIFY_CLICKED :
			uiFlags = input.getFlags()
			# Dummy data, all zero for now
			dummy_arg = ( input.iData, input.bShift, input.bCtrl, input.bAlt, 
				input.iMouseX, input.iMouseY, None )
			if ( uiFlags & MouseFlags.MOUSE_LBUTTONUP ):
				screen = self.CustomHandleInputMap.get(( screenEnum, 'leftMouseUp' ))
				if screen : return screen.leftMouseUp( dummy_arg )
			elif ( uiFlags & MouseFlags.MOUSE_RBUTTONUP ):
				screen = self.CustomHandleInputMap.get(( screenEnum, 'rightMouseUp' ))
				if screen : return screen.rightMouseUp( dummy_arg )
			# not working for now...
			elif ( uiFlags & MouseFlags.MOUSE_LBUTTONDOWN ):
				screen = self.CustomHandleInputMap.get(( screenEnum, 'leftMouseDown' ))
				if screen : return screen.leftMouseDown( dummy_arg )
			elif ( uiFlags & MouseFlags.MOUSE_RBUTTONDOWN ):
				screen = self.CustomHandleInputMap.get(( screenEnum, 'rightMouseDown' ))
				if screen : return screen.rightMouseDown( dummy_arg )
		
		elif notiy_code == NotifyCode.NOTIFY_CHARACTER :
			screen = self.CustomHandleInputMap.get(( screenEnum, 'keyInput' ))
			if screen :
				return screen.keyInput( input.getData(), (input.bShift, input.bCtrl, input.bAlt) )
		elif notiy_code == NotifyCode.NOTIFY_MOVIE_DONE :
			screen = self.CustomHandleInputMap.get(( screenEnum, 'movieDone' ))
			if screen : return screen.movieDone()
		
		return 0
	
#	# Screen closing
#	def onClose (self, argsList):
#		screenEnum = argsList[0]
#		return 0
	# Screen closing
	def onClose (self, argsList):
		screen = self.CustomHandleInputMap.get(( argsList[0], 'onClose' ))
		if screen :
			return screen.onClose( argsList[1:] )

	def leftMouseUp (self, argsList):
		screen = self.CustomHandleInputMap.get(( argsList[0], 'leftMouseUp' ))
		if screen :
			return screen.leftMouseUp( argsList[1:] )
	
	def rightMouseUp (self, argsList):
		screen = self.CustomHandleInputMap.get(( argsList[0], 'rightMouseUp' ))
		if screen :
			return screen.rightMouseUp( argsList[1:] )

	def leftMouseDown (self, argsList):
		screen = self.CustomHandleInputMap.get(( argsList[0], 'leftMouseDown' ))
		if screen :
			return screen.leftMouseDown( argsList[1:] )
			
	def rightMouseDown (self, argsList):
		screen = self.CustomHandleInputMap.get(( argsList[0], 'rightMouseDown' ))
		if screen :
			return screen.rightMouseDown( argsList[1:] )
	
	def mouseOverPlot(self, argsList):
		screen = self.CustomHandleInputMap.get(( argsList[0], 'mouseOverPlot' ))
		if screen :
			return screen.mouseOverPlot( argsList[1:] )
	
	# Forced screen update
	def forceScreenUpdate (self, argsList):
		screen = self.CustomHandleInputMap.get(( argsList[0], 'updateScreen' ))
		if screen :
			return screen.updateScreen()
	
	# Forced redraw
	def forceScreenRedraw (self, argsList):
		screen = self.CustomHandleInputMap.get(( argsList[0], 'redraw' ))
		if screen :
			return screen.redraw()
	
	# Minimap Clicked
	def minimapClicked(self, argsList):
		screen = self.CustomHandleInputMap.get(( argsList[0], 'minimapClicked' ))
		if screen :
			return screen.minimapClicked()
	
	def movieDone(self, argsList):
		screen = self.CustomHandleInputMap.get(( argsList[0], 'movieDone' ))
		if screen :
			return screen.movieDone()
