# $Source: /cvsroot/civ4geneticai/Assets/Python/CvSimCutieEventManager.py,v $: $Name:  $
#	Autoconfig'ing Event Manager for Sid Meier's Civilization 4
#	$CopyRight$
#	by SimCutie < simcutie (AT) hanmail.net >
#	Based on Civ4 Update 1.61
# CvSimCutieEventManager
## This class is called with argsList from CvAppInterface.onEvent
## The argsList can contain anything from mouse location to key info
## The EVENTLIST that are being notified can be found 
#	This file should reside in  "... /CustomAssets/python" folder.
#	Version: $Revision: 1.1 $ $Date: 2006/09/12 12:43:11 $
#
__author__ = "$Author: the_great_apple $"
__revision__ = "$Id: CvSimCutieEventManager.py,v 1.1 2006/09/12 12:43:11 the_great_apple Exp $"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2006/09/12 12:43:11 $"

############## EVENT HANDING FRAMEWORK ###################################
"""
You can use and redistribute this mod with your own mod as long as
it is of a non-profit nature and Simcutie is duely credited for this.

About SimCutie event handler module.

To use  SimCutie's event handler framework, Install folowing files under (Custom)Assets/Python/.
(Custom)Assets/Python:
	CvSimCutieEventManager.py
(Custom)Assets/Python/CUSTOM:
	PyBaseLib.py, PyExtLib.py, PyCivSim.py, CvCustomEvent.py

HOW TO install custom event handler for your MOD.

A python module in  your MOD should register custom event handlers in boot time.
The custom event handler register function should be  a *global* function named as
EventHandlerRegister(). Each MOD has its own EventHandlerRegister()
SYNOPSIS: EventHandlerRegister(eventManager, unused)

Arg. eventManager is instance of class CvSimCutieEventManager.
Return value should be a dict of custom event handeler pair.
Each dict entry is pair of "event-name" : event_handler_function.
For full list of valid event names, see "CvEnetManger.py" in 
"Assets/Python" folder under Your Civ4 game install directory.
example 1)
	EventHandlerRegister( eventManager, unused ):
		# CvUtil.pyPrint("MY MOD event register OK")
		return { "Init": Inithandler1, 
				"unitMove-" : OnMove, 
				"BeginPlayerTurn+" : my_obj.OnPlayerTurn }

Event may be normal events name string like "OnLoad", "buildingBuilt" (case-sensitive),
"+" or "-" sign after the event name is handler priority which affects handler calling order.
"event+" handlers are called before any normal "event" (including Fraxis original)
are processed. "event-" handlers are processed after Firaxis original event.

Or alternatively, you can call registering function directly than returning dict.
SYNOPSIS: addEventHandler(self, event, handler, priority = PRIORITY_NORMAL )
example 2)
	EventHandlerRegister( eventManager, unused ):
		# CvUtil.pyPrint("MY MOD event register OK")
		eventManager.addEventHandler( "Init", Inithandler1 )
		eventManager.addEventHandler( "unitMove", OnMove, eventManager.PRIORITY_LOW)
		eventManager.addEventHandler( "BeginPlayerTurn", my_obj.OnPlayerTurn,
										eventManager.PRIORITY_HIGH )

Both example 1, 2 will works in exactly same way.

The python file with EventHandlerRegister() should reside in one of these directories 
in Civ4 game data folder under User's My Document folder. 
These directory will be searched recursively in trees of its subdirectories. 
1) .../CustomAssets/python/custom/
2) .../CustomAssets/python/screen/
3) .../MODS/<mod-name>/assets/python/

Each EventHandlerRegister() of the all the installed MOD's is invoked one by one 
by CvSimCutieEventManager.EventHandlerSetup() at Civ4 application launch time. 
(Just before Python "Init" event time) and requested custom handlers are installed.

When the event happens, custom handlers are called one-by-one.
If any of the handler returns 1, original Firaxis handler will not be invoked.
If it returns 0x000F0000, handling of the event will be finished immediately.

Extended Keyboard/Mouse events Service.
Some keyboard/Mouse events are added other than standard 'kbdEvent', 'mouseEvent' events.
They are :	"LButtonDown", "LcButtonDblClick", "RButtonDown", "wheelBack", 
			"wheelForward", "KeyDown", KeyUp"

All event handler has single argument, argsList. See CvEventManager for details.

Keypress / mouse handler args change: 
Instead of standard argumentsm Keyboard/Mouse args are extended to a tuple of 8 elements.

Keypress:	( bool isKeyDownEvent, int theKey,  mx, my, px, py, 
				( keyModifier, bShift, bCtrl, bAlt), Unused )
Mouse :		( bool isLButtonDownEvent, int eventType, mx, my, px, py, 
				( keyModifier, bShift, bCtrl, bAlt), Unused )

keyModifier is result of ( bShift | ( bControl << 1 ) | ( bAlt << 2))

onSave()/onLoad() handler should use encodeSave()/decodeLoad() to save/load string data.
	encodeSave(self, saveStr, clientName, clientVer = "1" )
	decodeLoad(self, loadStr, clientName, clientVer = "1" )

CUSTOM EVENTS
To define and use brand-new custom event other than standard events in "CvEventManger.py",
Define the new events in "CvCustomEvent.py" in CustomAssets/custom/ directory.
Define new event and add it to CUSTOM_EVENTDICT in "CvCustomEvent.py"

Currently following custom events are defined. See "CvCustomEvent.py" for details.
"InitEnd", "gameBegin", "optionBegin", "optionEnd", "buttonShow"
"buttonAction", "unitReborn", "combatBegin", "combatHit".

ex) "gameBegin" event is union event of 'GameStart' event *OR* 'OnLoad' event.

Symbolic custom event name 
custom event has alternate symblic name like EVENT_OPTIONBEGIN
It can be used like CvCustomEvent.EVENT_OPTIONBEGIN or "EVENT_OPTIONBEGIN".

Changed Event Behavior by Extension DLL:
'unitCreated', 'unitLost' event are not tiggered any longer on unit upgrade.
Instead, it will trigger 'unitReborn' event with argsList = ( pUpgradedUnit, pOldUnit, cuase )

SDK genericEvent Argument
Arguments from Events triggered by SDK generic event API has changed for easier argument access.
(gDLL->getEventReporterIFace()->genericEvent ())

If hander is set to string "print", event name and args. will be printed 
on screen and log file for debugging.

If getModulePath() is used in EventHandlerRegister() of a module, 
It will return full path of the module that called the getModulePath().
"""
#
import os, time

from PyBaseLib import isInt, isBool, isTuple, isString, isDict, isSeq, className
from PyExtLib import ( path_join, path_isfile, path_listdir, path_normalize, 
						qqDebug, qPrint, reportError )
import PyCivSim
import CvEventManager

# globals
###################################################
# *TODO* should I add more folders for proper init odering? ( like python/library )
_PYTHON_FOLDER1 =		"python/"
_PYTHON_FOLDER2 =		"python/custom/"
_PYTHON_FOLDER3 =		"python/screens/"

EVENTHANDLER_REGISTER =	"EventHandlerRegister"

FIRAXIS_HANDLER = "CvEventManager"
TEST_HANDLER = "print"

HEADMARK = "\xfe\xfcH!1"		# version1
BODYMARK = "\xfe\xfcB!"
ENDMARK =  "\xfe\xfcE!"
DELIMETER = "\xfe\xfcD!"

MASK_ALLHANDLER			= 0x000F0000
#MASK_MINUSHANDLER		= 0x00000200
MASK_EVENTHANDLED		= 0x000000FF

class CvSimCutieEventManager(CvEventManager.CvEventManager):
	
	PRIORITY_HIGH = 2
	PRIORITY_NORMAL = 4
	PRIORITY_LOW = 6

	def EventHandlerSetup(self):
		
		self.parent = CvEventManager.CvEventManager
		#################### ON EVENT MAP ######################

		# call original handleEventr to invoke original event handler?
		self.option_use_original = False
		
		self.CustomEventHandlerMap = {}
		self.SetupMap = {}
		self.PopupEvents = {}
		# lookup key/mouse eventType to event name
		self.KeyMouseEvents =  [ -1, "LButtonDown", "LcButtonDblClick", "RButtonDown",
			"wheelBack", "wheelForward", 'KeyDown', 'KeyUp' ]
		
		try:
			import CvCustomOption
			PyCivSim.useOptionSection("SYSTEM")
			# Master switch. If True, disable all the custom event handlers.
			if PyCivSim.getCustomOption( "SYSTEM", "disable_all_mod", False ) :
				return
			if PyCivSim.getCustomOption("SYSTEM", "python_debug", False):
				qqDebug(1)
		except ImportError :
			pass

		import CvCustomEvent
		self.AllEventList = self.EventHandlerMap.keys()
		# Mouse event names 
		self.AllEventList += self.KeyMouseEvents[1:]

		# Custom event names. Add more events in "custom/CvCustomEvent.py"
		self.AllEventList += [ CvCustomEvent.EVENT_GAMEBEGIN ]

		for key, event in CvCustomEvent.CUSTOM_EVENTDICT.items() : # add custom_events
			if key not in self.AllEventList and  event not in self.AllEventList:
				self.AllEventList.append( key )
				self.AllEventList.append( event )

		self.ActiveMODPath = ""
		modPathSDK = ""
		if PyCivSim.hasExtGC() :
			# SDK DLL init/uninit stubs. 
			# These handlers should have the highest priorities of all. 
			CyExtGC = PyCivSim.getExtGC()
			self.addEventHandler('Init', CyExtGC.onInit, self.PRIORITY_HIGH )
			self.addEventHandler('UnInit', CyExtGC.onUnInit, self.PRIORITY_HIGH )
			self.addEventHandler('OnLoad', CyExtGC.onLoad, self.PRIORITY_HIGH )
			self.addEventHandler('GameStart', CyExtGC.onGameStart, self.PRIORITY_HIGH )

			modPathSDK = PyCivSim.getActiveMODPath(True)
			if modPathSDK :
				self.ActiveMODPath = modPathSDK
				modPathSDK = path_normalize(modPathSDK)
			else :
				modPathSDK = "NONE"

		# search folders and find python files
		pyfolders = []

		for look in self.AssetsPathList():
			if "/mods/" in path_normalize(look) :
				pyfolders.append( path_join ( look, _PYTHON_FOLDER1 ))
			else:
				pyfolders.append( path_join ( look, _PYTHON_FOLDER2 ))
				pyfolders.append( path_join ( look, _PYTHON_FOLDER3 ))
		
		dirs = []
		for top in pyfolders :
			 dirs += os.walk( top, True )

		for ( folder, dirnames, filenames ) in dirs :
			isMODp = False
			importOK = False
			norm_folder = path_normalize(folder)
			if  "/mods/" in norm_folder :
				isMODp = True
				if modPathSDK :
					if not norm_folder.startswith( modPathSDK ) :
						continue	# Not selected MOD
			
			for filename in filenames :
				mod_name, ext = os.path.splitext( filename )
				if ext.lower() != ".py" or mod_name.startswith("__") :
					continue
#				# ignore python file with same name with original "Assets/Python/*.py
#				if path_isfile( PyCivSim.CIV4_INSTALLDIR, "Assets/Python", filename ):
#					continue 

				# import the module, add handler if exist
				try :	# this will fail on not selected mod python file
					module = __import__ ( mod_name )
				except ImportError:
					importOK = False
					if not isMODp :
						raise
					else:
						continue
				importOK = True
				# Try to import "EventHandlerRegister()".
				try:
					call_setup = getattr( module,  EVENTHANDLER_REGISTER , None)
				except AttributeError:
					continue
				if not call_setup :
					continue
				
				# execute  "EventHandlerRegister()" to get response
				# response should be dict of custom event handeler map 
				self.moduleFullPath = path_join(folder, filename)
				response = call_setup( self, None )
				if not response :
					continue
				if not isDict(response) :
					reportError(self, "invalid event register response: " + mod_name)
					continue
				
				for event, handler in response.items() :
					# parse priority and append event to the event map
					if event[-1] == "+" :
						priority = self.PRIORITY_HIGH
						event =  event[:-1]
					elif event[-1] == "-" :
						priority = self.PRIORITY_LOW
						event =  event[:-1]
					else :
						priority = self.PRIORITY_NORMAL
					self.addEventHandler(event, handler, priority)

			# for non-SDK, find out MOD folder
			if not modPathSDK and not self.ActiveMODPath and isMODp and importOK :
				p2 = norm_folder.find( "/assets/", norm_folder.find( "/mods/" ) )
				self.ActiveMODPath = folder[:p2+1] # [ p1 + 6 : p2 ]

#		# It may report incorrectly. See CAVEAT in "PyCivSim.py/getActiveMODPath()"
#		if self.ActiveMODPath :
#			qPrint("Selected MOD Path is", self.ActiveMODPath )
		
		qPrint ( "Custom event handlers")
		for event, handlers in self.SetupMap.items():
			if handlers :
				qPrint( " *EVENT" + str(event), handlers)

	def addEventHandler(self, event, handler, priority = PRIORITY_NORMAL ):
		" add a event handler to eventMap"
		import CvCustomEvent
		assert priority in ( self.PRIORITY_HIGH, self.PRIORITY_NORMAL, self.PRIORITY_LOW )
		if (  priority not in ( self.PRIORITY_HIGH, self.PRIORITY_NORMAL, self.PRIORITY_LOW )
			or not isString(event) or (event not in self.AllEventList ) ) :
			reportError(self, "invalid event/handler (%s, %s)" % (str(event), handler) )
			return False
		if not callable(handler) :
			reportError(self, "invalid handler (%s, %s)" % (event, handler) )
			if handler == TEST_HANDLER :
				# valid event name but Null handler, provide default handler (for debugging.)
				handler = self.printEvent
			else :
				return False
		# translate symbolic name("EVENT_BUTTONSHOW") into real event name ('buttonShow')
		if event in CvCustomEvent.CUSTOM_EVENTDICT :
			event = CvCustomEvent.CUSTOM_EVENTDICT[event]

		if event == CvCustomEvent.EVENT_GAMEBEGIN :
			self.addEventHandler( 'GameStart', handler, priority )
			self.addEventHandler( 'OnLoad', handler, priority )
			return True
		
		# register new event handler.
		if (event, priority) in self.SetupMap :
			# append unique...
			if handler not in self.SetupMap[ (event, priority) ] : 
				self.SetupMap[ (event, priority)].append( handler )
		else:
			self.SetupMap[ (event, priority) ] = [ handler ]

		# merge it
		self.CustomEventHandlerMap[ event ] = (
			self.SetupMap.get((event, self.PRIORITY_HIGH), [])
			+ self.SetupMap.get((event, self.PRIORITY_NORMAL), []) + [ FIRAXIS_HANDLER ]
			+ self.SetupMap.get((event, self.PRIORITY_LOW), []) )
		return True
	
	def removeEventHandler(self, event, handler ):
		# Experimental...
		# remove event handler from Map
		if ( isString(event) and (event in self.AllEventList ) and callable(handler) ) :
			if ( event not in self.CustomEventHandlerMap
				or handler not in self.CustomEventHandlerMap[ event ] ) :
				return False
			self.CustomEventHandlerMap[ event ].remove(handler)
			qPrint("Event removed", event )
			return True
		else :
			reportError(self, "invalid event/handler to remove: (%s, %s)" % (str(event), handler))
			return False

#################### EVENT STARTERS ######################
	def handleEvent(self, argsList):
		'EventMgr entry point'
		# extract the last 6 args in the list, the first arg has already been consumed
		self.origArgsList = argsList	# point to original
		tag = argsList[0]				# event type string
		idx = len(argsList)-6
		bDummy = False
		self.bDbg, bDummy, self.bAlt, self.bCtrl, self.bShift, self.bAllowCheats = argsList[idx:]

		if tag == "Init" or not hasattr(self, "CustomEventHandlerMap" ) :
			self.EventHandlerSetup()
		
		# call custom event handler one-by-one. 
		# Triple priority : 'event+' : first, 'event', original handler, 'event-' : last.
		# If any of custom handler of 'event+', 'event' handler	returns 1,
		# original handler is skipped. and 'event-' handler is called

		cbArgs = argsList[1:idx]
		if tag in ( 'kbdEvent', 'mouseEvent' ) :
			# parse more detailed event type on 'kbdEvent', 'mouseEvent'
			fxn_list = self.CustomEventHandlerMap.get( tag, [] )
			eventType = argsList[idx:]
			if eventType > 0 and eventType < len(self.KeyMouseEvents):
				detail = self.KeyMouseEvents[eventType]
				fxn_list = self.CustomEventHandlerMap.get( detail, [] ) + fxn_list

			if fxn_list :
				keyModifier = self.bShift + self.bCtrl * 2 + self.bAlt * 4
				modifiers = ( keyModifier, self.bShift, self.bCtrl, self.bAlt)
				# len(cooked) == 8
				if tag == 'kbdEvent' :
					eventType,key,mx,my,px,py = cbArgs
					cooked  = ( bool( eventType  == self.EventKeyDown ), 
						int(key), mx, my, px, py, modifiers, None )
				elif tag == 'mouseEvent' :
					eventType,mx,my,px,py,interfaceConsumed,screens = cbArgs
					cooked = ( bool( eventType == self.EventLButtonDown ), 
						eventType, mx, my, px, py, modifiers, None )
		else :
			fxn_list = self.CustomEventHandlerMap.get( tag )
			# decode genericArgs to normal tuple args
			if len(cbArgs) == 1 and tag not in ( 'gameUpdate', 'combatLogCalc', 'combatLogHit' ) :
				try :
					genericArgs = cbArgs[0][0]
					if genericArgs[0] != None :
						cbArgs = genericArgs
				except (TypeError, AttributeError) :
					pass

		if not fxn_list :
			if self.option_use_original :
				ret = self.parent.handleEvent(self, argsList)
			elif self.EventHandlerMap.has_key(tag) :
				fxn = self.EventHandlerMap.get(tag)
				assert callable(fxn)
				if callable(fxn) :
					ret = fxn(cbArgs)
			else:
				ret = 0
			return ret
		
		flags = 0x0 ; ret = 0
		save = "" ; savecount = 0
		for fxn in fxn_list :
			if fxn != FIRAXIS_HANDLER :
				if ( flags == MASK_ALLHANDLER ):
					break
				if tag == 'kbdEvent' :
					try :
						ret = fxn(cooked)
					except ValueError:
						ret = fxn(cbArgs)
				elif tag == 'mouseEvent' :
					'mouse handler - returns 1 if the event was consumed'
					try :
						if eventType in ( self.EventLButtonDown, self.EventLcButtonDblClick, 
								self.EventRButtonDown ):
							if ( px!=-1 and py!=-1 and not interfaceConsumed ) :
								ret = fxn(cooked)
						else :
							ret = fxn(cbArgs)
					except ValueError:
						ret = fxn(cbArgs)	# old-style mouse handler
					
					if ret == 1 :
						interfaceConsumed = True
						cbArgs = (eventType,mx,my,px,py,interfaceConsumed,screens)
						ret = 0
				else :
					ret = fxn(cbArgs)
			else:
				if (flags & MASK_EVENTHANDLED ):
					continue
#				if ( flags & MASK_MINUSHANDLER ):
#					flags |= MASK_ALLHANDLER
				if self.option_use_original :
					ret = self.parent.handleEvent(self, argsList)
				else :
					fxn = self.EventHandlerMap.get(tag)
					if callable(fxn) :
						#	assert callable(fxn)
						ret = fxn(cbArgs)

			if tag == 'OnSave' :
				if ret and isString(ret):
					save +=  ( ret + DELIMETER )
			elif tag in ( 'kbdEvent', 'mouseEvent' ) and ret > 0 :
					return ret
			elif isinstance(ret, int):
				flags |= ret
			savecount += 1

		if tag == 'OnSave' :
			return save + "$%d$%s$" % ( savecount, time.ctime() )
		else:
			return flags
		
#################### EVENT APPLY ######################	
	
	def applyEvent( self, argsList ):
		'Apply the effects of an event '
		context, playerID, netUserData, popupReturn = argsList
		
		if context not in self.PopupEvents :
			return self.parent.applyEvent(self, argsList)

		entry = self.PopupEvents[context]
		entry( playerID, netUserData, popupReturn )   # the apply function
		del self.PopupEvents[context]
		
#################### ON EVENTS ######################
		
	def printEvent(self, argsList):
		'Called on testing handler for debugging'
		from CvPythonExtensions import CyInterface 
		msg = "Event: %s %s" % ( self.origArgsList[0], str( argsList ))
		CyInterface().addImmediateMessage(msg, "")
		qPrint (msg)
		return 0
	
######## EVENT MANAGER HELPERS ###########################################

	@staticmethod
	def AssetsPathList():
		""" Returns list of full path of all CustomAssets, MODS/<mods>/assets 
		under player's MyDocuments/My Games.. """
		assets_list = [ path_join ( PyCivSim.CIV4_USERDIR, "CustomAssets/" ) ]
		mod_dir = path_join( PyCivSim.CIV4_USERDIR, "MODS/" )
		for dirent  in path_listdir( mod_dir ):
			apath = path_join( mod_dir, dirent, "assets/" )	# MODS/<MOD-NAME>/assets
			if  os.path.isdir( apath ) \
				and path_isfile( mod_dir, dirent, dirent + ".ini") :
				assets_list.append( apath )
		
		return assets_list

	
	def addPopupEvent(self, applyHandler ):
		"make a new Popup context event, see PyCivSim/makePopup()"
		contextEnum = 2100
		while contextEnum in self.PopupEvents :
			contextEnum += 1
		assert callable(applyHandler)
		self.PopupEvents[contextEnum] = applyHandler
		return contextEnum

	def getModulePath(self):
		" get full path of the called python module in EventHandlerRegister()"
		return self.moduleFullPath

	# *TODO*
	# get keyboard for given Control...
	# getControlKey("CONTROL_MY_MOD") ==> ( key value and modifier..)
	# used in keyborad handler...
	#

	@staticmethod
	def encodeSave( saveStr, clientName, clientVer = "1" ):
		" Retuenes formated save string for individual onSave() handler"
		return "%s$%s$%d$%d$%s%s%s" % ( MAGIC, clientName, clientVer, len(saveStr), 
			BODYMARK, dataStr, ENDMARK )

	@staticmethod
	def decodeLoad(loadStr, clientName, clientVer = "1" ):
		"Returns decoded load string data for onSave() handler"
		header = MAGIC + "$" + clientName + "$"
		hpos = loadStr.find(header)
		bpos = loadStr.find(BODYMARK, hpos)
		epos = loadStr.find(ENDMARK, bpos)
		if found < 0 or bpos < 0 or endpos < 0:
			reportError(__class__, "SaveData not found: " + clientName )
			return ""
		hinfo = loadStr[hpos+len(header):npos]
		real_data = loadStr[bpos + len(NECKMARK): epos ]
		
		(save_ver, save_len, extra ) = hinfo.split("$", 3 )
		if int(save_len) != len(real_data) or save_ver != clientVer :
			reportError(__class__, "SaveData version or length mismatch: " + hinfo )
		
		return real_data

##########################################################################

