## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## #####   WARNING - MODIFYING THE FUNCTION NAMES OF THIS FILE IS PROHIBITED  #####
## 
## The app specifically calls the functions as they are named. Use this file to pass 
## args to another file that contains your modifications

#
import CvUtil
import CvGameUtils
import CvEventInterface
from CvPythonExtensions import *

# globals
gc = CyGlobalContext()
normalGameUtils = CvGameUtils.CvGameUtils()

def gameUtils():
	' replace normalGameUtils with your mod version'
	return normalGameUtils

#====== ADDED BY SIMCUTIE ===============================================
#	HOW TO USE EXTENSION DLL INTERFACE ENTRYPOINT
#
#	To enable new game intarface, call  PyCivSim.activateGameEntryPoint("new" )
#	To enable old game intarface, call  PyCivSim.activateGameEntryPoint("old" )
#	To activate particular call to interface entrypoints from Civ4CCP Extension DLL,
#	call PyCivSim.activateGameEntryPoint("entryPointName" ) in "OnInit" event time. 
#
#	Example) 
#	In "YourMOD.py":
#		def OnInit(argsList):
#			"onInit event Handler - activate custom extension entry Point"
#			import PySivSim
#			PyCivSim.activateGameEntryPoint( "new" )
#			PyCivSim.activateGameEntryPoint( "canPlayerContact" )
#
#	In "CvGameUtils.py":
#		def canPlayerContact(self, argsList):
#			( pPlayer, ePlayer, bcanContactResult) = argsList[:3]
#			bcanContact = .......	# decision logic of your MOD.
#			return bcanContact
#
#	In C++ SDK:
#	bool CyExtGC::activateGameEntryPoint(LPCSTR szPyFunc):
#		This function activates Python game entry point from DLL.
#
#======= END BY SIMCUTIE =================================================

############################################################################
# START | Enhanced Game Utils | 12monkeys | v1.0 | 29-Apr-2006 
# tools
def dllDebug(argsList):
	return gameUtils().dllDebug(argsList)

# CvPlayer
def canPlayerContact(argsList):
	return gameUtils().canPlayerContact(argsList)

def canPlayerTradeItem(argsList):
	return gameUtils().canPlayerTradeItem(argsList)
	
def canPlayerTradeNetworkWith(argsList):
	return gameUtils().canPlayerTradeNetworkWith(argsList)

def canPlayerRazeCity(argsList):
	return gameUtils().canPlayerRazeCity(argsList)
	
def razeCity(argsList):
	return gameUtils().razeCity(argsList)
	
def canPlayerFoundCity(argsList):
	return gameUtils().canPlayerFoundCity(argsList)
	
def canPlayerConvert(argsList):
	return gameUtils().canPlayerConvert(argsList)
	
def playerFoundReligion(argsList):
	return gameUtils().playerFoundReligion(argsList)

# CvSelectionGroup	
def canGroupStartMission(argsList):
	return gameUtils().canGroupStartMission(argsList)
	
def groupStartMission(argsList):
	return gameUtils().groupStartMission(argsList)
	
# CvUnit	
def doUnitCommand(argsList):
	return gameUtils().doUnitCommand(argsList)

def canUnitSpread(argsList):
	return gameUtils().canUnitSpread(argsList)

def canUnitMoveInto(argsList):
	return gameUtils().canUnitMoveInto(argsList)

def canUnitFortify(argsList):
	return gameUtils().canUnitFortify(argsList)
	
def canUnitHeal(argsList):
	return gameUtils().canUnitHeal(argsList)
	
def canUnitNukeAt(argsList):
	return gameUtils().canUnitNukeAt(argsList)
	
def unitUpgradePrice(argsList):
	return gameUtils().unitUpgradePrice(argsList)	
	
def isUnitInvisible(argsList):
	return gameUtils().isUnitInvisible(argsList)	
	
def canUnitJoinGroup(argsList):
	return gameUtils().canUnitJoinGroup(argsList)		

# CvCity	
def spreadReligionToCity(argsList):
	return gameUtils().spreadReligionToCity(argsList)
	
def canCityWorkOnPlot(argsList):
	return gameUtils().canCityWorkOnPlot(argsList)

def canCityHurry(argsList):
	return gameUtils().canCityHurry(argsList)

def canCityConscript(argsList):
	return gameUtils().canCityConscript(argsList)
	
def calculateCityDistanceMaintenance(argsList):
	return gameUtils().calculateCityDistanceMaintenance(argsList)
	
def calculateCityNumCitiesMaintenance(argsList):
	return gameUtils().calculateCityNumCitiesMaintenance(argsList)
	
# END | Enhanced Game Utils | 12monkeys | v1.0
###########################################################################

def isVictoryTest():
	#CvUtil.pyPrint( "CvGameInterface.isVictoryTest" )
	return gameUtils().isVictoryTest()

def isPlayerResearch(argsList):
	#CvUtil.pyPrint( "CvGameInterface.isPlayerResearch" )
	return gameUtils().isPlayerResearch(argsList)

def createBarbarianCities():
	#CvUtil.pyPrint( "CvGameInterface.createBarbarianCities" )
	return gameUtils().createBarbarianCities()

def createBarbarianUnits():
	#CvUtil.pyPrint( "CvGameInterface.createBarbarianUnits" )
	return gameUtils().createBarbarianUnits()

def skipResearchPopup(argsList):
	#CvUtil.pyPrint( "CvGameInterface.skipResearchPopup" )
	return gameUtils().skipResearchPopup(argsList)

def showTechChooserButton(argsList):
	#CvUtil.pyPrint( "CvGameInterface.showTechChooserButton" )
	return gameUtils().showTechChooserButton(argsList)

def getFirstRecommendedTech(argsList):
	#CvUtil.pyPrint( "CvGameInterface.getFirstRecommendedTech" )
	return gameUtils().getFirstRecommendedTech(argsList)

def getSecondRecommendedTech(argsList):
	#CvUtil.pyPrint( "CvGameInterface.getSecondRecommendedTech" )
	return gameUtils().getSecondRecommendedTech(argsList)

def skipProductionPopup(argsList):
	#CvUtil.pyPrint( "CvGameInterface.skipProductionPopup" )
	return gameUtils().skipProductionPopup(argsList)

def showExamineCityButton(argsList):
	#CvUtil.pyPrint( "CvGameInterface.showExamineCityButton" )
	return gameUtils().showExamineCityButton(argsList)

def getRecommendedUnit(argsList):
	#CvUtil.pyPrint( "CvGameInterface.getRecommendedUnit" )
	return gameUtils().getRecommendedUnit(argsList)

def getRecommendedBuilding(argsList):
	#CvUtil.pyPrint( "CvGameInterface.getRecommendedBuilding" )
	return gameUtils().getRecommendedBuilding(argsList)

def updateColoredPlots():
	#CvUtil.pyPrint( "CvGameInterface.updateColoredPlots" )
	return gameUtils().updateColoredPlots()

def isActionRecommended(argsList):
	#CvUtil.pyPrint( "CvGameInterface.isActionRecommended" )
	return gameUtils().isActionRecommended(argsList)

def cannotHandleAction(argsList):
	#CvUtil.pyPrint( "CvGameInterface.cannotHandleAction" )
	return gameUtils().cannotHandleAction(argsList)

def cannotSelectionListMove(argsList):
	#CvUtil.pyPrint( "CvGameInterface.cannotSelectionListMove" )
	return gameUtils().cannotSelectionListMove(argsList)

def cannotSelectionListGameNetMessage(argsList):
	#CvUtil.pyPrint( "CvGameInterface.cannotSelectionListGameNetMessage" )
	return gameUtils().cannotSelectionListGameNetMessage(argsList)

def cannotDoControl(argsList):
	#CvUtil.pyPrint( "CvGameInterface.cannotDoControl" )
	return gameUtils().cannotDoControl(argsList)

def canResearch(argsList):
	#CvUtil.pyPrint( "CvGameInterface.canResearch" )
	return gameUtils().canResearch(argsList)

def cannotResearch(argsList):
	#CvUtil.pyPrint( "CvGameInterface.cannotResearch" )
	return gameUtils().cannotResearch(argsList)

def canDoCivic(argsList):
	#CvUtil.pyPrint( "CvGameInterface.canDoCivic" )
	return gameUtils().canDoCivic(argsList)

def cannotDoCivic(argsList):
	#CvUtil.pyPrint( "CvGameInterface.cannotDoCivic" )
	return gameUtils().cannotDoCivic(argsList)

def canTrain(argsList):
	#CvUtil.pyPrint( "CvGameInterface.canTrain" )
	return gameUtils().canTrain(argsList)

def cannotTrain(argsList):
	#CvUtil.pyPrint( "CvGameInterface.cannotTrain" )
	return gameUtils().cannotTrain(argsList)

def canConstruct(argsList):
	#CvUtil.pyPrint( "CvGameInterface.canConstruct" )
	return gameUtils().canConstruct(argsList)

def cannotConstruct(argsList):
	#CvUtil.pyPrint( "CvGameInterface.cannotConstruct" )
	return gameUtils().cannotConstruct(argsList)

def canCreate(argsList):
	#CvUtil.pyPrint( "CvGameInterface.canCreate" )
	return gameUtils().canCreate(argsList)

def cannotCreate(argsList):
	#CvUtil.pyPrint( "CvGameInterface.cannotCreate" )
	return gameUtils().cannotCreate(argsList)

def canMaintain(argsList):
	#CvUtil.pyPrint( "CvGameInterface.canMaintain" )
	return gameUtils().canMaintain(argsList)

def cannotMaintain(argsList):
	#CvUtil.pyPrint( "CvGameInterface.cannotMaintain" )
	return gameUtils().cannotMaintain(argsList)

def AI_chooseTech(argsList):
	'AI chooses what to research'
	#CvUtil.pyPrint( "CvGameInterface.AI_chooseTech" )
	return gameUtils().AI_chooseTech(argsList)

def AI_chooseProduction(argsList):
	'AI chooses city production'
	#CvUtil.pyPrint( "CvGameInterface.AI_chooseProduction" )
	return gameUtils().AI_chooseProduction(argsList)

def AI_unitUpdate(argsList):
	'AI moves units - return 0 to let AI handle it, return 1 to say that the move is handled in python '
	#CvUtil.pyPrint( "CvGameInterface.AI_unitUpdate" )
	return gameUtils().AI_unitUpdate(argsList)

def AI_doWar(argsList):
	'AI decides whether to make war or peace - return 0 to let AI handle it, return 1 to say that the move is handled in python '
	#CvUtil.pyPrint( "CvGameInterface.AI_doWar" )
	return gameUtils().AI_doWar(argsList)

def AI_doDiplo(argsList):
	'AI decides does diplomacy for the turn - return 0 to let AI handle it, return 1 to say that the move is handled in python '
	#CvUtil.pyPrint( "CvGameInterface.AI_doDiplo" )
	return gameUtils().AI_doDiplo(argsList)

def calculateScore(argsList):
	#CvUtil.pyPrint( "CvGameInterface.calculateScore" )
	return gameUtils().calculateScore(argsList)

def doHolyCity():
	#CvUtil.pyPrint( "CvGameInterface.doHolyCity" )
	return gameUtils().doHolyCity()

def doHolyCityTech(argsList):
	#CvUtil.pyPrint( "CvGameInterface.doHolyCityTech" )
	return gameUtils().doHolyCityTech(argsList)

def doGold(argsList):
	#CvUtil.pyPrint( "CvGameInterface.doGold" )
	return gameUtils().doGold(argsList)

def doResearch(argsList):
	#CvUtil.pyPrint( "CvGameInterface.doResearch" )
	return gameUtils().doResearch(argsList)

def doGoody(argsList):
	#CvUtil.pyPrint( "CvGameInterface.doGoody" )
	return gameUtils().doGoody(argsList)

def doGrowth(argsList):
	#CvUtil.pyPrint( "CvGameInterface.doGrowth" )
	return gameUtils().doGrowth(argsList)

def doProduction(argsList):
	#CvUtil.pyPrint( "CvGameInterface.doProduction" )
	return gameUtils().doProduction(argsList)

def doCulture(argsList):
	#CvUtil.pyPrint( "CvGameInterface.doCulture" )
	return gameUtils().doCulture(argsList)

def doPlotCulture(argsList):
	#CvUtil.pyPrint( "CvGameInterface.doPlotCulture" )
	return gameUtils().doPlotCulture(argsList)

def doReligion(argsList):
	#CvUtil.pyPrint( "CvGameInterface.doReligion" )
	return gameUtils().doReligion(argsList)

def doGreatPeople(argsList):
	#CvUtil.pyPrint( "CvGameInterface.doGreatPeople" )
	return gameUtils().doGreatPeople(argsList)

def doMeltdown(argsList):
	#CvUtil.pyPrint( "CvGameInterface.doMeltdown" )
	return gameUtils().doMeltdown(argsList)

######### ADDED BY SIMCUTIE ##############################################
# $Source: /cvsroot/civ4geneticai/Assets/Python/EntryPoints/CvGameInterface.py,v $: $Name:  $
#	Custom  GameInterface entry points.
#	$CopyRight$
#	by SimCutie < simcutie (AT) hanmail.net >
#	Based on Civ4 Update 1.61
#	This file should reside in  "... /CustomAssets/python/entrypoints" folder.
#	Add Python entey point called by SDK/CvExtension.cpp/callPythonEntryPoint()
#	Use PyCivSim.py/setPythonEntryPoint() to add entry point with Python script.
#	Don't edit this file manually to add new Python entry point to here.
#	Use PyCivSim.py/setPythonEntryPoint(), instead.
#
#	Version: $Revision: 1.1 $ $Date: 2006/09/12 12:43:11 $
#
__author__ = "$Author: the_great_apple $"
__revision__ = "$Id: CvGameInterface.py,v 1.1 2006/09/12 12:43:11 the_great_apple Exp $"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2006/09/12 12:43:11 $"

import sys
from PyExtLib import getFuncName
# globals
#gc = CyGlobalContext()

normalGameUtils = CvGameUtils.CvSimCutieGameUtils()

def writeLog(argsList):
	""" called from CvGameCoreDLL CvExtension.cpp /logMessage() 
	to print message to log file or screen"""
	message = argsList[0]
	if argsList[1]:
		CyInterface().addImmediateMessage(message,"")
	message += "\n"
	sys.stdout.write(message)
	return 1

try :
#	Action Button handlers 
	import ActionButton
	from PyCivSim import hasExtGC

	def handleActionButton(argsList):
		""" SDK DLL version of Action Button.
		called from CvGameCoreDLL CvPyActionInfo.doActionButton() 
		when an action button pressed """
		#CvUtil.pyPrint(__name__ + ": handleActionButton()")

		return ActionButton.handleActionButton(argsList)

	def cannotHandleAction(argsList):
		" Python version of Action Button"
		#CvUtil.pyPrint( "CvGameInterface.cannotHandleAction" )
		if not hasExtGC():
			res =  ActionButton.handleActionButtonPython(argsList)
			if res :
				return res
		return gameUtils().cannotHandleAction(argsList)

except ImportError :
	pass

# CvPyActionInfo
def cannotHandlePyAction(argsList):
	"CvPyActionInfo::doAction() failed to call Python handler function."
	# ( actionID, iData2, bOpt, bRightClick, handlerName ) = argsList[:5]
	return  gameUtils().cannotHandlePyAction(argsList)

def dummyInterface(argsList):
	"Don't delete this -- used by CvGameCoreDLL/CvExtension.cpp/callGameEntryPoint()"
	return false

def testGameInterface(argsList):
	#CvUtil.pyPrint( "CvGameInterface.testGameInterface" )
	return gameUtils().testGameInterface(argsList)

#-------------------------------------------------------------------------
def canUnitDoCommand(argsList):
	return gameUtils().canUnitDoCommand(argsList)

def willUnitDoCommand(argsList):
	return gameUtils().willUnitDoCommand(argsList)

def calculateUnitHealRate(argsList):
	return gameUtils().calculateUnitHealRate(argsList)

def calculateUnitUpgradePrice(argsList):
	return gameUtils().calculateUnitUpgradePrice(argsList)

def willSpreadReligionToCity(argsList):
	return gameUtils().willSpreadReligionToCity(argsList)

def willPlayerRazeCity(argsList):
	return gameUtils().willPlayerRazeCity(argsList)

def willPlayerFoundReligion(argsList):
	return gameUtils().willPlayerFoundReligion(argsList)

def willGroupStartMission(argsList):
	return gameUtils().willGroupStartMission(argsList)

#-------------------------------------------------------------------------
# Don't delete or rename func's above this line
# They are used in CvGameCoreDLL/"CvExtension.cpp"
#======= END BY SIMCUTIE ===============================================
