# $Source: /cvsroot/civ4geneticai/Assets/Python/Screens/CvCustomOption.py,v $: $Name:  $
#	Custom option screen or custom tab related codes 
#	$CopyRight$
#	by SimCutie < simcutie (AT) hanmail.net >
#	Based on Civ4 Update 1.61
#	This file should reside in  "... /CustomAssets/python/screens" folder.
#	See DEMO_CustomOption.py for example usaage.
#
#	Version: $Revision: 1.1 $ $Date: 2006/09/12 12:43:11 $
#
__author__ = "$Author: the_great_apple $"
__revision__ = "$Id: CvCustomOption.py,v 1.1 2006/09/12 12:43:11 the_great_apple Exp $"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2006/09/12 12:43:11 $"

"""
##########################################################################

This service consist of two service: 
1. INI file configuration parser
2. Custom option screen.

INI file configuration parser:
The INI file configuration parser is regular ini file parser with save / load confiuration data  to/from persistent data store (INI file). 
Individual players will use different INI file by their respective Civ4 player profiles. Hot-seat game is not yet supported, though.
The ini file is stored in same folder with Civ4 Profile folder:
(C:\Documents and Settings\<User_NAME>\Application Data\My Games\Sid Meier's Civilization 4\Profiles) in file named "<Civ4_player_profile_name>.ini".

 Custom option screen.
 Custom option screen is enables player can change the option value of the installed custom MODS using in-game option screen (Hotkey: Control-O). This service will add addtional "Custom" tab to game option screen and player can change configuration option of the individual custom MOD's from the screen. 
 All custom option screen related code should be added to the individual mod source itself, not  "CvCustomOption.py" or any other common file.

To use custom option service, You need to install:
SimCutie's EventHandler framework. See "SimCutieEventhandler.py" for details.
And install these file:
(Custom)Assets/Python/Screens : CvCustomOption.py
(Custom)Assets/Python/EntryPoints : CvOptionsScreenCallbackInterface.py
Library used: PyBaseLib.py, PyExtLib.py, PyDataStore.py, PyCivSim.py

This service dos not requrire SDK DLL.

STEP TO ADD CUSTOM OPTION
I will explain general step to add/use custom option in Python mod frist.

1. first, dedcide your option secton name and option name.
Let's assume that section name "MY_MOD" and it has two option.
One is "my_enable_feature" which is boolean checkbox option (True/False). default is True. 
Another is "my_selection_choice" which is dropdown selection listbox which has three list item, "First Chice", "Second Choice", "Third Choice", and represented as integer index value 0, 1, 2, respectively. Default value is index 0 ("First CHoice").

2. Add EventHandlerRegister() global funtion to *your* python mod file.
Put this Pyton file into CustomAssets/Python/{custom, screen} or MOD/<mod-name>/Assers/Python folder or subdirectory trees of these folders. If you don't need to change your option in custom option scrren, You can skip step 2 and 3, and go to step 4.
--------------------------------------------------------------------------
#
#	DEMO Example for SimCutie's Custom Option screen facility.
#	See "CvCustomOption.py" for detailed information.
#
import PyCivSim
import CvUtil

def EventHandlerRegister( evt_mgr, unused = None ):	
	import CvCustomEvent
	CvUtil.pyPrint(__name__ + ": MY Mod Event Register OK.")
#	PyCivSim.addText("TXT_KEY_OPTIONS_CB1_ENABLE_MY_MOD", "Enable My Mod Dmo")
	return { CvCustomEvent.EVENT_OPTIONBEGIN : onMyDrawOption }

def onMyDrawOption(argsList):
	optscr, canvas, Unused = argsList[:3]

	PyCivSim.useOptionSection("MY_SECTION")
	# Show Checkbox on Custom Option Screen

	optscr.addGenericCheckbox( canvas, "EanleMyModCheckbox",
		"TXT_KEY_OPTIONS_CB1_ENABLE_MY_MOD", "MY_SECTION", "my_mod_enable", False )

	# show Drop down on Custom Option Screen
	aszDropdownElements = ( 
		"TXT_KEY_OPTIONS_DD2_CHOICE1",
		"TXT_KEY_OPTIONS_DD2_CHOICE2",
		"TXT_KEY_OPTIONS_DD2_CHOICE3" )
	
	optscr.addGenericDropdown( canvas, "MySelctionDropdown", 
		"TXT_KEY_OPTIONS_MY_SELECT_LABEL", aszDropdownElements, "MY_SECTION",
		"my_dd2_choice", 0 )

def DemoCustomOption():

	if PyCivSim.getCutomOption("MY_SECTION","my_mod_enable", False) :
		PyCivSim.MSGBEBUG("CheckBox is On")
	else :
		PyCivSim.MSGBEBUG("CheckBox is OFF")

	optVal =  PyCivSim.getCutomOption("MY_SECTION","my_dd2_choice", 0)
	if optVal == 0 :
		PyCivSim.MSGBEBUG("First Dropdown selected")
	elif optVal == 1 or optVal == 2 :
		PyCivSim.MSGBEBUG("2nd or 3rd Dropdown selected")
	elif optVal == 3: 
		PyCivSim.MSGBEBUG("Last Dropdown selected")
 
#. Add appropiate TXT_KEY_'s to text XML
#	"TXT_KEY_OPTIONS_CB1_ENABLE_MY_MOD" (Enable feature checkbox label)
#	"TXT_KEY_OPTIONS_CB1_ENABLE_MY_MOD_HELP" (for tool tip),
#	"TXT_KEY_OPTIONS_MY_SELECT_LABEL" (Selection Dropdown title label)
#	"TXT_KEY_OPTIONS_MY_SELECT_LABEL_HELP",  (for tool tip),
#	"TXT_KEY_OPTIONS_DD2_CHOICE1" (Selection Dropdown item 0 "First Choice")
#	"TXT_KEY_OPTIONS_DD2_CHOICE2"
#	"TXT_KEY_OPTIONS_DD2_CHOICE3"
--------------------------------------------------------------------------

3. Then add these TEXT_KEY' entries to your CIV4GameText_Your_MOD.xml in "CustomAssets\xml\text" XML file in "CIV4GameText_Your_MOD.xml"

Now when you start up option screen and click "Custom" tab. You will see your Enable fature check box and selection Dropdown.

4. Now I will explain how to get a the option value in your python file.
	Use following 3 functions from PyCivSim.py
	This is InI file configuration parser and no need to use CustomOption screen.
	The Ini file of the given user (one file for each user Profile) is saved in 
	"<USER_HOME>\Application Data\My Games\Sid Meier's Civilization 4\Profiles\
	<ProfileN>.ini"

#	useOptionSection ( szSection )
#		-- Create or load the section if not yet created or loaded
#	getCustomOption ( szSection, szOptionName, [default = ""] )
#		-- get option value. Return value is String, bool or int/float. 
#		-- If option is not set but default value if given, set it as new value. 
#	setCustomOption ( szSection, szOptionName, value)
Ex)
	PyCivSim.useOptionSection ( "section_name")
	PyCivSim.getCustomOption ( "section_name", "option_name", default_value = "" )
	PyCivSim.setCustomOption ( "section_name", "option_name", newValue)

example) in your Python file
import PyCivSim

def Your_init_func():
	PyCivSim.useOptionSection("MY_SECTION")

def Your_Working_func():
	if PyCivSim.getCustomOption("MY_SECTION", "my_mod_enable", True) :
		do blaba

	choice = PyCivSim.getCustomOption("MY_SECTION", "my_dd2_choice", 0)
	if cjoice == 0 :
		do First Choice bla bla...
	elif cjoice == 1:
		do Second Choice bla bla
	elif cjoice == 2:
		do Third Choice bla bla
	else :
		error....
		
useOptionSection("MY_SECTION") loads the section data into memory or create the section if it does not exist in ".ini" file. So called it before using getCustomOption("MY_SECTION"...) and need to call only once through out Civ game and MOD run.

These player profile Ini configuration option parser and custom option screen are written in pure Python. 
So you can use these facility without custom DLL installed and use them in pure-python MODs

##########################################################################

Now I will explain how to add C++ SDK DLL runtime configuration option.

To add SDK DLL configuration runtime option from C++ code,
Call addExtDLLConf() of "CvExtention.cpp" in SimCutie's SDK DLL code.
Call it in 'Init' event time. It is recommanded in add it in CyExtGC:: onInit() function in "CvExtension.cpp" file
Please add appropiate comment there and explain what the config option does.
Then, the option checkbox will displayed when player opens Option Screen from Civ4 aplication main menu.

C++ API SYNOPSIS:
	GETEXTCONF(confkey)
		
	get runtime configurable DLL config option value. 
	It returns bool type.
	ex) if ( GETEXTCONF(MOD_FEATURE) )
			do_mod();

	void addExtDLLConf( LPCTSTR szConfKey, bool bInitValue, 
		LPCWSTR wszCheckboxLabel =NULL, LPCWSTR wszCheckboxToolTip =NULL )
	
	Add ExtDLL runtime configuration option. 
	Should be called in init stage 2 or later.

	param	szConfKey		- config name string (ex "LEGACY_EVENT", "USE_CITYAI")
	param	bInitValue		-  initial value set when user installed the MOD or DLL first time. (bool)
	param	wszCheckboxLabel	- Checkbox label displayed on custom Option screen. wide char. string
	param	wszCheckboxToolTip	- Checkbox tooltip. wide char. string 

If this wszCheckboxLabel is not specified or NULL string, check box will not displayed in Custom Option Screen.
So player can not change it during running Civ4 game. Python mod can change it, though.
Config option without Option screen checkbox can be chnaged by editing "<User-profile>.ini" file.

example )
// Display full unit name as "UnitType, Custom name" instead of "Custom name (UnitType)"
// Addedfor SIMCUTIE_GAMETEXT feature by SimCutie
addExtDLLConf( "UNITNAME_TYPEFIRST" ,	true,
	L"Type first in Unit Name",
	L"Display full unit name like \n'UnitType, Unique Name" );

// unitCreated/unitLost/unitUpgrade event behavior. If true, restore old behavior.
// Added for SIMCUTIE_UNITEXT feature by SimCutie
addExtDLLConf( "LEGACY_EVENT",	false );	// no checkbox

To access the option value from your C++ SDK, use GETEXTCONF(conf_name) macro.
It is definded in "CvExtension.h" or "CvExtGlobal.h"

It is definded in "CvExtension.h" or "CvExtGlobal.h"

ex)  if ( !GETEXTCONF(LEGACY_EVENT))
		do new action;
	else
		old action ; 
		
Note that conf_name argument of GETEXTCONF(conf_name) is not enclosed in double quotation.
To access it in Python, use below function in CustomAssets/python/custom/PyCivSim.py

PyCivSim.getExtDLLConf(szConfKey) 
PyCivSim.setExtDLLConf(szConfKey, bNewvalue )

example)
    import PyCivSim
	isLegacy = PyCivSim.getExtConf( "LEGACY_EVENT")
	PyCivSim.setExtConf( "UNITNAME_TYPEFIRST", True)
--------------------------------------------------------------------------------
"""
##########################################################################

from CvPythonExtensions import CyUserProfile, getAlarmHourLeft, getAlarmMinLeft, isAlarmOn
from PyCivSim import gc, localText, TXT_KEY
import PyCivSim
from PyExtLib import path_join, path_isfile, path_listdir, qqDebug, qPrint
import CvUtil

# globals
# gc = CyGlobalContext()
UserProfile = CyUserProfile()

def __getIniFileName(szProfile):
	""" 
	Find and returns string of full path name of custom ini file. 
	(same directory/same file name with Civ4 game profile data (xxxx.pfl) 
	but with .ini extenstion. On error, return empty string.
	("C:/Documents and Settings/<User-Name>/Application Data/
	My Games/Sid Meier's Civilization 4/Profiles/<Profile-name.ini>")
	"""
	if szProfile and UserProfile.isProfileFileExist(szProfile):
		for ii in range(UserProfile.getNumProfileFiles()):
			szPath = UserProfile.getProfileFileName(ii)
			# Cut off file dir and extension and compare
			if ( szPath[ szPath.find("PROFILES\\")+9:-4] == szProfile ):
				# replace extension partwith "ini"
				return ( szPath[:-4] + ".ini")
			
	return ""		# Not found.

#	CustomProfile Global variable holds single global instance of ConfigMgr class.
#		use this for accessing other methods and class data.
#	Use getCustomOption(), setCustomOption() in PyCivSim module for general use.

import PyDataStore

CustomProfile = PyDataStore.ConfigMgr()		# make a singleton instance.
CustomProfile.getCurrentConfigName = UserProfile.getProfileName
CustomProfile.getConfigFileName =  __getIniFileName
CustomProfile.reloadConfig()		# initialize

CivIVIni = PyDataStore.ConfigMgr()
CivIVIni.getCurrentConfigName =	lambda : "_Player_" 
CivIVIni.getConfigFileName = (	
	lambda x: path_join( PyCivSim.CIV4_USERDIR , "CivilizationIV.ini" ))
CivIVIni.reloadConfig()

def makeCBArgs(arg1, arg2 = "", arg3 = ""):
	" make call back arguments list for interface handler "
	return "%s::%s::%s" % (arg1, arg2, arg3)

def breakCBArgs(args):
	" parse call back arguments (used in interface handler) "
	argslist = args.split("::", 2) 
	# force list of 3 args
	return (argslist + [ "", "", "" ])[:3]

#CUSTOMCALLBACKIF = "CvCustomInterface"
CBCUSTOM = "Custom"
genericCheckboxCallback = "handleGenericCheckboxInput"
genericDropdownCallback = "handleGenericDropdownInput"

from CvCustomEvent import EVENT_OPTIONBEGIN, EVENT_OPTIONEND

def EventHandlerRegister( evt_mgr, unused = None ):
	import CvScreenEnums
	optionsScreen = CvCustomOptionScreen()
	# replace original CvOptionsScreen instance
	PyCivSim.ScreenRegister( optionsScreen, CvScreenEnums.OPTIONS_SCREEN )
	qPrint(__name__, "CustomOption screen Register OK.")	
	return { "UnInit" : onUnInit }

def onUnInit(self):
	" This is called on game UnInit event"
	CustomProfile.saveConfig()
#	CivIVIni.saveConfig()

##########################################################################
# this class instance replaces original CvOptionsScreen
# For Input see CvCustomInterface in Python\EntryPoints\
import CvOptionsScreen

class CvCustomOptionScreen(CvOptionsScreen.CvOptionsScreen):
	"Options Screen"
	
	def __init__(self):
		
		self.parent = CvOptionsScreen.CvOptionsScreen
		self.parent.__init__(self)
		
	def interfaceScreen (self):
		"Initial creation of the screen"
		self.parent.interfaceScreen(self)
#-------- ADDED BY SIMCUTIE ----------------------------------------------
		self.drawCustomOptionsTab()
#-------- END BY SIMCUTIE ------------------------------------------------
		
		
	def drawOtherTab(self):

		tab = self.pTabControl

		tab.attachVBox("OtherForm", "OtherVBox")		
					
		tab.attachScrollPanel("OtherVBox", "OtherPanel")
		tab.setLayoutFlag("OtherPanel", "LAYOUT_SIZE_HEXPANDING")
		tab.setLayoutFlag("OtherPanel", "LAYOUT_SIZE_VEXPANDING")
		
		tab.attachHBox("OtherPanel", "OtherPanelHBox")
		tab.setLayoutFlag("OtherPanelHBox", "LAYOUT_SPACING_INNERFORM")
		tab.setLayoutFlag("OtherPanelHBox", "LAYOUT_SIZE_HEXPANDING")
		
		
		########### CLOCK
		
		tab.attachVBox("OtherPanelHBox", "ClockVBox")
		tab.setLayoutFlag("ClockVBox", "LAYOUT_SIZE_HEXPANDING")
		tab.setLayoutFlag("ClockVBox", "LAYOUT_SIZE_VEXPANDING")
		
		tab.attachLabel("ClockVBox", "ClockLabel", localText.getText("TXT_KEY_OPTIONS_CLOCK", ()).upper() )

		tab.attachPanel("ClockVBox", "ClockPanel")
		tab.setStyle("ClockPanel", "Panel_Tan15_Style")
		tab.setLayoutFlag("ClockPanel", "LAYOUT_SIZE_HPREFERREDEXPANDING")
		tab.setLayoutFlag("ClockPanel", "LAYOUT_SIZE_VPREFERREDEXPANDING")
		
		tab.attachVBox("ClockPanel", "ClockPanelVBox")
		tab.setLayoutFlag("ClockPanelVBox", "LAYOUT_SIZE_HPREFERREDEXPANDING")
		tab.setLayoutFlag("ClockPanelVBox", "LAYOUT_SIZE_VPREFERREDEXPANDING")
		
		# Clock On Checkbox
		szOptionDesc = localText.getText("TXT_KEY_OPTIONS_CLOCK_ON", ())
		szCallbackFunction = "handleClockOnCheckboxInput"
		szWidgetName = "ClockOnCheckbox"
		bClockOn = UserProfile.isClockOn()
		tab.attachCheckBox("ClockPanelVBox", szWidgetName, szOptionDesc, self.callbackIFace, szCallbackFunction, szWidgetName, bClockOn)
		
		# 24 Hour Clock Checkbox
		szOptionDesc = localText.getText("TXT_KEY_OPTIONS_24CLOCK", ())
		szCallbackFunction = "handle24HourClockCheckboxInput"
		szWidgetName = "24HourClockCheckbox"
		b24HourClock = UserProfile.is24Hours()
		tab.attachCheckBox("ClockPanelVBox", szWidgetName, szOptionDesc, self.callbackIFace, szCallbackFunction, szWidgetName, b24HourClock)
		
#======== CHANGED BY SIMCUTIE ============================================
#		# Edit Box Hours
#		tab.attachLabel("ClockPanelVBox", "HoursLabel", localText.getText("TXT_KEY_OPTIONS_HOURS", ()))	# Label
#		szEditBoxDesc = str(getAlarmHourLeft())
#		szCallbackFunction = "DummyCallback"
#		szWidgetName = "AlarmHourEditBox"
#		tab.attachEdit("ClockPanelVBox", szWidgetName, szEditBoxDesc, self.callbackIFace, szCallbackFunction, szWidgetName)
#		
#		# Edit Box Mins
#		tab.attachLabel("ClockPanelVBox", "MinsLabel", localText.getText("TXT_KEY_OPTIONS_MINS", ()))	# Label
#		szEditBoxDesc = str(getAlarmMinLeft())
#		szCallbackFunction = "DummyCallback"
#		szWidgetName = "AlarmMinEditBox"
#		tab.attachEdit("ClockPanelVBox", szWidgetName, szEditBoxDesc, self.callbackIFace, szCallbackFunction, szWidgetName)
#		
#		# Alarm On Checkbox
#		szOptionDesc = localText.getText("TXT_KEY_OPTIONS_ALARMON", ())
#		szCallbackFunction = "handleAlarmOnCheckboxInput"
#		szWidgetName = "AlarmOnCheckbox"
#		bAlarmOn = isAlarmOn()
#		tab.attachCheckBox("ClockPanelVBox", szWidgetName, szOptionDesc, self.callbackIFace, szCallbackFunction, szWidgetName, bAlarmOn)
#=========================================================================
		self.bDrawStage = True
		self.optionRowLimitLow = 0
		self.optionRowLimitHigh = 100
		self.optionsCount = 0
		self.optionsRowCount = 0

#		# Turn numbers On Checkbox
#		PyCivSim.useOptionSection("MAINSCR")
#		self.addGenericCheckbox( "ClockPanelVBox", "TurnNumberCheckbox",
#			"TXT_KEY_OPTIONS_TURNNUMBER", "MAINSCR", "turn_number_on", True)
#		
#		# Era On Checkbox
#		szOptionDesc = localText.getText("TXT_KEY_OPTIONS_ERAON", ())
#		self.addGenericCheckbox( "ClockPanelVBox", "EraOnCheckbox",
#			"TXT_KEY_OPTIONS_ERAON", "MAINSCR", "era_on", False)
		
		# Edit Box Alam Time (Hours and Mins)
		tab.attachLabel("ClockPanelVBox", "AlarmTimeLabel", 
			localText.getText( "TXT_KEY_OPTIONS_ALARMTIME", ()))	# Label

		
		szEditBoxDesc = "%d:%02d" % (getAlarmHourLeft(), getAlarmMinLeft())
		szCallbackFunction = "DummyCallback"
		szWidgetName = "AlarmTimeEditBox"
		tab.attachEdit("ClockPanelVBox", szWidgetName, szEditBoxDesc, self.callbackIFace,
			szCallbackFunction, szWidgetName)
		tab.setToolTip(szWidgetName, localText.getText("TXT_KEY_OPTIONS_ALARMTIME_HELP", ()))
		tab.setEnabled(szWidgetName, not isAlarmOn())
		
		# Alarm On Checkbox
		szOptionDesc = localText.getText("TXT_KEY_OPTIONS_ALARMON", ())
		szCallbackFunction = "handleAlarmOnCheckboxInput"
		szWidgetName = "AlarmOnCheckbox"
		setCtrl = isAlarmOn()
		szCallbackFunction +=  CBCUSTOM
		tab.attachCheckBox("ClockPanelVBox", szWidgetName, szOptionDesc,
			self.callbackIFace, szCallbackFunction, szWidgetName, setCtrl)
		tab.setToolTip(szWidgetName, localText.getText("TXT_KEY_OPTIONS_ALARMON_HELP", ()))

		#========== MISC OTHER TAB OPTIONS ==================================
		
		tab.attachHSeparator( "ClockPanelVBox" , "OtherMiscSeparator")
		tab.attachLabel( "ClockPanelVBox" , "OtherMiscLabel", TXT_KEY( "OPTIONS_OTHERMISC"))	# Label
		# Add here...
		
#======== END BY SIMCUTIE ================================================
		
		
		########### PROFILE
		
		UserProfile.loadProfileFileNames()
		
		tab.attachVBox("OtherPanelHBox", "ProfileVBox")
		tab.setLayoutFlag("ProfileVBox", "LAYOUT_SIZE_HEXPANDING")
		tab.setLayoutFlag("ProfileVBox", "LAYOUT_SIZE_VEXPANDING")
		
		tab.attachLabel("ProfileVBox", "ProfileLabel", localText.getText("TXT_KEY_OPTIONS_SCREEN_PROFILES", ()).upper() )

		tab.attachPanel("ProfileVBox", "ProfilePanel")
		tab.setStyle("ProfilePanel", "Panel_Tan15_Style")
		tab.setLayoutFlag("ProfilePanel", "LAYOUT_SIZE_HPREFERREDEXPANDING")
		tab.setLayoutFlag("ProfilePanel", "LAYOUT_SIZE_VPREFERREDEXPANDING")
		
		tab.attachVBox("ProfilePanel", "ProfilePanelVBox")
		tab.setLayoutFlag("ProfilePanelVBox", "LAYOUT_SIZE_HPREFERREDEXPANDING")
		tab.setLayoutFlag("ProfilePanelVBox", "LAYOUT_SIZE_VPREFERREDEXPANDING")


		# Profiles Dropdown
		
		tab.attachLabel("ProfilePanelVBox", "ProfileComboLabel", localText.getText("TXT_KEY_OPTIONS_SCREEN_PROFILES", ()))
		
		szDropdownDesc = "ProfilesDropdownBox"
		aszDropdownElements = ()
		iInitialSelection = 0
		for iProfileLoop in range(UserProfile.getNumProfileFiles()):
			szProfileFileName = UserProfile.getProfileFileName(iProfileLoop)
			
			# Cut off file path and extension
			szProfile = szProfileFileName[szProfileFileName.find("PROFILES\\")+9:-4]
			
			aszDropdownElements = aszDropdownElements + (szProfile,)
						
			if (UserProfile.getProfileName() == szProfile):
				iInitialSelection = iProfileLoop
			
		szCallbackFunction = "handleProfilesDropdownInput"
		szWidgetName = "ProfilesDropdownBox"
		# iInitialSelection set above
#-------- ADDED BY SIMCUTIE ----------------------------------------------
		szCallbackFunction += CBCUSTOM
#-------- END BY SIMCUTIE ------------------------------------------------
		tab.attachDropDown("ProfilePanelVBox",szWidgetName,szDropdownDesc, aszDropdownElements, self.callbackIFace, szCallbackFunction, szWidgetName, iInitialSelection)
		
		# Edit Box ProfileName
		tab.attachLabel("ProfilePanelVBox","ProfilesName",localText.getText("TXT_KEY_OPTIONS_SCREEN_PROFILE_NAME", ()))	# Label
		
	
		#szCallbackIFace = ""
		szEditBoxDesc = UserProfile.getProfileName()
		szCallbackFunction = "DummyCallback"
		szWidgetName = "ProfileNameEditBox"
		szWideEditBoxDesc = CvUtil.convertToUnicode(szEditBoxDesc)
		tab.attachEdit("ProfilePanelVBox", szWidgetName, szWideEditBoxDesc, self.callbackIFace, szCallbackFunction, szWidgetName)
		
		# New Profile Button
		szOptionDesc = localText.getText("TXT_KEY_OPTIONS_NEW_PROFILE", ())
		szCallbackFunction = "handleNewProfileButtonInput"
		szWidgetName = "NewProfileButton"
#-------- ADDED BY SIMCUTIE ----------------------------------------------
		szCallbackFunction += CBCUSTOM
#-------- END BY SIMCUTIE ------------------------------------------------
		tab.attachButton("ProfilePanelVBox", szWidgetName, szOptionDesc, self.callbackIFace, szCallbackFunction, szWidgetName)
		
		# Delete Profile Button
		szOptionDesc = localText.getText("TXT_KEY_OPTIONS_DELETE_PROFILE", ())
		szCallbackFunction = "handleDeleteProfileButtonInput"
		szWidgetName = "DeleteProfileButton"
#-------- ADDED BY SIMCUTIE ----------------------------------------------
		szCallbackFunction += CBCUSTOM
#-------- END BY SIMCUTIE ------------------------------------------------
		tab.attachButton("ProfilePanelVBox", szWidgetName, szOptionDesc, self.callbackIFace, szCallbackFunction, szWidgetName)
		
		
		########## NETWORKING		
		
		tab.attachVBox("OtherPanelHBox", "NetVBox")
		tab.setLayoutFlag("NetVBox", "LAYOUT_SIZE_HEXPANDING")
		tab.setLayoutFlag("NetVBox", "LAYOUT_SIZE_VEXPANDING")
		
		tab.attachLabel("NetVBox", "NetLabel", localText.getText("TXT_KEY_OPTIONS_NETWORK", ()).upper() )

		tab.attachPanel("NetVBox", "NetPanel")
		tab.setStyle("NetPanel", "Panel_Tan15_Style")
		tab.setLayoutFlag("NetPanel", "LAYOUT_SIZE_HPREFERREDEXPANDING")
		tab.setLayoutFlag("NetPanel", "LAYOUT_SIZE_VPREFERREDEXPANDING")
		
		tab.attachVBox("NetPanel", "NetPanelVBox")
		tab.setLayoutFlag("NetPanelVBox", "LAYOUT_SIZE_HPREFERREDEXPANDING")
		tab.setLayoutFlag("NetPanelVBox", "LAYOUT_SIZE_VPREFERREDEXPANDING")

		# Radio Buttons
		tab.attachLabel("NetPanelVBox", "NetBandwidthLabel", localText.getText("TXT_KEY_OPTIONS_BANDWIDTH_DESC", ()) )
		
		bIsModem = gc.getGame().isModem()
		szCallbackFunction = "handleBroadbandSelected"
		szWidgetName = "BroadbandSelection"
		szWidgetLbl = localText.getText("TXT_KEY_OPTIONS_BROADBAND_LBL", ())
		tab.attachRadioButton("NetPanelVBox", szWidgetName, szWidgetLbl, self.callbackIFace, szCallbackFunction, str(szWidgetName), (not bIsModem))
		
		szCallbackFunction = "handleModemSelected"
		szWidgetName = "ModemSelection"
		szWidgetLbl = localText.getText("TXT_KEY_OPTIONS_MODEM_LBL", ())
		tab.attachRadioButton("NetPanelVBox", szWidgetName, szWidgetLbl, self.callbackIFace, szCallbackFunction, str(szWidgetName), bIsModem)


		########## EXIT

		tab.attachHSeparator("OtherVBox", "OtherExitSeparator")
		
		tab.attachHBox("OtherVBox", "LowerHBox")
		tab.setLayoutFlag("LowerHBox", "LAYOUT_HCENTER")
		
		szOptionDesc = localText.getText("TXT_KEY_OPTIONS_RESET", ())
		szCallbackFunction = "handleOtherReset"
		szWidgetName = "OtherOptionsResetButton"
		tab.attachButton("LowerHBox", szWidgetName, szOptionDesc, self.callbackIFace, szCallbackFunction, szWidgetName)
		
		szOptionDesc = localText.getText("TXT_KEY_PEDIA_SCREEN_EXIT", ())
		szCallbackFunction = "handleExitButtonInput"
		szWidgetName = "OtherOptionsExitButton"
#-------- ADDED BY SIMCUTIE ----------------------------------------------
		szCallbackFunction += CBCUSTOM
#-------- END BY SIMCUTIE ------------------------------------------------
		tab.attachButton("LowerHBox", szWidgetName, szOptionDesc, self.callbackIFace, szCallbackFunction, szWidgetName)

		tab.setLayoutFlag(szWidgetName, "LAYOUT_HCENTER")
		

######### ADDED BY SIMCUTIE #############################################
	
	def refreshCustomScreen (self):
		#	add code for refreshing custom option tab here..
		pass	# not yet implemented
	
	def resetCustomOptions(self):
		pass	# not yet implemented

	def addVFrame(self, szParent, prefix, titleKey ):
		" Add vertical pane in the tab"
		
		tab = self.pTabControl
		myVBox = prefix + "VBox"
		myPanel = prefix + "Panel"
		myCanvas = prefix + "PanelVBox"
		
		tab.attachVBox(szParent, myVBox)
		tab.setLayoutFlag(myVBox, "LAYOUT_SIZE_HEXPANDING")
		tab.setLayoutFlag(myVBox, "LAYOUT_SIZE_VEXPANDING")
		tab.attachLabel(myVBox, "PopupLabel", localText.getText(titleKey, ()).upper() )
		
		tab.attachPanel(myVBox, myPanel)
		tab.setStyle(myPanel, "Panel_Tan15_Style")
		tab.setLayoutFlag(myPanel, "LAYOUT_SIZE_HPREFERREDEXPANDING")
		tab.setLayoutFlag(myPanel, "LAYOUT_SIZE_VPREFERREDEXPANDING")
		
		tab.attachVBox(myPanel, myCanvas)
		tab.setLayoutFlag(myCanvas, "LAYOUT_SIZE_HPREFERREDEXPANDING")
		tab.setLayoutFlag(myCanvas, "LAYOUT_SIZE_VPREFERREDEXPANDING")
		
		return myCanvas
	
	def addGenericCheckbox( self, canvas, szWidgetName, szDescKey, 
			szSection, szOption, bDefault, toolTip = "", bEnable=True ) :
		""" add generic check box to custom tab. See excamples below
			szDescKey is TXT_KEY_ of check box label"""

		if self.willDrawNow(canvas, szSection, szWidgetName, 1) :
			szOptionDesc = localText.getText(szDescKey, ())
			szCBArgs = makeCBArgs( szWidgetName, szSection, szOption )
			setCtrl = PyCivSim.getCustomOption( szSection, szOption, bDefault )
			self.pTabControl.attachCheckBox( canvas, szWidgetName, szOptionDesc, self.callbackIFace,
				genericCheckboxCallback, szCBArgs, bool(setCtrl))
			self.setToolTip(szWidgetName, toolTip, szDescKey )
			if not bEnable :
				self.pTabControl.setEnabled( szWidgetName, False )
			
	def addGenericDropdown( self, canvas, szWidgetName, szDescKey, szDropdownElements,
			szSection, szOption, iDefault, toolTip = "", bEnable=True ):
		""" add generic dropdown box to custom tab. See excamples below.
			szDropdownElements are TXT_KEY_ of dropdown list text"""
		
		if self.willDrawNow(canvas, szSection, szWidgetName, 2) :
			tab = self.pTabControl
			tab.attachLabel( canvas, szWidgetName + "Label", 
				localText.getText(szDescKey, ()))
			szDropdownDesc =szWidgetName + "Desc"
			szDropdownElements = tuple(map ( lambda k: localText.getText(k, ()), szDropdownElements))
			szCBArgs = makeCBArgs( szWidgetName, szSection, szOption )
			setCtrl = PyCivSim.getCustomOption( szSection, szOption, iDefault )
			tab.attachDropDown( canvas, szWidgetName, szDropdownDesc, szDropdownElements,
				self.callbackIFace, genericDropdownCallback, szCBArgs, int(setCtrl))
			tab.setLayoutFlag(szWidgetName, "LAYOUT_SIZE_HFIXEDEXPANDING")
			tab.setLayoutFlag(szWidgetName, "LAYOUT_LEFT")
			self.setToolTip(szWidgetName, toolTip, szDescKey )
			if not bEnable :
				self.pTabControl.setEnabled( szWidgetName, False )
	
	def addCivIVIniCheckbox(self, canvas, szWidgetName, szDescKey,
			szSection, szOption, iDefault, toolTip = "") :
		"generic checkbox for Civilization.ini file"
		
		szOptionDesc = localText.getText(szDescKey, ())
		szCBArgs = makeCBArgs( szWidgetName, szSection, szOption )
		setCtrl = CivIVIni.getOption( szSection, szOption, iDefault )
		self.pTabControl.attachCheckBox( canvas, szWidgetName, szOptionDesc, self.callbackIFace,
			"handleCivIVIniCheckboxInput", szCBArgs, int(setCtrl))
		self.setToolTip(szWidgetName, toolTip, szDescKey )

	def setToolTip(self, szWidgetName, toolTip, szDescKey = ""):
		""" add tooltip to the Widget. 
		default tooltip text key is Widget description text key  + "_HELP".
		ex) "TXT_KEY_OPTIONS_TECHSPLASH" =>"TXT_KEY_OPTIONS_TECHSPLASH_HELP" """
		
		tab = self.pTabControl
		if toolTip :
			tab.setToolTip(szWidgetName, localText.getText(toolTip, ()))
		elif szDescKey :
			toolTip = szDescKey + "_HELP"
			loc_txt = localText.getText(toolTip, ())
			if loc_txt != toolTip :
				tab.setToolTip(szWidgetName, localText.getText(toolTip, ()))
			
	def willDrawNow(self, canvas, sZSection, szWidgetName, rows):

		drawP =  ( self.optionsRowCount >= self.optionRowLimitLow
				 and self.optionsRowCount < self.optionRowLimitHigh )

		self.optionsCount += 1
		self.optionsRowCount += rows

		return self.bDrawStage and drawP

	####################### DRAW CUSTOM TAB ##############################
	
	def drawCustomOptionsTab(self):
		" Draw custom tab"
		tab = self.pTabControl
		
		tab.attachTabItem("CustomForm", TXT_KEY( "OPTIONS_TTL_CUSTOMTAB" ))
		tab.attachVBox("CustomForm", "CustomVBox")		
					
		tab.attachScrollPanel("CustomVBox", "CustomPanel")
		tab.setLayoutFlag("CustomPanel", "LAYOUT_SIZE_HEXPANDING")
		tab.setLayoutFlag("CustomPanel", "LAYOUT_SIZE_VEXPANDING")
		
		tab.attachHBox("CustomPanel", "CustomPanelHBox")
		tab.setLayoutFlag("CustomPanelHBox", "LAYOUT_SPACING_INNERFORM")
		tab.setLayoutFlag("CustomPanelHBox", "LAYOUT_SIZE_HEXPANDING")
		
		#------------------------------------------------------------------
		########## EXIT
		
		tab.attachHSeparator("CustomVBox", "CustomExitSeparator")
		
		tab.attachHBox("CustomVBox", "LowerHBox")
		tab.setLayoutFlag("LowerHBox", "LAYOUT_HCENTER")
		
		szOptionDesc = TXT_KEY( "OPTIONS_RESET" )
		szCallbackFunction = "handleCustomReset"
		szWidgetName = "CustomOptionsResetButton"
		tab.attachButton("LowerHBox", szWidgetName, szOptionDesc, self.callbackIFace,
			szCallbackFunction, szWidgetName)
		
		szOptionDesc = TXT_KEY( "PEDIA_SCREEN_EXIT" )
		szCallbackFunction = "handleExitButtonInput" + CBCUSTOM
		szWidgetName = "CustomOptionsExitButton"
		tab.attachButton("LowerHBox", szWidgetName, szOptionDesc, self.callbackIFace,
			szCallbackFunction, szWidgetName)
		tab.setLayoutFlag(szWidgetName, "LAYOUT_HCENTER")

		#========== DYNAMIC OPTIONS PANEL ================================
		PyCivSim.useOptionSection("SYSTEM")
		# Call event handlers of "virtual" event "optionBegin"
		# first, count total number of rows to display
		self.bDrawStage = False
		self.optionsCount = 0
		self.optionsRowCount = 0
		PyCivSim.triggerEvent(EVENT_OPTIONBEGIN, self, None, 0)
		totalrows = self.optionsRowCount

		# display first columns
		self.bDrawStage = True
		self.optionRowLimitLow = 0
		self.optionRowLimitHigh = max((totalrows+1) //2, 0)	#	2 = # of column
		self.optionsRowCount = 0
		myCanvas = self.addVFrame( "CustomPanelHBox", "ModsVBox1", "TXT_KEY_OPTIONS_TTL_MODS1")
		PyCivSim.triggerEvent(EVENT_OPTIONBEGIN, self, myCanvas, 1)

		# second columns
		self.optionRowLimitLow = self.optionRowLimitHigh
		self.optionRowLimitHigh = totalrows
		self.optionsRowCount = 0
		myCanvas = self.addVFrame( "CustomPanelHBox", "ModsVBox2", "TXT_KEY_OPTIONS_TTL_MODS2")
		PyCivSim.triggerEvent(EVENT_OPTIONBEGIN, self, myCanvas, 2)


#		#------------------------------------------------------------------
#		
#		# Show Blank movie instead of wonder movie
#		self.addGenericCheckbox( myCanvas, "WonderMovieCheckbox",
#			"TXT_KEY_OPTIONS_WONDERMOVIE", "MODS", "blank_wonder_movie", True )
#
#		# Master Switch for Custom Event Handler CheckBox
#		self.addGenericCheckbox( myCanvas, "MasterSwitchCheckbox",
#			"TXT_KEY_OPTIONS_MASTERSWITCH", "MODS", "custom_event_handler", True )
		
		#======= CIVIV.INI OPTIONS PANEL =================================
		
		myCanvas = self.addVFrame( "CustomPanelHBox", "CivIVIniVBox",
			"TXT_KEY_OPTIONS_TTL_CIVIVINI")
		
		#-----------------------------------------------------------------
		# Game Options in "CivilizationIV.ini"
		#; Specify a Mod folder (Mods\Mesopotamia), '0' for none
		#Mod = 0
		#; Set to 1 for no tech splash screens
		#NoTechSplash = 0
		#; Set to 1 for no in-game movies
		#NoMovies = 0
		#; Set to 1 for no python exception popups
		#HidePythonExceptions = 1
		#; Establish connection to Python Debugger
		#HAPDebugger = 0
		#; Show python debug msgs in IDE console
		#ShowPythonDebugMsgs = 0
		#; Enable the logging system
		#LoggingEnabled = 0
		#; Enable message logging
		#MessageLog = 0

		mods_list =[str(TXT_KEY("OPTIONS_STARTMODNONE"))]	# No Mod Selected
		current_mod = str(CivIVIni.getOption("CONFIG", "Mod" )).lower()
		mods_dir = path_join( PyCivSim.CIV4_USERDIR, "MODS/" )

		ii = 1 ; iSelected = 0
		for dirent in path_listdir( mods_dir ):
			if path_isfile( mods_dir, dirent, dirent + ".ini" ):
				mods_list.append( dirent )
				if dirent.lower() == current_mod :
					iSelected = ii
				ii += 1
		
		tab.attachLabel( myCanvas, "StartModDropdownBoxLabel", TXT_KEY( "OPTIONS_STARTMOD" ))
		szWidgetName = "StartModDropdownBox"
		szDropdownDesc = "StartModDropdownBox"
		szCallbackFunction = "handleStartModDropdownInput"
		szCBArgs = "::".join(mods_list)
		
		tab.attachDropDown( myCanvas, szWidgetName, szDropdownDesc, tuple(mods_list),
			self.callbackIFace, szCallbackFunction, szCBArgs, iSelected)
		tab.setLayoutFlag(szWidgetName, "LAYOUT_SIZE_HFIXEDEXPANDING")
		tab.setLayoutFlag(szWidgetName, "LAYOUT_LEFT")
		self.setToolTip(szWidgetName, "TXT_KEY_OPTIONS_STARTMOD_HELP" )
		
		self.addCivIVIniCheckbox(myCanvas, "NoTechSplashCheckbox",
			"TXT_KEY_OPTIONS_TECHSPLASH", "CONFIG", "NoTechSplash", 0 )
		
		self.addCivIVIniCheckbox(myCanvas, "HidePythonExceptionsCheckbox",
			"TXT_KEY_OPTIONS_HIDEPYTHONEXCEPTIONS", "CONFIG", "HidePythonExceptions", 1 )
		
		self.addCivIVIniCheckbox(myCanvas, "LoggingEnabledCheckbox",
			"TXT_KEY_OPTIONS_LOGGINGENABLED", "DEBUG", "LoggingEnabled", 0 )
		
		self.addCivIVIniCheckbox(myCanvas, "MessageLogCheckbox",
			"TXT_KEY_OPTIONS_MESSAGELOG", "DEBUG", "MessageLog", 0 )
		
		self.addCivIVIniCheckbox(myCanvas, "HAPDebuggerCheckbox",
			"TXT_KEY_OPTIONS_HAPDEBUGGER", "DEBUG", "HAPDebugger", 0 )
		# Developer only option. You should know what you are doing.
		tab.setEnabled( "HAPDebuggerCheckbox", False )
		
#		self.addCivIVIniCheckbox(myCanvas, "NoMoviesCheckbox",
#			"TXT_KEY_OPTIONS_NOMOVIES", "CONFIG", "NoMovies", 0 )
		
#		self.addCivIVIniCheckbox(myCanvas, "ShowPythonDebugMsgsCheckbox",
#			"TXT_KEY_OPTIONS_SHOWPYTHONDEBUGMSGS", "DEBUG", "ShowPythonDebugMsgs", 0 )

####################### DRAW MORE CUSTOM TAB #############################
# for further addtion of Custom "tab"

