## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005

import CvUtil
from CvPythonExtensions import *
import genAIEvents
from genAIEnums import *
import PyCivSim
from PyCivSim import PopupStyles

ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()
gc = CyGlobalContext()


class CvDawnOfMan:
	"Dawn of man screen"
	def __init__(self, iScreenID):
		self.iScreenID = iScreenID
		
		self.X_SCREEN = 500
		self.Y_SCREEN = 396
		self.W_SCREEN = 1024 
		self.H_SCREEN = 768
		
		self.W_MAIN_PANEL = 1024
		self.H_MAIN_PANEL = 768
		self.X_MAIN_PANEL = 0
		self.Y_MAIN_PANEL = 0
		
		self.iMarginSpace = 15
		
		self.X_LEFT_PANEL = self.X_MAIN_PANEL + self.iMarginSpace
		self.Y_LEFT_PANEL = self.Y_MAIN_PANEL + self.iMarginSpace
		self.W_LEFT_PANEL = int(self.W_MAIN_PANEL / 2) - self.iMarginSpace
		self.H_LEFT_PANEL = self.H_MAIN_PANEL - (self.iMarginSpace * 2) - 40
		
		self.X_CENTRE_PANEL = self.X_LEFT_PANEL + self.W_LEFT_PANEL
		self.Y_CENTRE_PANEL = self.Y_MAIN_PANEL + self.iMarginSpace
		self.W_CENTRE_PANEL = int(self.W_MAIN_PANEL / 4) - self.iMarginSpace
		self.H_CENTRE_PANEL = self.H_MAIN_PANEL - (self.iMarginSpace * 2) - 40
		
		self.X_RIGHT_PANEL = self.X_CENTRE_PANEL + self.W_CENTRE_PANEL + self.iMarginSpace
		self.Y_RIGHT_PANEL = self.Y_MAIN_PANEL + self.iMarginSpace
		self.W_RIGHT_PANEL = int(self.W_MAIN_PANEL / 4) - self.iMarginSpace
		self.H_RIGHT_PANEL = self.H_MAIN_PANEL - (self.iMarginSpace * 2) - 40
		
		self.Y_TITLES = self.Y_MAIN_PANEL + 26
		
		self.W_EXIT = 120
		self.H_EXIT = 30
		self.X_EXIT = (self.W_MAIN_PANEL - self.W_EXIT) / 2
		self.Y_EXIT = self.H_MAIN_PANEL - self.H_EXIT - 20
		
		# Replace with an AI object
		self.iAIsMoved = 0
		self.bAIsMoved = False
		self.rightListAIs = []
		self.leftListAIs = []
		self.rightIDSelected = 0
		self.leftIDSelected = 0
		
		self.bRightSelected = True
		
		
	def interfaceScreen(self):
		'Use a popup to display the opening text'
		if ( CyGame().isPitbossHost() ):
			return
		
		self.player = gc.getPlayer(gc.getGame().getActivePlayer())
		self.EXIT_TEXT = localText.getText("TXT_KEY_SCREEN_CONTINUE", ())
		
		# Create screen
		
		screen = CyGInterfaceScreen( "CvDawnOfMan", self.iScreenID )	
		screen.showScreen(PopupStates.POPUPSTATE_QUEUED, False)
		screen.showWindowBackground( False )
		screen.setDimensions(screen.centerX(0), screen.centerY(0), self.W_SCREEN, self.H_SCREEN)
		screen.enableWorldSounds( false )
		
		# Create panels
		
		# Maim
		szMainPanel = "DawnOfManMainPanel"
		screen.addPanel( szMainPanel, "", "", true, true,
			self.X_MAIN_PANEL, self.Y_MAIN_PANEL, self.W_MAIN_PANEL, self.H_MAIN_PANEL, PanelStyles.PANEL_STYLE_MAIN )
		
		# Main panels
		szLeftPanel = "DawnOfManLeftPanel"
		screen.addPanel( szLeftPanel, "", "", true, false,
			self.X_LEFT_PANEL, self.Y_LEFT_PANEL, self.W_LEFT_PANEL, self.H_LEFT_PANEL, PanelStyles.PANEL_STYLE_DAWNBOTTOM )
		
		screen.setText("LeftPanelTitle", szLeftPanel, u"<font=4b>" + "AI Info" + "</font>", CvUtil.FONT_CENTER_JUSTIFY,
			self.X_LEFT_PANEL + self.W_LEFT_PANEL / 2, self.Y_TITLES, 0, FontTypes.TITLE_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)
		
		szCentrePanel = "DawnOfManCentrePanel"
		screen.addPanel( szCentrePanel, "", "", true, true,
			self.X_CENTRE_PANEL, self.Y_CENTRE_PANEL, self.W_CENTRE_PANEL, self.H_CENTRE_PANEL, PanelStyles.PANEL_STYLE_DAWNBOTTOM )
	
		screen.setText("CentrePanelTitle", szCentrePanel, u"<font=4b>" + "Active AIs" + "</font>", CvUtil.FONT_CENTER_JUSTIFY,
			self.X_CENTRE_PANEL + self.W_CENTRE_PANEL / 2, self.Y_TITLES, 0, FontTypes.TITLE_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)
	
		szRightPanel = "DawnOfManRightPanel"
		screen.addPanel( szRightPanel, "", "", true, true,
			self.X_RIGHT_PANEL, self.Y_RIGHT_PANEL, self.W_RIGHT_PANEL, self.H_RIGHT_PANEL, PanelStyles.PANEL_STYLE_DAWNBOTTOM )

		screen.setText("RightPanelTitle", szRightPanel, u"<font=4b>" + "Stored AIs" + "</font>", CvUtil.FONT_CENTER_JUSTIFY,
			self.X_RIGHT_PANEL + self.W_RIGHT_PANEL / 2, self.Y_TITLES, 0, FontTypes.TITLE_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)		

		# Boxes and swap buttons
		screen.addListBoxGFC( "AIStoreList", "", self.X_RIGHT_PANEL + 10, self.Y_RIGHT_PANEL + 40, self.W_RIGHT_PANEL - 20, self.H_RIGHT_PANEL - 50, TableStyles.TABLE_STYLE_STANDARD)
		screen.setStyle( "AIStoreList", "Table_StandardCiv_Style")
		
		screen.addListBoxGFC( "AICurrentList", "", self.X_CENTRE_PANEL + 10, self.Y_CENTRE_PANEL + 40, self.W_CENTRE_PANEL - 20, self.H_CENTRE_PANEL - 50, TableStyles.TABLE_STYLE_STANDARD)
		screen.setStyle( "AICurrentList", "Table_StandardCiv_Style")
		
		# Fudged these positions. Need to make that a tiny bit better
		screen.setButtonGFC("moveAIIntoGame", "", "", self.X_RIGHT_PANEL - 20 , (self.H_MAIN_PANEL - self.Y_MAIN_PANEL) / 2 - 52, 32, 32, WidgetTypes.WIDGET_GENERAL, -1, -1, ButtonStyles.BUTTON_STYLE_ARROW_LEFT)
		screen.setButtonGFC("moveAIOutOfGame", "", "", self.X_RIGHT_PANEL - 27 , (self.H_MAIN_PANEL - self.Y_MAIN_PANEL) / 2 - 25, 32, 32, WidgetTypes.WIDGET_GENERAL, -1, -1, ButtonStyles.BUTTON_STYLE_ARROW_RIGHT)
	
		# Info panel
		screen.setLabel("AI Name", szLeftPanel, u"<font=3b>" + "Name: " + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 20, self.Y_TITLES + 50, 0, FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)
			
		screen.setLabel("AINumGames", szLeftPanel, u"<font=3b>" + "Number of games: "  + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 20, self.Y_TITLES + 75, 0, FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)			
			
		screen.setLabel("AIFitness", szLeftPanel, u"<font=3b>" + "Current Fitness: " + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 20, self.Y_TITLES + 100, 0, FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)
			
		screen.setLabel("AITraits", szLeftPanel, u"<font=3b>" + "Traits: " + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 20, self.Y_TITLES + 125, 0, FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)

		screen.setLabel("TraitTitle1", szLeftPanel, u"<font=3b>" + "Financial" + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 80, self.Y_TITLES + 125, 0,  FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)
			
		screen.setLabel("TraitTitle2", szLeftPanel, u"<font=3b>" + "Industrious" + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 80, self.Y_TITLES + 150, 0,  FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)
	
		screen.setLabel("TraitTitle3", szLeftPanel, u"<font=3b>" + "Agricultural" + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 80, self.Y_TITLES + 175, 0,  FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)
			
		screen.setLabel("TraitTitle4", szLeftPanel, u"<font=3b>" + "Militeristic" + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 80, self.Y_TITLES + 200, 0,  FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)
			
		screen.setLabel("TraitTitle5", szLeftPanel, u"<font=3b>" + "Imperialistic" + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 80, self.Y_TITLES + 225, 0,  FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)
			
		screen.setLabel("TraitTitle6", szLeftPanel, u"<font=3b>" + "Expansive" + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 80, self.Y_TITLES + 250, 0,  FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)
			
		screen.setLabel("TraitTitle7", szLeftPanel, u"<font=3b>" + "Creative" + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 80, self.Y_TITLES + 275, 0,  FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)
	
		screen.setLabel("TraitTitle8", szLeftPanel, u"<font=3b>" + "Spiritual" + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 80, self.Y_TITLES + 300, 0,  FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)	
			
		# Game history
		screen.addTableControlGFC("GameHistoryTable", 6, self.X_LEFT_PANEL + 10, self.Y_TITLES + 413, self.W_LEFT_PANEL - 20,
			264, true, true, 32,32, TableStyles.TABLE_STYLE_STANDARD)
		screen.setTableColumnHeader("GameHistoryTable", 0, "Game", 55) # Total graph width is 430
		screen.setTableColumnHeader("GameHistoryTable", 1, "Map Type", 100) # Total graph width is 430
		screen.setTableColumnHeader("GameHistoryTable", 2, "Map Size", 95)
		screen.setTableColumnHeader("GameHistoryTable", 3, "Game Speed", 105)
		screen.setTableColumnHeader("GameHistoryTable", 4, "Score", 55)
		screen.setTableColumnHeader("GameHistoryTable", 5, "Rank", 70)

		# Create new AI
		# screen.setButtonGFC("NewAI", "Create new AI", "", self.X_LEFT_PANEL + (self.W_LEFT_PANEL - self.W_EXIT) / 2, self.Y_TITLES + 350, self.W_EXIT, self.H_EXIT, WidgetTypes.WIDGET_GENERAL, -1, -1, ButtonStyles.BUTTON_STYLE_STANDARD )

		# Rename AI
		screen.setButtonGFC("RenameAI", "Rename AI", "", self.X_LEFT_PANEL + (self.W_LEFT_PANEL - self.W_EXIT) / 2 + 130, self.Y_TITLES + 35, self.W_EXIT, self.H_EXIT, WidgetTypes.WIDGET_GENERAL, -1, -1, ButtonStyles.BUTTON_STYLE_STANDARD )

		# Track AI
		screen.setButtonGFC("TrackAI", "Track AI", "", self.X_LEFT_PANEL + (self.W_LEFT_PANEL - self.W_EXIT) / 2 + 130, self.Y_TITLES + 65, self.W_EXIT, self.H_EXIT, WidgetTypes.WIDGET_GENERAL, -1, -1, ButtonStyles.BUTTON_STYLE_STANDARD )

		# Continue button
		screen.setButtonGFC("Continue", self.EXIT_TEXT, "", self.X_EXIT, self.Y_EXIT, self.W_EXIT, self.H_EXIT, WidgetTypes.WIDGET_CLOSE_SCREEN, -1, -1, ButtonStyles.BUTTON_STYLE_STANDARD )
		
		# Exit button
		screen.setButtonGFC("Exit", "Exit", "", self.X_EXIT + 400, self.Y_EXIT, self.W_EXIT, self.H_EXIT, WidgetTypes.WIDGET_CLOSE_SCREEN, -1, -1, ButtonStyles.BUTTON_STYLE_STANDARD )
		
		self.populateLists()
		self.updateInfoPanel()
	
	def populateLists(self):
		screen = CyGInterfaceScreen( "CvDawnOfMan", self.iScreenID )
		
		self.rightListAIs = []
		self.leftListAIs = []
		self.rightIDSelected = 0
		self.leftIDSelected = 0
		screen.clearListBoxGFC("AIStoreList")
		screen.clearListBoxGFC("AICurrentList")
		
		i = 0
		j = 0
		
		if genAIEvents.AIStore.bBadLoad == False:
			for AI in genAIEvents.AIStore.AIList:
				if AI.getAIName() == "Team Firaxis":
					screen.appendListBoxString("AIStoreList", AI.getAIName(), WidgetTypes.WIDGET_GENERAL, i, -1, 0)
					self.rightListAIs.append(AI)
					i += 1
					break
		
			for AI in genAIEvents.AIStore.AIList:
				if AI.getAIName() != "Team Firaxis":
					screen.appendListBoxString("AIStoreList", AI.getAIName(), WidgetTypes.WIDGET_GENERAL, i, -1, 0)
					self.rightListAIs.append(AI)
					i += 1
									
			iNumColumns = 0
			for AI in genAIEvents.gameAIList:
				iNumColumns += 1
				if iNumColumns <= CyGame().countCivPlayersAlive() + self.iAIsMoved:
					screen.appendListBoxString("AICurrentList", AI.getAIName(), WidgetTypes.WIDGET_GENERAL, j, -1, 0)
					self.leftListAIs.append(AI)
					j += 1
				else:
					screen.appendListBoxString("AIStoreList", AI.getAIName(), WidgetTypes.WIDGET_GENERAL, i, -1, 0)
					self.rightListAIs.append(AI)
					i += 1
		else:
			screen.appendListBoxString("AICurrentList", "AI LOAD ERROR", WidgetTypes.WIDGET_GENERAL, j, -1, 0)
			screen.appendListBoxString("AIStoreList", "AI LOAD ERROR", WidgetTypes.WIDGET_GENERAL, i, -1, 0)
			
				
		if CyGame().countCivPlayersAlive() + self.iAIsMoved >= gc.getMAX_CIV_PLAYERS():
			screen.enable("moveAIIntoGame", False)
		else:
			screen.enable("moveAIIntoGame", True)
			
		if CyGame().countCivPlayersAlive() + self.iAIsMoved <= 2:
			screen.enable("moveAIOutOfGame", False)
		else:
			screen.enable("moveAIOutOfGame", True)
			
			
	def updateInfoPanel(self):
		screen = CyGInterfaceScreen( "CvDawnOfMan", self.iScreenID )
		
		bRightPanel = self.bRightSelected
		
		if bRightPanel:
			AI = self.rightListAIs[self.rightIDSelected]
		else:
			AI = self.leftListAIs[self.leftIDSelected]
			
		szLeftPanel = "DawnOfManLeftPanel"
		
		# Stat text
		screen.setLabel("AI Name", szLeftPanel,u"<font=3b>" + "Name: " + AI.getAIName() + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 20, self.Y_TITLES + 50, 0, FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)
			
		screen.setLabel("AINumGames", szLeftPanel, u"<font=3b>" + "Number of games: " + str(AI.getNumGames()) + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 20, self.Y_TITLES + 75, 0, FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)			
			
		screen.setLabel("AIFitness", szLeftPanel, u"<font=3b>" + "Current Fitness: " + str(AI.getFitness()) + "</font>",
			CvUtil.FONT_LEFT_JUSTIFY, self.X_LEFT_PANEL + 20, self.Y_TITLES + 100, 0, FontTypes.GAME_FONT,
			WidgetTypes.WIDGET_GENERAL, -1, -1)	

		# Stat bars and +/- buttons
		for i in range(NUM_NORMAL_STATS):
			screen.addStackedBarGFC("Statbar" + str(i), self.X_LEFT_PANEL + 195, self.Y_TITLES + 100 + (25 * i), 100, 20, 1,
				WidgetTypes.WIDGET_GENERAL, -1, -1)
				
			statRanking = self.findAIStatRanking(AI, i)
			screen.setBarPercentage("Statbar" + str(i), 0, statRanking)
			if statRanking > .66:
				screen.setStackedBarColorsRGB("Statbar" + str(i),  0, 50, 250, 50, 250)
			elif statRanking < .33:
				screen.setStackedBarColorsRGB("Statbar" + str(i),  0, 250, 50, 50, 250)
			else:
				screen.setStackedBarColorsRGB("Statbar" + str(i),  0, 250, 250, 50, 250)
			
			#screen.setButtonGFC( "Plusbutton" + str(i), u"", "", self.X_LEFT_PANEL + 300, self.Y_TITLES + 100 + (25 * i), 20, 20, WidgetTypes.WIDGET_GENERAL, i, 1, ButtonStyles.BUTTON_STYLE_CITY_PLUS )
			#screen.setButtonGFC( "MinusButton" + str(i), u"", "", self.X_LEFT_PANEL + 320, self.Y_TITLES + 100 + (25 * i), 20, 20, WidgetTypes.WIDGET_GENERAL, i, -1, ButtonStyles.BUTTON_STYLE_CITY_MINUS )
			
			
		# Wipes the table... except the first value
		screen.setTableNumRows("GameHistoryTable", 1)
		screen.setTableNumRows("GameHistoryTable", 10)
		
		# Table
		for iGame in range(AI.getNumGames()):
			iIndex = iGame
			
			if iGame == 10:
				break
			
			if AI.getNumGames() > 10:
				iIndex = iGame * AI.getNumGames() / 10
			historyList = AI.getGameHistory()[iIndex]
			screen.setTableText("GameHistoryTable", 0, iGame, str(iIndex + 1), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)

			mapScriptString = historyList[0]
			
			screen.setTableText("GameHistoryTable", 1, iGame, mapScriptString, "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			screen.setTableText("GameHistoryTable", 2, iGame, gc.getWorldInfo(int(historyList[1])).getDescription(), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			screen.setTableText("GameHistoryTable", 3, iGame, gc.getGameSpeedInfo(int(historyList[2])).getDescription(), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			try:
				screen.setTableText("GameHistoryTable", 4, iGame, str(historyList[3]), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			except IndexError:
				screen.setTableText("GameHistoryTable", 4, iGame, "-", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			try:
				screen.setTableText("GameHistoryTable", 5, iGame, str(historyList[4] + 1), "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			except IndexError:
				screen.setTableText("GameHistoryTable", 5, iGame, "-", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)

		# Clears the first value
		if AI.getNumGames() == 0:
			screen.setTableText("GameHistoryTable", 0, 0, "", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			screen.setTableText("GameHistoryTable", 1, 0, "", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			screen.setTableText("GameHistoryTable", 2, 0, "", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			screen.setTableText("GameHistoryTable", 3, 0, "", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			screen.setTableText("GameHistoryTable", 4, 0, "", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)
			screen.setTableText("GameHistoryTable", 5, 0, "", "", WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)	
	
		# Track AI
		if AI.isTracked():
			screen.setButtonGFC("TrackAI", "Untrack AI", "", self.X_LEFT_PANEL + (self.W_LEFT_PANEL - self.W_EXIT) / 2 + 130, self.Y_TITLES + 65, self.W_EXIT, self.H_EXIT, WidgetTypes.WIDGET_GENERAL, -1, -1, ButtonStyles.BUTTON_STYLE_STANDARD )
		else:
			screen.setButtonGFC("TrackAI", "Track AI", "", self.X_LEFT_PANEL + (self.W_LEFT_PANEL - self.W_EXIT) / 2 + 130, self.Y_TITLES + 65, self.W_EXIT, self.H_EXIT, WidgetTypes.WIDGET_GENERAL, -1, -1, ButtonStyles.BUTTON_STYLE_STANDARD )		
	
		if AI.getAIName() == "Team Firaxis":
			screen.enable("RenameAI", False)
			screen.enable("TrackAI", False)
		else:
			screen.enable("RenameAI", True)
			screen.enable("TrackAI", True)
			
	def findAIStatRanking(self, AI, iStat):
		AIList1 = genAIEvents.AIStore.getAIList()
		AIList2 = genAIEvents.getGameAIList()
		
		AIList = AIList1 + AIList2
		
		iOurRating = AI.getStat(iStat)
		iBestRating = iOurRating
		
		for AI in AIList:
			iRating = AI.getStat(iStat)
			if iRating > iBestRating:
				iBestRating = iRating
				
		if iBestRating == 0:
			return 0.5
				
		return float(iOurRating) / iBestRating
	
	def handleInput( self, inputClass ):
		screen = CyGInterfaceScreen( "CvDawnOfMan", self.iScreenID )
		if (inputClass.getFunctionName() == "AIStoreList"):
			self.rightIDSelected = inputClass.getData1()
			self.bRightSelected = True
			self.updateInfoPanel()
			return 1
		elif (inputClass.getFunctionName() == "AICurrentList"):
			self.leftIDSelected = inputClass.getData1()
			self.bRightSelected = False
			self.updateInfoPanel()
			return 1
		elif (inputClass.getFunctionName() == "moveAIIntoGame"):
			AI = self.rightListAIs[self.rightIDSelected]
			# This will sometimes fail as I've overflowed AIs which are actually in the game but not
			# being used into the right hand list. Hence they'll actually already be in the correct store.
			try:
				genAIEvents.AIStore.AIList.remove(AI)
				genAIEvents.gameAIList.insert(0, AI)
			except ValueError:
				genAIEvents.gameAIList.remove(AI)
				genAIEvents.gameAIList.insert(0, AI)
			self.iAIsMoved += 1
			self.bAIsMoved = True
			self.populateLists()
			return 1
		elif (inputClass.getFunctionName() == "moveAIOutOfGame"):
			AI = self.leftListAIs[self.leftIDSelected]
			genAIEvents.AIStore.AIList.insert(0, AI)
			genAIEvents.gameAIList.remove(AI)
			self.iAIsMoved -= 1
			self.bAIsMoved = True
			self.populateLists()
			return 1
		elif (inputClass.getFunctionName() == "NewAI"):
			if self.bRightSelected:
				AI = self.rightListAIs[self.rightIDSelected]
			else:
				AI = self.leftListAIs[self.leftIDSelected]
			newAI = genAIEvents.AIStore.mutateAI(AI)
			genAIEvents.AIStore.AIList.insert(0, newAI)
			self.bRightSelected = True
			self.populateLists()
			self.updateInfoPanel()
			return 1
		elif (inputClass.getFunctionName() == "RenameAI"):		
			if self.bRightSelected:
				AI = self.rightListAIs[self.rightIDSelected]
			else:
				AI = self.leftListAIs[self.leftIDSelected]
				
			if AI.getAIName() == "Team Firaxis":
				return 1
				
			popup = PyCivSim.createPopup( self.renameHandler, 'Rename AI')
			popup.createEditBox(AI.getAIName())
			popup.setEditBoxMaxCharCount( 15 )
			popup.launch()
			return 1
		elif (inputClass.getFunctionName() == "TrackAI"):
			if self.bRightSelected:
				AI = self.rightListAIs[self.rightIDSelected]
				iSelected = self.rightIDSelected
				bRight = True
			else:
				AI = self.leftListAIs[self.leftIDSelected]
				iSelected = self.leftIDSelected
				bRight = False
				
			if AI.getAIName() == "Team Firaxis":
				return 1
				
			if AI.isTracked():
				AI.setTracked(False)
				for AI2 in genAIEvents.AIStore.trackedAIList:
					if AI2.getAIData() == AI.getAIData():
						genAIEvents.AIStore.trackedAIList.remove(AI)
			else:
				AI.setTracked(True)
				genAIEvents.AIStore.trackedAIList.append(AI)
		
			self.populateLists()
						
			if bRight:
				screen.setSelectedListBoxStringGFC("AIStoreList", iSelected)
				self.rightIDSelected = iSelected
			else:
				screen.setSelectedListBoxStringGFC("AICurrentList", iSelected)
				self.leftIDSelected = iSelected
				
			self.updateInfoPanel()
			return 1
		elif (inputClass.getFunctionName() == "Continue"):
			if self.bAIsMoved:
				self.restartGame(CyGame().countCivPlayersAlive() + self.iAIsMoved)
				return 0
				
		return 0
		
	def renameHandler( self, playerID, netUserData, popupReturn ):
		screen = CyGInterfaceScreen( "CvDawnOfMan", self.iScreenID )
		if self.bRightSelected:
			AI = self.rightListAIs[self.rightIDSelected]
			iSelected = self.rightIDSelected
			bRight = True
		else:
			AI = self.leftListAIs[self.leftIDSelected]
			iSelected = self.leftIDSelected
			bRight = False
	 
		if (popupReturn.getEditBoxString(0) != '') and (popupReturn.getEditBoxString(0).find(";") < 0) and (popupReturn.getEditBoxString(0) != "Team Firaxis"):
			AI.setAIName(popupReturn.getEditBoxString(0))
			self.populateLists()
			
			if bRight:
				screen.setSelectedListBoxStringGFC("AIStoreList", iSelected)
				self.rightIDSelected = iSelected
			else:
				screen.setSelectedListBoxStringGFC("AICurrentList", iSelected)
				self.leftIDSelected = iSelected
				
			self.updateInfoPanel()		
	
	def update(self, fDelta):
		return

	def restartGame( self, iNumPlayers ) :
		iWorldSize = 0 # Doesn't matter - is set inside the function. Currently not supported
		iClimate = CyMap().getClimate()
		iSeaLevel = CyMap().getSeaLevel()
		iGameSpeed = CyGame().getGameSpeedType()
		paiCivilizations = []
		paiLeaderHeads = []
		paiTeams = []
		
		i = 0
		if iNumPlayers > CyGame().countCivPlayersAlive():
			for i in range(CyGame().countCivPlayersAlive()):
				pPlayer = gc.getPlayer(i)
				paiCivilizations.append(pPlayer.getCivilizationType())
				paiLeaderHeads.append(pPlayer.getLeaderType())
				paiTeams.append(pPlayer.getTeam())
			for j in range (iNumPlayers - CyGame().countCivPlayersAlive()):
				i += 1
				
				while len(paiCivilizations) <= i:
					rand = CyGame().getSorenRandNum(gc.getNumCivilizationInfos(), "randomCiv")
					if gc.getCivilizationInfo(rand).isAIPlayable():
						if rand not in paiCivilizations:
							paiCivilizations.append(rand)	
						
				civLeaderList = []
				for k in range(gc.getNumLeaderHeadInfos()):
					if gc.getCivilizationInfo(rand).isLeaders(k):
						civLeaderList.append(k)
						
				rand = CyGame().getSorenRandNum(len(civLeaderList), "randomLeader")
				paiLeaderHeads.append(civLeaderList[rand])
				
				paiTeams.append(i)
		else:
			for i in range(iNumPlayers):
				pPlayer = gc.getPlayer(i)
				paiCivilizations.append(pPlayer.getCivilizationType())
				paiLeaderHeads.append(pPlayer.getLeaderType())
				paiTeams.append(pPlayer.getTeam())
		
		self.iAIsMoved = 0
		self.bAIsMoved = False
		
		CyGame().newGame(iWorldSize, iClimate, iSeaLevel, iGameSpeed, paiCivilizations, paiLeaderHeads, paiTeams)
			