# Genetic AI storage by TGA

from random import *
from genAIData import *
from copy import deepcopy
from genAIEnums import *

g_iNumGames = 8 # Number of games played before an AI can be analysed

from PyCivSim import CIV4_USERDIR

saveFolder = CIV4_USERDIR + "/MODS/GeneticAI/"
filePrefix = ""
fileSuffix = ""
version = 0.33
fileVersion = 0.02

try:
	filePrefix = "Best"
	f = open(saveFolder + filePrefix + "AIData" + ".txt", 'r')
except:
	filePrefix = ""
	for i in range(1000):
		try:
			f = open(saveFolder + "AIData" + str(i) + ".txt", 'r')
			fileSuffix = str(i)
			f.close()
			break
		except IOError:
			continue
			
print "----- GENETIC AI -----"
print "GenAI Version: " + str(version)
print "Data Set Loaded: " + filePrefix + "AIData" + fileSuffix + ".txt"
print "----------------------"
		
def getDefaultData():
	defaultData = [getDefaultCityData(), getDefaultGameData(), getDefaultPlayerData(),
		getDefaultSelectionData(), getDefaultTeamData(), getDefaultUnitData(), getNumCityBools() * [0],
		getNumGameBools() * [0], getNumPlayerBools() * [0], getNumSelectionBools() * [0], getNumTeamBools() * [0],
		getNumUnitBools() * [0]]

	return defaultData		
		
# AI data handler class
class createNewAI:
	# Note: * gameHistory holds all of an AI's games.
	#	* iElite is the number of consecutive times the AI
	#	  has been an elite

	def __init__(self):
		self.name = self.generateName()
		self.iFitness = 0
		self.gameHistory = []
		self.iElite = 0
		self.AIData = getDefaultData()
		self.iGeneration = 1
		# Some stats!
		self.stats = []
		for i in range(NUM_STATS):
			self.stats.append(0)
			
		self.bTrack = 0
		
	def setGeneration(self, iGen):
		self.iGeneration = iGen
		
	def getGeneration(self):
		return self.iGeneration

	def setAIName(self, name):
		self.name = name

	def getAIName(self):
		if self.isTracked():
			return self.name + " (Tracked)"
		else:
			return self.name

	def setFitness(self, iValue):
		self.iFitness = iValue

	def getFitness(self):
		return self.iFitness
		
	def setStat(self, iStat, iValue):
		self.stats[iStat] = iValue
		
	def getStat(self, iStat):
		return self.stats[iStat]
	
	def setStatList(self, statList):
		self.stats = statList
	
	def getStatList(self):
		return self.stats

	def setGameHistory(self, history):
		self.gameHistory = history

	def getGameHistory(self):
		return self.gameHistory

	def getNumGames(self):
		return len(self.getGameHistory())

	def setElite(self, iElite):
		self.iElite = iElite

	def getElite(self):
		return self.iElite

	def setAIData(self, AIData):
		self.AIData = AIData

	def getAIData(self):
		return self.AIData
		
	def setTracked(self, bTrack):
		self.bTrack = bTrack
		
	def isTracked(self):
		return self.bTrack
		
	def generateName(self):
		nameList = ("Mike", "Tron", "HAL-9000", "Deep Blue", "T-800", "Johnny 5",
		"C-3PO", "R2-D2", "Bender", "K-9", "Optimus Prime", "Anne Droid", "R. Daneel Olivaw",
		"ASIMO", "Proteus", "Colossus", "Guardian", "Locutus", "Multivac")
		name = choice(nameList) + "-" + str(randint(0,9999))
		
		return name

# Stores the AIs when not in use. Can be called to add or remove AIs from the database
class geneticAIStorage:
	def __init__(self):
		self.AIList = []
		self.deadAIList = []
		self.trackedAIList = []
		self.bTestComplete = False
		self.bBadLoad = False

		self.loadAIs()
		
		bDefault = False
		for AI in self.AIList:
			if AI.getAIName() == "Team Firaxis":
				bDefault = True
				break
		
		if not bDefault:
			defaultAI = createNewAI()
			defaultAI.setAIName("Team Firaxis")
			self.AIList.append(defaultAI)

	def getAIList(self):
		return self.AIList
		
	# Called to get the AIs to play in a game
	# AIs are removed from pool when being played
	def getAIs(self, number):
		print "GETTING AIS"
		finalAIList = []
		iNumAvailable = 0
		
		for AI in self.AIList:
			if AI.getNumGames() < g_iNumGames * (AI.getElite() + 1):
				iNumAvailable += 1
							
		if iNumAvailable == 0:
			self.bTestComplete = True
			
		while len(finalAIList) < number:
			AI = choice(self.AIList)
			if AI.getNumGames() < g_iNumGames * (AI.getElite() + 1) or iNumAvailable <= 0:	
				if AI not in finalAIList:
					finalAIList.append(AI)
					# Check it out of the main AI list. Note - quitting the game will not cause
					# the loss of the AI as they are only saved when all are in the pool
					self.AIList.remove(AI)
					iNumAvailable -= 1

		return finalAIList

	# Called after a game to add the AIs that have been
	# played back into the pool
	def recieveAIs(self, AIList):	
		for AI in AIList:
			self.AIList.append(AI)
			self.attemptAITrim(AI)
			
		self.saveAIs()
		
	def loadAIs(self):
		self.loadAIToList(filePrefix + 'AIData' + fileSuffix + '.txt', self.AIList, True)
		self.loadAIToList('TrackedAIData.txt', self.trackedAIList, False)
				
		if self.trackedAIList != []:
			for AI1 in self.trackedAIList:
				for AI2 in self.AIList:
					if AI1.getAIName() == AI2.getAIName():
						break
				else:
					self.AIList.append(AI1)
			
	def loadAIToList(self, fileName, AIList, bCheckBadLoad):
		try:
			f = open(saveFolder + fileName, "r")

			lines = f.readlines()
						
			for iLine in range(len(lines)):
				line = lines[iLine]
								
				if iLine == 0:
					if line[:2] != "//":
						version = 0.01
					else:
						version = float(line[2:])
						continue
				
				entries = line.split(";")
				
				AI = createNewAI()

				i = 0
				
				AI.setAIName(entries[i])
				i += 1
				if version > 0.01:
					AI.setGeneration(int(entries[i]))
					i += 1
				AI.setFitness(int(entries[i]))
				i += 1
				AI.setTracked(bool(int(entries[i])))
				i += 1
				AI.setElite(int(entries[i]))
				i += 1 
				iNumGameHistory = int(entries[i])
				i += 1
				AI.setStatList([int(x) for x in entries[i][1:-1].split(",")])
				i += 1
				AIGameHistory = []
				
				# AI history
				# This line is quite fiddly. Need to get the one unicode string and several numbers into an array
				for j in range(iNumGameHistory):
					tmpList1 = entries[i][1:-1]
					tmpList1 = tmpList1.split(",")
					
					tmpList2 = []
					for chars in tmpList1:
						try:
							tmpList2.append(int(chars))
						except ValueError:
							# Implies the int() operation didn't work, and we have a string
							# We want to strip out any internal strings which aren't ' blah'
							while chars.find("'") < len(chars) / 2:
								chars = chars[1:]
							while chars.rfind("'") > len(chars) / 2:
								chars = chars[:-1]
							tmpList2.append(chars)
							
					AIGameHistory.append(tmpList2)
					i += 1
			
				
				# These are the AI data values
				AIData = []

				for j in range(NUM_AIS):
					entry = entries[i]
					if entry != "\n":
						# Need this to convert the string which is loaded from the file to a integer list object
						if entry == "[]":
							AIData.append(getDefaultData()[j])
						else:
							AIData.append([int(x) for x in entry[1:-1].split(",")])
					i += 1
							
				# Now some AI boolean values. These define custom functions
				# When empty load from default
				if version > 0.01:
					for j in range(NUM_AIS):
						entry = entries[i]
						if entry != "\n":
							# Need this to convert the string which is loaded from the file to a integer list object
							if entry == "[]":
								AIData.append(getDefaultData()[j + NUM_AIS])
							else:
								AIData.append([int(x) for x in entry[1:-1].split(",")])	

						i += 1
						
				# Any additions to the default values should be loaded with this function
				for j in range(len(AIData)):
					while len(AIData[j]) < len(getDefaultData()[j]):
						AIData[j].append(getDefaultData()[j][len(AIData[j])])	
				
				AI.setGameHistory(AIGameHistory)
				AI.setAIData(AIData)
				AIList.append(AI)

			f.close()
		except IOError:
			if bCheckBadLoad:
				for i in range(20):
					self.AIList.append(createNewAI())
				
				self.bBadLoad = True

	def saveAIs(self):
		self.writeAIsToFile(saveFolder + filePrefix + 'AIData' + fileSuffix + '.txt', 'w', self.AIList)
		self.writeAIsToFile(saveFolder + filePrefix + 'AIData' + fileSuffix + '.bak', 'w', self.AIList)
		self.writeAIsToFile(saveFolder + 'DeadAIData' + fileSuffix + '.txt', 'a', self.deadAIList)
		self.writeAIsToFile(saveFolder + 'DeadAIData' + fileSuffix + '.bak', 'a', self.deadAIList)
		self.writeAIsToFile(saveFolder + 'TrackedAIData.txt', 'w', self.trackedAIList)
		self.writeAIsToFile(saveFolder + 'TrackedAIData.bak', 'w', self.trackedAIList)
		
		self.deadAIList = []
		
	def writeAIsToFile(self, fileName, writeType, AIList):
		f = open(fileName, writeType)
		
		if writeType == 'w':
			f.write("// " + str(fileVersion) + "\n")
		
		for AI in AIList:
			AIGameHistory = AI.getGameHistory()

			f.write(str(AI.getAIName()) + ";")
			f.write(str(AI.getGeneration()) + ";")
			f.write(str(AI.getFitness()) + ";")
			f.write(str(int(AI.isTracked())) + ";")
			f.write(str(AI.getElite()) + ";")
			f.write(str(AI.getNumGames()) + ";")
			f.write(str(AI.getStatList()) + ";")

			for i in range (len(AIGameHistory)):
				f.write(str(AIGameHistory[i]) + ";")

			AIData = AI.getAIData()

			for i in range(NUM_AIS):
				f.write(str(AIData[i]) + ";") 
			for i in range(NUM_AIS):
				index = NUM_AIS + i
				try:
					f.write(str(AIData[index]) + ";") 
				except IndexError:
					f.write(str(getDefaultData()[index]) + ";") 

			f.write("\n")
			
		f.close()		


	# Kill some AIs which aren't doing well at the intermediate stages
	def attemptAITrim(self, AI):
		if not AI.isTracked() and not AI.getAIName() == "Team Firaxis" and len(self.AIList) > 18:
			averageFitness = 0
			iNumTested = 0
			
			for AI in self.AIList:
				if AI.getNumGames() > 1:
					averageFitness *= iNumTested
					iNumTested += 1
					averageFitness += AI.getFitness()
					averageFitness /= iNumTested
					
			if AI.getNumGames() > float(g_iNumGames) / 2:
				if AI.getFitness() < averageFitness / 2:
					if randint(0,1):
						self.deadAIList.append(AI)
						self.AIList.remove(AI)
			elif AI.getNumGames() > float(g_iNumGames) / 3:
				if AI.getFitness() < averageFitness / 3:
					if not randint(0,2):
						self.deadAIList.append(AI)
						self.AIList.remove(AI)
			elif AI.getNumGames() > float(g_iNumGames) / 4:
				if AI.getFitness() < averageFitness / 4:
					if not randint(0,3):
						self.deadAIList.append(AI)
						self.AIList.remove(AI)
				